//-------------------------------------------------------------------
//                                                                  -
// Module      : XmlAccessUtils                                     -
//                                                                  -
// Description : Provides utility functions                         -
//                                                                  -
//-------------------------------------------------------------------
//                                                                  -
// Copyright Candle Corporation 2001,2003                           -
//                                                                  -
//-------------------------------------------------------------------
//                                                                  -
// Details:  public static method can be used                       - 
//    constructor is private so no instancing can be made           -
//       writeString( KxmMessage message )                          -
//            returns KxmMessage as a string in UTF-8 encoding      -
//       createKxmMessage( String formatName )                      -
//            constructs KxmMessage object based on MDR's format    -
//            as a standalone                                       -
//       createKxmMessage(                                          -
//         String formatName,                                       -
//         KxmMsgCreateOptions options )                            -
//            constructs KxmMessage object based on MDR's format    -
//            and provided options                                  -
//       parseKxmMessage( byte[] bytes )                            -
//            returns parsed KxmMessage based on supplied           -
//            byte array                                            -
//       destroyKxmMessages( java.util.Vector messages )            -
//            calls destroy() method for each Vector's element      -
//            which is KxmMessage instance                          -
//                                                                  -
//                                                                  -
// Invocation Syntax:                                               -
//                                                                  -
// Diagnostics:                                                     -
//                                                                  -
//   WAS RAS trace can be enabled setting Server trace properties   -
//   com.candle.roma.kxo.outbound.sample.*=all=enabled              -
//                                                                  -
//-------------------------------------------------------------------


package com.candle.roma.kxo.outbound.sample;
/*
* Utility class
*/

import com.candle.roma.kxm.*;

public final class XmlAccessUtils
{
  private static final KxmMsgCreateOptions ms_optionsStandalone = 
      new KxmMsgCreateOptions( KxmMsgCreateOptions.STANDALONE );

  private XmlAccessUtils(){}
  /*
  * get KxmMessage's string representation in UTF-8 encoding
  */
  public static String writeString( KxmMessage message )
      throws KxmException, java.io.UnsupportedEncodingException
  {
    return new String( message.write(), "UTF-8" );
  }

  /*
  * constructs and returns KxmMessage for formatName
  * with options STANDALONE
  */
  public static KxmMessage createKxmMessage( String formatName ) throws KxmException
  {
      return new KxmMessage( formatName, ms_optionsStandalone );
  }

  public static KxmMessage createKxmMessage( 
      String formatName,
      KxmMsgCreateOptions options ) throws KxmException
  {
      return new KxmMessage( formatName, options );
  }

  /*
  * constructs and returns KxmMessage for formatName
  * with options STANDALONE
  */
  public static KxmMessage parseKxmMessage( byte[] bytes ) throws KxmException
  {
      return new KxmMessage(bytes, "", ms_optionsStandalone);
  }

  public static void destroyKxmMessages( java.util.Vector messages )
  {
      for( int i = 0; i < messages.size(); ++i )
      {
          try
          {
              KxmMessage msg = (KxmMessage)messages.get(i);
              if( msg != null )
              {
                  msg.destroy();
              }
          }
          catch(KxmException exc){}
      }
      messages.clear();
  }
}