//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003-2004
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoaxhc.cpp,v 1.1 2004/01/22 17:53:24 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoaxhc.cpp,v $
//--------------------------------------------------------------------
// Roma ActiveX API helper classes.
//--------------------------------------------------------------------


#include <atlbase.h>
#include "kxcothm.hpp"
#include "kxcoigdt.hpp"
#include "kxcoiems.hpp"
#include "kxcoaxhc.hpp"


//--------------------------------------------------------------------
// Calculates COM array length.
//--------------------------------------------------------------------
static HRESULT getArrayLength(SAFEARRAY* sa, long& len)
{
    len = 0;

    if (sa == NULL)
    {
        return E_POINTER;
    }

    long ub = 0, lb = 0;

    HRESULT hRes = SafeArrayGetUBound(sa, 1, &ub);

    if (FAILED(hRes))
    {
        return hRes;
    }

    hRes = SafeArrayGetLBound(sa, 1, &lb);

    if (FAILED(hRes))
    {
        return hRes;
    }

    len = ub - lb + 1;
    return S_OK;
}


//====================================================================
// KxcxTheme class
//====================================================================

KxcxTheme::KxcxTheme(long h)
{ 
    *m_impl = (HKXCTHEME)h; 
}


//====================================================================
// KxcxEteInfo class
//====================================================================

HRESULT KxcxEteInfo::load(VARIANT& var)
{
    if (m_impl != NULL)
    {
        m_impl->releaseInstance();
        m_impl = NULL;
    }

    if (var.vt == VT_NULL || var.vt == VT_EMPTY)
    {
        return S_OK;
    }

    if (var.vt != (VT_ARRAY | VT_UI1) || var.parray == NULL)
    {
        return E_UNEXPECTED;
    }

    long al = 0;
    HRESULT hRes = getArrayLength(var.parray, al);

    if (FAILED(hRes))
    {
        return hRes;
    }

    BYTE* pData = NULL;
    hRes = SafeArrayAccessData(var.parray, (void**)&pData);

    if (FAILED(hRes))
    {
        return hRes;
    }

    m_impl = new RomaImplEteInfo(pData, al);
    return SafeArrayUnaccessData(var.parray);
}

HRESULT KxcxEteInfo::save(VARIANT& vout) const
{
    if (m_impl == NULL)
    {
        VariantInit(&vout);
        return S_OK;
    }

    unsigned long length = 0;
    const void* buffer = m_impl->getData(length);

    if (buffer == NULL || !length)
    {
        VariantInit(&vout);
        return S_OK;
    }

    SAFEARRAY* darr = SafeArrayCreateVector(VT_UI1, 0, length);

    if (darr == NULL)
    {
        return E_OUTOFMEMORY;
    }

    BYTE* pData = NULL;
    HRESULT hRes = SafeArrayAccessData(darr, (void**)&pData);

    if (FAILED(hRes))
    {
        SafeArrayDestroy(darr);
        return hRes;
    }

    memcpy(pData, buffer, length);
    SafeArrayUnaccessData(darr);

    vout.vt = VT_ARRAY | VT_UI1;
    vout.parray = darr;
    return S_OK;
}


//====================================================================
// KxcxRoute class
//====================================================================

HRESULT KxcxRoute::load(VARIANT& var)
{
    if (var.vt != (VT_ARRAY | VT_VARIANT) || var.parray == NULL)
    {
        return E_UNEXPECTED;
    }

    VARIANT* pVar = NULL;
    HRESULT hRes = SafeArrayAccessData(var.parray, (void**)&pVar);

    if (FAILED(hRes))
    {
        return hRes;
    }

    USES_CONVERSION;

    // type
    setType((Roma::AppType)V_I4(pVar));
    pVar++;

    // flags
    setFlags(V_I4(pVar));
    pVar++;

    // target queue
    setTargetQueue(W2A(V_BSTR(pVar)));
    pVar++;

    // target element
    setTargetElement(W2A(V_BSTR(pVar)));
    pVar++;

    // target component
    setTargetComponent(W2A(V_BSTR(pVar)));
    pVar++;

    // target code page
    setTargetCodePage(W2A(V_BSTR(pVar)));
    pVar++;

    // target source service
    setSourceService(W2A(V_BSTR(pVar)));
    pVar++;

    // target source flow
    setSourceFlow(W2A(V_BSTR(pVar)));
    pVar++;

    // target source operation
    setSourceOperation(W2A(V_BSTR(pVar)));

    return SafeArrayUnaccessData(var.parray);
}

HRESULT KxcxRoute::save(VARIANT& vout) const
{
    SAFEARRAY* sa = SafeArrayCreateVector(VT_VARIANT, 0, 9);

    if (sa == NULL)
    {
        return E_OUTOFMEMORY;
    }

    VARIANT* pVar = NULL;
    HRESULT res = SafeArrayAccessData(sa, (void**)&pVar);

    if (FAILED(res))
    {
        SafeArrayDestroy(sa);
        return res;
    }

    // type
    V_VT(pVar) = VT_I4;
    V_I4(pVar) = getType();
    pVar++;

    // flags
    V_VT(pVar) = VT_I4;
    V_I4(pVar) = getFlags();
    pVar++;

    // target queue
    const char* str = getTargetQueue();
    V_BSTR(pVar) = A2BSTR(str);
    V_VT(pVar) = VT_BSTR;
    pVar++;

    // target element
    str = getTargetElement();
    V_BSTR(pVar) = A2BSTR(str);
    V_VT(pVar) = VT_BSTR;
    pVar++;

    // target component
    str = getTargetComponent();
    V_BSTR(pVar) = A2BSTR(str);
    V_VT(pVar) = VT_BSTR;
    pVar++;

    // target code page
    str = getTargetCodePage();
    V_BSTR(pVar) = A2BSTR(str);
    V_VT(pVar) = VT_BSTR;
    pVar++;

    // target source service
    str = getSourceService();
    V_BSTR(pVar) = A2BSTR(str);
    V_VT(pVar) = VT_BSTR;
    pVar++;

    // target source flow
    str = getSourceFlow();
    V_BSTR(pVar) = A2BSTR(str);
    V_VT(pVar) = VT_BSTR;
    pVar++;

    // target source operation
    str = getSourceOperation();
    V_BSTR(pVar) = A2BSTR(str);
    V_VT(pVar) = VT_BSTR;

    SafeArrayUnaccessData(sa);
    vout.vt = VT_ARRAY | VT_VARIANT;
    vout.parray = sa;
    return S_OK;
}


//====================================================================
// KxcxTrailer class
//====================================================================

HRESULT KxcxTrailer::load(VARIANT& var)
{
    if (var.vt != (VT_ARRAY | VT_VARIANT) || var.parray == NULL)
    {
        return E_UNEXPECTED;
    }

    VARIANT* pVar = NULL;
    HRESULT hRes = SafeArrayAccessData(var.parray, (void**)&pVar);

    if (FAILED(hRes))
    {
        return hRes;
    }

    // type
    setType((Roma::ILSType)V_I4(pVar));
    pVar++;

    // method
    setMethod(V_I4(pVar));
    pVar++;

    // data
    if (V_VT(pVar) == VT_NULL || V_VT(pVar) == VT_EMPTY)
    {
        setData(NULL, 0);
    }
    else
    {
        long al = 0;
        hRes = getArrayLength(V_ARRAY(pVar), al);

        if (FAILED(hRes))
        {
            SafeArrayUnaccessData(var.parray);
            return hRes;
        }

        BYTE* pData = NULL;
        hRes = SafeArrayAccessData(V_ARRAY(pVar), (void**)&pData);

        if (FAILED(hRes))
        {
            SafeArrayUnaccessData(var.parray);
            return hRes;
        }

        setData(pData, al);
        SafeArrayUnaccessData(V_ARRAY(pVar));
    }

    return SafeArrayUnaccessData(var.parray);
}

HRESULT KxcxTrailer::save(VARIANT& vout) const
{
    SAFEARRAY* sa = SafeArrayCreateVector(VT_VARIANT, 0, 3);

    if (sa == NULL)
    {
        return E_OUTOFMEMORY;
    }

    VARIANT* pVar = NULL;
    HRESULT res = SafeArrayAccessData(sa, (void**)&pVar);

    if (FAILED(res))
    {
        SafeArrayDestroy(sa);
        return res;
    }

    // type
    V_VT(pVar) = VT_I4;
    V_I4(pVar) = getType();
    pVar++;

    // method
    V_VT(pVar) = VT_I4;
    V_I4(pVar) = getMethod();
    pVar++;

    // data
    unsigned long dlen = 0;
    const void* dptr = getData(dlen);

    if (dptr == NULL || !dlen)
    {
        V_VT(pVar) = VT_NULL;
    }
    else
    {
        SAFEARRAY* darr = SafeArrayCreateVector(VT_UI1, 0, dlen);

        if (darr == NULL)
        {
            SafeArrayUnaccessData(sa);
            SafeArrayDestroy(sa);
            return E_OUTOFMEMORY;
        }

        BYTE* pData = NULL;
        res = SafeArrayAccessData(darr, (void**)&pData);

        if (FAILED(res))
        {
            SafeArrayUnaccessData(sa);
            SafeArrayDestroy(sa);
            SafeArrayDestroy(darr);
            return res;
        }

        memcpy(pData, dptr, dlen);
        SafeArrayUnaccessData(darr);

        V_VT(pVar) = VT_ARRAY | VT_UI1;
        V_ARRAY(pVar) = darr;
    }

    SafeArrayUnaccessData(sa);
    vout.vt = VT_ARRAY | VT_VARIANT;
    vout.parray = sa;
    return S_OK;
}


//====================================================================
// KxcxThemeInfo class
//====================================================================

HRESULT KxcxThemeInfo::load(VARIANT& var)
{
    if (var.vt != (VT_ARRAY | VT_VARIANT) || var.parray == NULL)
    {
        return E_UNEXPECTED;
    }

    VARIANT* pVar = NULL;
    HRESULT hRes = SafeArrayAccessData(var.parray, (void**)&pVar);

    if (FAILED(hRes))
    {
        return hRes;
    }

    // workflow
    KxcxWorkflow wf;
    hRes = wf.load(*pVar);

    if (FAILED(hRes))
    {
        SafeArrayUnaccessData(var.parray);
        return hRes;
    }

    setWorkflow(wf);
    pVar++;

    // ete info
    KxcxEteInfo ete;
    hRes = ete.load(*pVar);

    if (FAILED(hRes))
    {
        SafeArrayUnaccessData(var.parray);
        return hRes;
    }

    setEteInfo(ete);
    pVar++;

    // trailers
    KxcxTrailerList tl;
    hRes = tl.load(*pVar);

    if (FAILED(hRes))
    {
        SafeArrayUnaccessData(var.parray);
        return hRes;
    }

    setTrailers(tl);
    pVar++;

    // retry count
    setRetryCount(V_I4(pVar));
    pVar++;

    // confirm flags
    setConfirmFlags((Roma::ConfirmFlags)V_I4(pVar));

    return SafeArrayUnaccessData(var.parray);
}

HRESULT KxcxThemeInfo::save(VARIANT& vout) const
{
    SAFEARRAY* sa = SafeArrayCreateVector(VT_VARIANT, 0, 5);

    if (sa == NULL)
    {
        return E_OUTOFMEMORY;
    }

    VARIANT* pVar = NULL;
    HRESULT hRes = SafeArrayAccessData(sa, (void**)&pVar);

    if (FAILED(hRes))
    {
        SafeArrayDestroy(sa);
        return hRes;
    }

    // workflow
    KxcxWorkflow wf = getWorkflow();
    hRes = wf.save(*pVar);

    if (FAILED(hRes))
    {
        SafeArrayUnaccessData(sa);
        SafeArrayDestroy(sa);
        return hRes;
    }

    ++pVar;

    // ete info
    KxcxEteInfo ete = getEteInfo();
    hRes = ete.save(*pVar);

    if (FAILED(hRes))
    {
        SafeArrayUnaccessData(sa);
        SafeArrayDestroy(sa);
        return hRes;
    }

    ++pVar;

    // trailers
    KxcxTrailerList tl = getTrailers();
    hRes = tl.save(*pVar);

    if (FAILED(hRes))
    {
        SafeArrayUnaccessData(sa);
        SafeArrayDestroy(sa);
        return hRes;
    }

    ++pVar;

    // retry count
    V_I4(pVar) = getRetryCount();
    V_VT(pVar) = VT_I4;
    ++pVar;

    // confirm flags
    V_I4(pVar) = getConfirmFlags();
    V_VT(pVar) = VT_I4;

    SafeArrayUnaccessData(sa);
    vout.vt = VT_ARRAY | VT_VARIANT;
    vout.parray = sa;
    return S_OK;
}


//====================================================================
// KxcxWorkflowService class
//====================================================================

HRESULT KxcxWorkflowService::load(VARIANT& var)
{
    if (var.vt != (VT_ARRAY | VT_VARIANT) || var.parray == NULL)
    {
        return E_UNEXPECTED;
    }

    VARIANT* pVar = NULL;
    HRESULT hRes = SafeArrayAccessData(var.parray, (void**)&pVar);

    if (FAILED(hRes))
    {
        return hRes;
    }

    USES_CONVERSION;

    // current service
    setService(W2A(V_BSTR(pVar)));
    pVar++;

    // current component
    setComponent(W2A(V_BSTR(pVar)));
    pVar++;

    // audit
    setAudit(W2A(V_BSTR(pVar)));
    pVar++;

    // source service
    m_impl->setSourceService(W2A(V_BSTR(pVar)));
    pVar++;

    // source flow
    setSourceFlow(W2A(V_BSTR(pVar)));
    pVar++;

    // source operation
    setSourceOperation(W2A(V_BSTR(pVar)));

    return SafeArrayUnaccessData(var.parray);
}

HRESULT KxcxWorkflowService::save(VARIANT& vout) const
{
    SAFEARRAY* sa = SafeArrayCreateVector(VT_VARIANT, 0, 6);

    if (sa == NULL)
    {
        return E_OUTOFMEMORY;
    }

    VARIANT* pVar = NULL;
    HRESULT hRes = SafeArrayAccessData(sa, (void**)&pVar);

    if (FAILED(hRes))
    {
        SafeArrayDestroy(sa);
        return hRes;
    }

    // current service
    V_BSTR(pVar) = A2BSTR(getService());
    V_VT(pVar) = VT_BSTR;
    pVar++;

    // current component
    V_BSTR(pVar) = A2BSTR(getComponent());
    V_VT(pVar) = VT_BSTR;
    pVar++;

    // audit
    V_BSTR(pVar) = A2BSTR(getAudit());
    V_VT(pVar) = VT_BSTR;
    pVar++;

    // source service
    V_BSTR(pVar) = A2BSTR(getSourceService());
    V_VT(pVar) = VT_BSTR;
    pVar++;

    // source flow
    V_BSTR(pVar) = A2BSTR(getSourceFlow());
    V_VT(pVar) = VT_BSTR;
    pVar++;

    // source operation
    V_BSTR(pVar) = A2BSTR(getSourceOperation());
    V_VT(pVar) = VT_BSTR;

    SafeArrayUnaccessData(sa);
    vout.vt = VT_ARRAY | VT_VARIANT;
    vout.parray = sa;
    return S_OK;
}


//====================================================================
// KxcxWorkflow class
//====================================================================

HRESULT KxcxWorkflow::load(VARIANT& var)
{
    if (var.vt != (VT_ARRAY | VT_VARIANT) || var.parray == NULL)
    {
        return E_UNEXPECTED;
    }

    VARIANT* pVar = NULL;
    HRESULT hRes = SafeArrayAccessData(var.parray, (void**)&pVar);

    if (FAILED(hRes))
    {
        return hRes;
    }

    // workflow service
    KxcxWorkflowService ws;
    hRes = ws.load(*pVar);

    if (FAILED(hRes))
    {
        SafeArrayUnaccessData(var.parray);
        return hRes;
    }

    setWorkflowService(ws);
    ++pVar;

    // workflow routes
    KxcxRouteList rl;
    hRes = rl.load(*pVar);

    if (FAILED(hRes))
    {
        SafeArrayUnaccessData(var.parray);
        return hRes;
    }

    setRoutes(rl);

    return SafeArrayUnaccessData(var.parray);
}

HRESULT KxcxWorkflow::save(VARIANT& vout) const
{
    SAFEARRAY* sa = SafeArrayCreateVector(VT_VARIANT, 0, 2);

    if (sa == NULL)
    {
        return E_OUTOFMEMORY;
    }

    VARIANT* pVar = NULL;
    HRESULT hRes = SafeArrayAccessData(sa, (void**)&pVar);

    if (FAILED(hRes))
    {
        SafeArrayDestroy(sa);
        return hRes;
    }

    // workflow service
    KxcxWorkflowService ws = getWorkflowService();
    hRes = ws.save(*pVar);

    if (FAILED(hRes))
    {
        SafeArrayUnaccessData(sa);
        SafeArrayDestroy(sa);
        return hRes;
    }

    ++pVar;

    // workflow routes
    KxcxRouteList rl = getRoutes();
    hRes = rl.save(*pVar);

    if (FAILED(hRes))
    {
        SafeArrayUnaccessData(sa);
        SafeArrayDestroy(sa);
        return hRes;
    }

    SafeArrayUnaccessData(sa);
    vout.vt = VT_ARRAY | VT_VARIANT;
    vout.parray = sa;
    return S_OK;
}


//====================================================================
// KxcxTrailerList class
//====================================================================

HRESULT KxcxTrailerList::load(VARIANT& var)
{
    clear();

    if (var.vt == VT_EMPTY || var.vt == VT_NULL)
    {
        return S_OK;
    }

    if (var.vt != (VT_ARRAY | VT_VARIANT) || var.parray == NULL)
    {
        return E_UNEXPECTED;
    }

    long al = 0;
    HRESULT hRes = getArrayLength(var.parray, al);

    if (FAILED(hRes))
    {
        return hRes;
    }

    VARIANT* pVar = NULL;
    hRes = SafeArrayAccessData(var.parray, (void**)&pVar);

    if (FAILED(hRes))
    {
        return hRes;
    }

    KxcxTrailer rt;

    while (al-- > 0)
    {
        hRes = rt.load(*pVar);

        if (FAILED(hRes))
        {
            SafeArrayUnaccessData(var.parray);
            return hRes;
        }

        add(rt);
        ++pVar;
    }

    return SafeArrayUnaccessData(var.parray);
}

HRESULT KxcxTrailerList::save(VARIANT& var) const
{
    unsigned num = getCount();

    if (!num)
    {
        VariantInit(&var);
        return S_OK;
    }

    SAFEARRAY* sa = SafeArrayCreateVector(VT_VARIANT, 0, num);

    if (sa == NULL)
    {
        return E_OUTOFMEMORY;
    }

    VARIANT* pVar = NULL;
    HRESULT hRes = SafeArrayAccessData(sa, (void**)&pVar);

    if (FAILED(hRes))
    {
        SafeArrayDestroy(sa);
        return hRes;
    }

    KxcxTrailer rt;
    RomaTrailerListIterator it = getIterator();

    while (it.getNext(rt))
    {
        hRes = rt.save(*pVar);

        if (FAILED(hRes))
        {
            SafeArrayUnaccessData(sa);
            SafeArrayDestroy(sa);
            return hRes;
        }

        ++pVar;
    }

    SafeArrayUnaccessData(sa);
    var.vt = VT_ARRAY | VT_VARIANT;
    var.parray = sa;
    return S_OK;
}

bool KxcxTrailerList::removeHandle(long h)
{
    return m_impl->remove(reinterpret_cast<RomaImplTrailer*>(h));
}


//====================================================================
// KxcxRouteList class
//====================================================================

HRESULT KxcxRouteList::load(VARIANT& var)
{
    clear();

    if (var.vt == VT_EMPTY || var.vt == VT_NULL)
    {
        return S_OK;
    }

    if (var.vt != (VT_ARRAY | VT_VARIANT) || var.parray == NULL)
    {
        return E_UNEXPECTED;
    }

    long al = 0;
    HRESULT hRes = getArrayLength(var.parray, al);

    if (FAILED(hRes))
    {
        return hRes;
    }

    VARIANT* pVar = NULL;
    hRes = SafeArrayAccessData(var.parray, (void**)&pVar);

    if (FAILED(hRes))
    {
        return hRes;
    }

    KxcxRoute rt;

    while (al-- > 0)
    {
        hRes = rt.load(*pVar);

        if (FAILED(hRes))
        {
            SafeArrayUnaccessData(var.parray);
            return hRes;
        }

        add(rt);
        ++pVar;
    }

    return SafeArrayUnaccessData(var.parray);
}

HRESULT KxcxRouteList::save(VARIANT& var) const
{
    unsigned num = getCount();

    if (!num)
    {
        VariantInit(&var);
        return S_OK;
    }

    SAFEARRAY* sa = SafeArrayCreateVector(VT_VARIANT, 0, num);

    if (sa == NULL)
    {
        return E_OUTOFMEMORY;
    }

    VARIANT* pVar = NULL;
    HRESULT hRes = SafeArrayAccessData(sa, (void**)&pVar);

    if (FAILED(hRes))
    {
        SafeArrayDestroy(sa);
        return hRes;
    }

    KxcxRoute rt;
    RomaRouteListIterator it = getIterator();

    while (it.getNext(rt))
    {
        hRes = rt.save(*pVar);

        if (FAILED(hRes))
        {
            SafeArrayUnaccessData(sa);
            SafeArrayDestroy(sa);
            return hRes;
        }

        ++pVar;
    }

    SafeArrayUnaccessData(sa);
    var.vt = VT_ARRAY | VT_VARIANT;
    var.parray = sa;
    return S_OK;
}

bool KxcxRouteList::removeHandle(long h)
{
    return m_impl->remove(reinterpret_cast<RomaImplRoute*>(h));
}


