//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003-2004
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoaxhc.hpp,v 1.1 2004/01/22 17:53:24 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoaxhc.hpp,v $
//--------------------------------------------------------------------
// Roma ActiveX API helper classes.
//--------------------------------------------------------------------

#pragma once

#include <kxcoall.hpp>


template < class T > kxcxLoadData(IDispatch* disp, T& obj)
{
    VARIANT var;
    VariantInit(&var);

    HRESULT hRes = CComDispatchDriver::GetProperty(disp, 99, &var);
    if (FAILED(hRes)) return hRes;
    hRes = obj.load(var);
    VariantClear(&var);
    return hRes;
}


class KXCODLL KxcxTheme : public RomaTheme
{
public:
    KxcxTheme(long);
};


class KXCODLL KxcxEteInfo : public RomaEteInfo
{
public:
    KxcxEteInfo() : RomaEteInfo(NULL) {}
    KxcxEteInfo(const RomaEteInfo& ei) : RomaEteInfo(ei) {}
    KxcxEteInfo& operator=(const RomaEteInfo& ei)
    {
        RomaEteInfo::operator=(ei);
        return *this;
    }

    HRESULT load(VARIANT&);
    HRESULT save(VARIANT&) const;
};


class KXCODLL KxcxRoute : public RomaRoute
{
public:
    KxcxRoute() {}
    KxcxRoute(const RomaRoute& rt) : RomaRoute(rt) {}
    KxcxRoute& operator=(const RomaRoute& rt)
    {
        RomaRoute::operator=(rt);
        return *this;
    }

    HRESULT load(VARIANT&);
    HRESULT save(VARIANT&) const;

    long getHandle() const { return (long)m_impl; }
};


class KXCODLL KxcxTrailer : public RomaTrailer
{
public:
    KxcxTrailer() {}
    KxcxTrailer(const RomaTrailer& tr) : RomaTrailer(tr) {}
    KxcxTrailer& operator=(const RomaTrailer& tr)
    {
        RomaTrailer::operator=(tr);
        return *this;
    }

    HRESULT load(VARIANT&);
    HRESULT save(VARIANT&) const;

    long getHandle() const { return (long)m_impl; }
};


class KXCODLL KxcxThemeInfo : public RomaThemeInfo
{
public:
    KxcxThemeInfo() {}
    KxcxThemeInfo(const RomaThemeInfo& ti) : RomaThemeInfo(ti) {}
    KxcxThemeInfo& operator=(const RomaThemeInfo& ti)
    {
        RomaThemeInfo::operator=(ti);
        return *this;
    }

    HRESULT load(VARIANT&);
    HRESULT save(VARIANT&) const;
};


class KXCODLL KxcxWorkflowService : public RomaWorkflowService
{
public:
    KxcxWorkflowService() {}
    KxcxWorkflowService(const RomaWorkflowService& ws)
        : RomaWorkflowService(ws) {}
    KxcxWorkflowService& operator=(const RomaWorkflowService& ws)
    {
        RomaWorkflowService::operator=(ws);
        return *this;
    }

    HRESULT load(VARIANT&);
    HRESULT save(VARIANT&) const;
};


class KXCODLL KxcxWorkflow : public RomaWorkflow
{
public:
    KxcxWorkflow() {}
    KxcxWorkflow(const RomaWorkflow& wf) : RomaWorkflow(wf) {}
    KxcxWorkflow& operator=(const RomaWorkflow& wf)
    {
        RomaWorkflow::operator=(wf);
        return *this;
    }

    HRESULT load(VARIANT&);
    HRESULT save(VARIANT&) const;
};


class KXCODLL KxcxRouteList : public RomaRouteList
{
public:
    KxcxRouteList() {}
    KxcxRouteList(const RomaRouteList& rl) : RomaRouteList(rl) {}
    KxcxRouteList& operator=(const RomaRouteList& rl)
    {
        RomaRouteList::operator=(rl);
        return *this;
    }

    bool removeHandle(long);

    HRESULT load(VARIANT&);
    HRESULT save(VARIANT&) const;
};


class KXCODLL KxcxTrailerList : public RomaTrailerList
{
public:
    KxcxTrailerList() {}
    KxcxTrailerList(const RomaTrailerList& tl) : RomaTrailerList(tl) {}
    KxcxTrailerList& operator=(const RomaTrailerList& tl)
    {
        RomaTrailerList::operator=(tl);
        return *this;
    }

    bool removeHandle(long);

    HRESULT load(VARIANT&);
    HRESULT save(VARIANT&) const;
};


class KxcxRomaConfig : public RomaConfig
{
public:
    KxcxRomaConfig() {}
    KxcxRomaConfig(const RomaConfig& rc) : RomaConfig(rc) {}
    KxcxRomaConfig& operator=(const RomaConfig& rc)
    {
        RomaConfig::operator=(rc);
        return *this;
    }
};


class KxcxSystemConfig : public RomaSystemConfig
{
public:
    KxcxSystemConfig() {}
    KxcxSystemConfig(const RomaSystemConfig& rc) : RomaSystemConfig(rc) {}
    KxcxSystemConfig& operator=(const RomaSystemConfig& rc)
    {
        RomaSystemConfig::operator=(rc);
        return *this;
    }
};


class KxcxExceptionInfo : public RomaExceptionInfo
{
public:
    KxcxExceptionInfo() {}
    KxcxExceptionInfo(const RomaExceptionInfo& rc) : RomaExceptionInfo(rc) {}
    KxcxExceptionInfo& operator=(const RomaExceptionInfo& rc)
    {
        RomaExceptionInfo::operator=(rc);
        return *this;
    }
};


class KxcxExcErrorData : public RomaExcErrorData
{
public:
    KxcxExcErrorData() {}
    KxcxExcErrorData(const RomaExcErrorData& rc) : RomaExcErrorData(rc) {}
    KxcxExcErrorData& operator=(const RomaExcErrorData& rc)
    {
        RomaExcErrorData::operator=(rc);
        return *this;
    }
};

