//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoema.cpp,v 1.15 2004/01/22 17:53:24 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoema.cpp,v $
// $Revision: 1.15 $
//--------------------------------------------------------------------
// The RomaExceptionMessage, the RomaExceptionOptions and 
// the RomaReportOptions public interfaces.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoem.hpp>
#include <kxcoipxy.hpp>
#include <kxcots.hpp>

TRACE_MODULE_VERSION($Revision: 1.15 $);


//====================================================================
// The RomaExceptionMessage class.
//====================================================================

//--------------------------------------------------------------------
// Defines RTTI for the class.
//--------------------------------------------------------------------
ROMA_DEFINE_TYPEID(RomaExceptionMessage);

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaExceptionMessage::RomaExceptionMessage()
{
    TRACE_ENTRY(RomaExceptionMessage::RomaExceptionMessage,
		<< TRACE(this));

    m_impl = new RomaImplExceptionMessage;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the object with data of the specified message.
//--------------------------------------------------------------------
RomaExceptionMessage::RomaExceptionMessage(const RomaClientRequest& msg)
{
    TRACE_ENTRY(RomaExceptionMessage::RomaExceptionMessage,
        << TRACE(this) << TRACE(msg));

    m_impl = new RomaImplExceptionMessage(RomaImplProxy::getImpl(msg));

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the object with data of the specified message.
//--------------------------------------------------------------------
RomaExceptionMessage::RomaExceptionMessage(const RomaClientResponse& msg)
{
    TRACE_ENTRY(RomaExceptionMessage::RomaExceptionMessage,
        << TRACE(this) << TRACE(msg));

    m_impl = new RomaImplExceptionMessage(RomaImplProxy::getImpl(msg));

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the object with data of the specified message.
//--------------------------------------------------------------------
RomaExceptionMessage::RomaExceptionMessage(const RomaBeRequest& msg)
{
    TRACE_ENTRY(RomaExceptionMessage::RomaExceptionMessage,
        << TRACE(this) << TRACE(msg));

    m_impl = new RomaImplExceptionMessage(RomaImplProxy::getImpl(msg));

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the object with data of the specified message.
//--------------------------------------------------------------------
RomaExceptionMessage::RomaExceptionMessage(const RomaBeResponse& msg)
{
    TRACE_ENTRY(RomaExceptionMessage::RomaExceptionMessage,
        << TRACE(this) << TRACE(msg));

    m_impl = new RomaImplExceptionMessage(RomaImplProxy::getImpl(msg));

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the object with data of the specified message.
//--------------------------------------------------------------------
RomaExceptionMessage::RomaExceptionMessage(const RomaStorable& msg)
{
    TRACE_ENTRY(RomaExceptionMessage::RomaExceptionMessage,
        << TRACE(this) << TRACE(msg));

    m_impl = new RomaImplExceptionMessage(RomaImplProxy::getImpl(msg));

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the object with data of the specified message.
//--------------------------------------------------------------------
RomaExceptionMessage::RomaExceptionMessage(const RomaPublication& msg)
{
    TRACE_ENTRY(RomaExceptionMessage::RomaExceptionMessage,
        << TRACE(this) << TRACE(msg));

    m_impl = new RomaImplExceptionMessage(RomaImplProxy::getImpl(msg));

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Constructor.
// Initializes the object with the specified user data.
//--------------------------------------------------------------------
RomaExceptionMessage::RomaExceptionMessage(const void* buffer, 
                                           int length)
{
    TRACE_ENTRY(RomaExceptionMessage::RomaExceptionMessage,
        << TRACE(this) << TRACE(buffer) << TRACE(length));

    m_impl = new RomaImplExceptionMessage;

    if (buffer && length)
    {
        m_impl->setUserBuffer(buffer, length);
    }

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaExceptionMessage::~RomaExceptionMessage()
{
    TRACE_ENTRY(RomaExceptionMessage::~RomaExceptionMessage,
        << TRACE(this));

    m_impl->releaseInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaExceptionMessage::RomaExceptionMessage(
    const RomaExceptionMessage& msg) : m_impl(msg.m_impl)
{
    TRACE_ENTRY(RomaExceptionMessage::RomaExceptionMessage,
        << TRACE(this) << TRACE(msg));

    m_impl->lockInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaExceptionMessage& 
RomaExceptionMessage::operator = (const RomaExceptionMessage& msg)
{
    TRACE_ENTRY(RomaExceptionMessage::operator =, 
        << TRACE(this) << TRACE(msg));

    if (this != &msg)
    {
        m_impl->releaseInstance();
        m_impl = msg.m_impl;
        m_impl->lockInstance();
    }

    TRACE_RETURN(*this);
}

//--------------------------------------------------------------------
// Set exception elements.
//--------------------------------------------------------------------
void RomaExceptionMessage::setElements(KXCULONG elements)
{
    m_impl->setElements(elements);
}

//--------------------------------------------------------------------
// Set the call type.
//--------------------------------------------------------------------
void RomaExceptionMessage::setCallType(Roma::CallType type)
{
    m_impl->setCallType(type);
}

//--------------------------------------------------------------------
// Set the error status.
//--------------------------------------------------------------------
void RomaExceptionMessage::setStatus(Roma::ErrorStatus status)
{
    m_impl->setStatus(status);
}

//--------------------------------------------------------------------
// Set the completion code.
//--------------------------------------------------------------------
void RomaExceptionMessage::setCompCode(Roma::ErrorSubclass which,
                                       Roma::CompCode code)
{
    m_impl->setCompCode(which, code);
}

//--------------------------------------------------------------------
// Set the return code.
//--------------------------------------------------------------------
void RomaExceptionMessage::setReturnCode(Roma::ErrorSubclass which,
                                         int code)
{
    m_impl->setReturnCode(which, code);
}

//--------------------------------------------------------------------
// Set the error reason.
//--------------------------------------------------------------------
void RomaExceptionMessage::setReason(Roma::ErrorSubclass which, 
                                     int reason)
{
    m_impl->setReason(which, reason);
}

//--------------------------------------------------------------------
// Set the operation type.
//--------------------------------------------------------------------
void RomaExceptionMessage::setOperationType(Roma::ErrorSubclass which, 
                                            int type)
{
    m_impl->setOperationType(which, type);
}

//--------------------------------------------------------------------
// Set the error position.
//--------------------------------------------------------------------
void RomaExceptionMessage::setPosition(Roma::ErrorSubclass which,
                                       int position)
{
    m_impl->setPosition(which, position);
}

//--------------------------------------------------------------------
// Set the error type.
//--------------------------------------------------------------------
void RomaExceptionMessage::setErrorType(Roma::ErrorSubclass which,
                                        Roma::ErrorType type)
{
    m_impl->setErrorType(which, type);
}

//--------------------------------------------------------------------
// Set additional error information.
//--------------------------------------------------------------------
void RomaExceptionMessage::setAdditionalInfo(Roma::ErrorSubclass which,
                                             const char* info)
{
    m_impl->setAdditionalInfo(which, info);
}

//--------------------------------------------------------------------
// Set the user data buffer.
//--------------------------------------------------------------------
bool RomaExceptionMessage::setUserBuffer(const void *buffer, int length)
{
    return m_impl->setUserBuffer(buffer, length);
}

//--------------------------------------------------------------------
// Get the theme.
//--------------------------------------------------------------------
const RomaTheme& RomaExceptionMessage::theme() const
{
    return m_impl->getTheme();
}

//--------------------------------------------------------------------
// Get the Message ID.
//--------------------------------------------------------------------
const RomaMsgId& RomaExceptionMessage::msgId() const
{
    return m_impl->getMsgId();
}

//--------------------------------------------------------------------
// Set the Message ID.
//--------------------------------------------------------------------
void RomaExceptionMessage::setMsgId(const RomaMsgId& msgId)
{
    m_impl->setMsgId(msgId);
}

//--------------------------------------------------------------------
// Get the Correlation ID.
//--------------------------------------------------------------------
const RomaMsgId& RomaExceptionMessage::correlId() const
{
    return m_impl->getCorrelId();
}

//--------------------------------------------------------------------
// Set the Correlation ID.
//--------------------------------------------------------------------
void RomaExceptionMessage::setCorrelId(const RomaMsgId& correlId)
{
    m_impl->setCorrelId(correlId);
}

//--------------------------------------------------------------------
// Set the message priority.
//--------------------------------------------------------------------
void RomaExceptionMessage::setPriority(int priority)
{
    m_impl->setPriority(priority);
}

//--------------------------------------------------------------------
// Set the message persistence.
//--------------------------------------------------------------------
void RomaExceptionMessage::setPersistence(Roma::Persistence persistence)
{
    m_impl->setPersistence(persistence);
}

//--------------------------------------------------------------------
// Set the message format.
//--------------------------------------------------------------------
void RomaExceptionMessage::setFormat(int format)
{
    m_impl->setFormat(format);
}

//--------------------------------------------------------------------
// Set the message feedback.
//--------------------------------------------------------------------
void RomaExceptionMessage::setFeedback(int feedback)
{
    m_impl->setFeedback(feedback);
}

//--------------------------------------------------------------------
// Set the message expiry interval.
//--------------------------------------------------------------------
void RomaExceptionMessage::setExpiry(int expiry)
{
    m_impl->setExpiry(expiry);
}

//--------------------------------------------------------------------
// Set the message codepage.
//--------------------------------------------------------------------
void RomaExceptionMessage::setCodepage(int codepage)
{
    m_impl->setCodepage(codepage);
}

//--------------------------------------------------------------------
// Set the message encoding.
//--------------------------------------------------------------------
void RomaExceptionMessage::setEncoding(int encoding)
{
    m_impl->setEncoding(encoding);
}

//--------------------------------------------------------------------
// Set the message format name.
//--------------------------------------------------------------------
void RomaExceptionMessage::setFormatName(const char* name)
{
    m_impl->setFormatName(name);
}

//--------------------------------------------------------------------
// Set the message name.
//--------------------------------------------------------------------
void RomaExceptionMessage::setMsgName(const char* name)
{
    m_impl->setMsgName(name);
}

//--------------------------------------------------------------------
// Set the message data buffer.
//--------------------------------------------------------------------
bool RomaExceptionMessage::setMsgBuffer(const void* buffer, int length)
{
    return m_impl->setMsgBuffer(buffer, length);
}

//--------------------------------------------------------------------
// Set the tag data buffer.
//--------------------------------------------------------------------
bool RomaExceptionMessage::setTagBuffer(const void* buffer, int length)
{
    return m_impl->setTagBuffer(buffer, length);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaExceptionMessage::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaExceptionMessage, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}


//====================================================================
// The RomaExceptionOptions class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaExceptionOptions::RomaExceptionOptions()
{
    m_impl = new RomaImplExceptionOptions;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaExceptionOptions::~RomaExceptionOptions()
{
    delete m_impl;
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaExceptionOptions::RomaExceptionOptions(
    const RomaExceptionOptions& opt)
{
    m_impl = new RomaImplExceptionOptions(*opt.m_impl);
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaExceptionOptions& 
RomaExceptionOptions::operator = (const RomaExceptionOptions& opt)
{
    if (m_impl != opt.m_impl)
    {
        *m_impl = *opt.m_impl;
    }

    return *this;
}

//--------------------------------------------------------------------
// Set the business service name.
//--------------------------------------------------------------------
void RomaExceptionOptions::setServiceName(const char* name)
{
    m_impl->setServiceName(name);
}

//--------------------------------------------------------------------
// Indicate whether a new theme must be created for a message.
//--------------------------------------------------------------------
void RomaExceptionOptions::setNewTheme(bool f)
{
    m_impl->setNewTheme(f);
}

//--------------------------------------------------------------------
// Indicate whether an exsisting theme must be used.
//--------------------------------------------------------------------
void RomaExceptionOptions::setUseTheme(bool f)
{
    m_impl->setUseTheme(f);
}

//--------------------------------------------------------------------
// Indicate whether the theme must be retained.
//--------------------------------------------------------------------
void RomaExceptionOptions::setRetainTheme(bool f)
{
    m_impl->setRetainTheme(f);
}

//--------------------------------------------------------------------
// Indicate whether the theme must be closed.
//--------------------------------------------------------------------
void RomaExceptionOptions::setCloseTheme(bool f)
{
    m_impl->setCloseTheme(f);
}

//--------------------------------------------------------------------
// Set the theme.
//--------------------------------------------------------------------
void RomaExceptionOptions::setTheme(const RomaTheme& theme)
{
    m_impl->setTheme(theme);
}

//--------------------------------------------------------------------
// Set the put selector.
//--------------------------------------------------------------------
void RomaExceptionOptions::setSelector(Roma::PutSelector selector)
{
    m_impl->setSelector(selector);
}

//--------------------------------------------------------------------
// Indicate whether a message under a transaction can be sent to a
// non-transactional queue.
//--------------------------------------------------------------------
void RomaExceptionOptions::setNoSyncAllowed(bool flag)
{
    m_impl->setNoSyncAllowed(flag);
}

//--------------------------------------------------------------------
// Set the API profile name.
//--------------------------------------------------------------------
void RomaExceptionOptions::setApiProfileName(const char* name)
{
    m_impl->setApiProfile(name);
}

//--------------------------------------------------------------------
// Set the error profile name.
//--------------------------------------------------------------------
void RomaExceptionOptions::setErrorProfileName(const char* name)
{
    m_impl->setErrorProfile(name);
}

//--------------------------------------------------------------------
// Indicate whether an EMS may be invoked on en error.
//--------------------------------------------------------------------
void RomaExceptionOptions::setNoEms(bool flag)
{
    m_impl->setNoEms(flag);
}

//--------------------------------------------------------------------
// Specify true to prohibit notifications.
//--------------------------------------------------------------------
void RomaExceptionOptions::setNoNotification(bool flag)
{
    m_impl->setNoNotification(flag);
}

//--------------------------------------------------------------------
// Specify true to prohibit a message transformation.
//--------------------------------------------------------------------
void RomaExceptionOptions::setNoTransformation(bool flag)
{
    m_impl->setNoTransformation(flag);
}

//--------------------------------------------------------------------
// Set the termination indicator.
//--------------------------------------------------------------------
void RomaExceptionOptions::setTermSupported(bool flag)
{
    m_impl->setTermSupported(flag);
}

//--------------------------------------------------------------------
// Specify true to suppress error profile use.
//--------------------------------------------------------------------
void RomaExceptionOptions::setSuppressErrorProfile(bool flag)
{
    m_impl->setSuppressErrorProfile(flag);
}

//--------------------------------------------------------------------
// Indicates whether Roma should increment the message retry count.
//--------------------------------------------------------------------
void RomaExceptionOptions::setIncrementRetryCount(bool flag)
{
    m_impl->setIncrementRetryCount(flag);
}

//--------------------------------------------------------------------
// Set the security definition.
//--------------------------------------------------------------------
void RomaExceptionOptions::setSyncpointDef(const RomaSyncpointDef& def)
{
    m_impl->setSyncpointDef(def);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaExceptionOptions::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaExceptionOptions, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}


//====================================================================
// The RomaReportOptions class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaReportOptions::RomaReportOptions()
{
    m_impl = new RomaImplReportOptions;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaReportOptions::~RomaReportOptions()
{
    delete m_impl;
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaReportOptions::RomaReportOptions(const RomaReportOptions& opt)
{
    m_impl = new RomaImplReportOptions(*opt.m_impl);
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaReportOptions& 
RomaReportOptions::operator = (const RomaReportOptions& opt)
{
    *m_impl = *opt.m_impl;
    return *this;
}

//--------------------------------------------------------------------
// Set the error profile name.
//--------------------------------------------------------------------
void RomaReportOptions::setErrorProfileName(const char* name)
{
    m_impl->setErrorProfile(name);
}

//--------------------------------------------------------------------
// Set default actions.
//--------------------------------------------------------------------
void RomaReportOptions::setDefaultActions(KXCULONG act)
{
    m_impl->setDefaultActions(act);
}

//--------------------------------------------------------------------
// Indicate whether an EMS may be invoked on en error.
//--------------------------------------------------------------------
void RomaReportOptions::setNoEms(bool f)
{
    m_impl->setNoEms(f);
}

//--------------------------------------------------------------------
// Specify true to prohibit notifications.
//--------------------------------------------------------------------
void RomaReportOptions::setNoNotification(bool f)
{
    m_impl->setNoNotification(f);
}

//--------------------------------------------------------------------
// Specify true to prohibit a message transformation.
//--------------------------------------------------------------------
void RomaReportOptions::setNoTransformation(bool f)
{
    m_impl->setNoTransformation(f);
}

//--------------------------------------------------------------------
// Specify true to prohibit alerts.
//--------------------------------------------------------------------
void RomaReportOptions::setNoAlert(bool f)
{
    m_impl->setNoAlert(f);
}

//--------------------------------------------------------------------
// Set the syncpoint definition.
//--------------------------------------------------------------------
void RomaReportOptions::setSyncpointDef(const RomaSyncpointDef& def)
{
    m_impl->setSyncpointDef(def);
}

//--------------------------------------------------------------------
// Set the theme.
//--------------------------------------------------------------------
void RomaReportOptions::setTheme(const RomaTheme& theme)
{
    m_impl->setTheme(theme);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaReportOptions::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaReportOptions, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

