//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoems.cpp,v 1.3 2004/02/06 18:44:42 e123479 Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoems.cpp,v $
// $Revision: 1.3 $
//--------------------------------------------------------------------
// The EMS API public classes implementation.
//--------------------------------------------------------------------

#include <kxcoems.hpp>
#include <kxcoiems.hpp>
#include <kxcoipxy.hpp>
#include <kxcoilst.hpp>


//====================================================================
// The RomaBusinessFlow class.
//====================================================================

RomaBusinessFlow::RomaBusinessFlow(const char* flow, const char* oper)
{
    m_impl = new RomaImplBusFlow(flow, oper);
}

RomaBusinessFlow::~RomaBusinessFlow()
{
    delete m_impl;
}

RomaBusinessFlow::RomaBusinessFlow(const RomaBusinessFlow& flow)
{
    m_impl = new RomaImplBusFlow(*flow.m_impl);
}

RomaBusinessFlow&
RomaBusinessFlow::operator = (const RomaBusinessFlow& flow)
{
    if (m_impl != flow.m_impl)
    {
        *m_impl = *flow.m_impl;
    }

    return *this;
}

const char* RomaBusinessFlow::getFlowName() const
{
    return m_impl->getFlowName();
}

void RomaBusinessFlow::setFlowName(const char* flow)
{
    m_impl->setFlowName(flow);
}

const char* RomaBusinessFlow::getOperationName() const
{
    return m_impl->getOperationName();
}

void RomaBusinessFlow::setOperationName(const char* oper)
{
    m_impl->setOperationName(oper);
}


//====================================================================
// The RomaBusinessService class.
//====================================================================

RomaBusinessService::RomaBusinessService(const char* name)
{
    m_impl = new RomaImplBusService(name);
}

RomaBusinessService::~RomaBusinessService()
{
    delete m_impl;
}

RomaBusinessService::RomaBusinessService(const RomaBusinessService& serv)
{
    m_impl = new RomaImplBusService(*serv.m_impl);
}

RomaBusinessService&
RomaBusinessService::operator = (const RomaBusinessService& serv)
{
    if (m_impl != serv.m_impl)
    {
        *m_impl = *serv.m_impl;
    }

    return *this;
}

const char* RomaBusinessService::getName() const
{
    return m_impl->getName();
}

void RomaBusinessService::setName(const char* name)
{
    m_impl->setName(name);
}


//====================================================================
// The RomaRoute class.
//====================================================================

RomaRoute::RomaRoute()
{
    m_impl = new RomaImplRoute;
}

RomaRoute::RomaRoute(RomaImplRoute* impl) : m_impl(impl)
{
}

RomaRoute::~RomaRoute()
{
    m_impl->releaseInstance();
}

RomaRoute::RomaRoute(const RomaRoute& rt) : m_impl(rt.m_impl)
{
    m_impl->lockInstance();
}

RomaRoute& RomaRoute::operator=(const RomaRoute& rt)
{
    KXCO_ASSIGN_SHARED(rt);
}

RomaRoute RomaRoute::clone() const
{
    return RomaRoute(new RomaImplRoute(*m_impl));
}

Roma::AppType RomaRoute::getType() const
{
    return (Roma::AppType)m_impl->getType();
}

void RomaRoute::setType(Roma::AppType type)
{
    m_impl->setType(type);
}

unsigned long RomaRoute::getFlags() const
{
    return m_impl->getFlags();
}

void RomaRoute::setFlags(unsigned long flags)
{
    m_impl->setFlags(flags);
}

const char* RomaRoute::getTargetQueue() const
{
    return m_impl->getTargetQueue();
}

void RomaRoute::setTargetQueue(const char* str)
{
    m_impl->setTargetQueue(str);
}

const char* RomaRoute::getTargetElement() const
{
    return m_impl->getTargetElement();
}

void RomaRoute::setTargetElement(const char* str)
{
    m_impl->setTargetElement(str);
}

const char* RomaRoute::getTargetComponent() const
{
    return m_impl->getTargetComponent();
}

void RomaRoute::setTargetComponent(const char* str)
{
    m_impl->setTargetComponent(str);
}

const char* RomaRoute::getTargetCodePage() const
{
    return m_impl->getTargetCodePage();
}

void RomaRoute::setTargetCodePage(const char* str)
{
    m_impl->setTargetCodePage(str);
}

const char* RomaRoute::getSourceService() const
{
    return m_impl->getSourceService();
}

void RomaRoute::setSourceService(const char* str)
{
    m_impl->setSourceService(str);
}

const char* RomaRoute::getSourceFlow() const
{
    return m_impl->getSourceFlow();
}

void RomaRoute::setSourceFlow(const char* str)
{
    m_impl->setSourceFlow(str);
}

const char* RomaRoute::getSourceOperation() const
{
    return m_impl->getSourceOperation();
}

void RomaRoute::setSourceOperation(const char* str)
{
    m_impl->setSourceOperation(str);
}


//====================================================================
// The RomaRouteList class.
//====================================================================

RomaRouteList::RomaRouteList()
{
    m_impl = new RomaImplRouteList;
}

RomaRouteList::~RomaRouteList()
{
    m_impl->releaseInstance();
}

RomaRouteList::RomaRouteList(RomaImplRouteList* impl) : m_impl(impl)
{
}

RomaRouteList::RomaRouteList(const RomaRouteList& list)
    : m_impl(list.m_impl)
{
    m_impl->lockInstance();
}

RomaRouteList& RomaRouteList::operator=(const RomaRouteList& list)
{
    KXCO_ASSIGN_SHARED(list);
}

bool RomaRouteList::isEmpty() const
{
    return m_impl->isEmpty();
}

unsigned RomaRouteList::getCount() const
{
    return m_impl->getCount();
}

void RomaRouteList::clear()
{
    m_impl->clear();
}

bool RomaRouteList::insert(const RomaRoute& route, unsigned i)
{
    return m_impl->insert(RomaImplProxy::getImplPtr(route), i);
}

void RomaRouteList::add(const RomaRoute& route)
{
    m_impl->add(RomaImplProxy::getImplPtr(route));
}

bool RomaRouteList::remove(unsigned i)
{
    return m_impl->remove(i);
}

bool RomaRouteList::remove(const RomaRoute& route)
{
    return m_impl->remove(RomaImplProxy::getImplPtr(route));
}

bool RomaRouteList::getAt(RomaRoute& route, unsigned i) const
{
    RomaImplRoute* ptr = m_impl->getAt(i);

    if (ptr == NULL)
    {
        return false;
    }

    RomaImplProxy::setImpl(route, ptr);
    return true;
}

RomaRouteList RomaRouteList::clone() const
{
    return RomaRouteList(new RomaImplRouteList(*m_impl));
}

RomaRouteListIterator RomaRouteList::getIterator() const
{
    return RomaRouteListIterator(*m_impl);
}


//====================================================================
// The RomaTrailer class.
//====================================================================

RomaTrailer::RomaTrailer()
{
    m_impl = new RomaImplTrailer;
}

RomaTrailer::RomaTrailer(Roma::ILSType type, unsigned long method,
                         const void* data, unsigned long length)
{
    m_impl = new RomaImplTrailer(type, method, data, length);
}

RomaTrailer::RomaTrailer(RomaImplTrailer* impl) : m_impl(impl)
{
}

RomaTrailer::~RomaTrailer()
{
    m_impl->releaseInstance();
}

RomaTrailer::RomaTrailer(const RomaTrailer& tr) : m_impl(tr.m_impl)
{
    m_impl->lockInstance();
}

RomaTrailer& RomaTrailer::operator = (const RomaTrailer& tr)
{
    KXCO_ASSIGN_SHARED(tr);
}

RomaTrailer RomaTrailer::clone() const
{
    return RomaTrailer(new RomaImplTrailer(*m_impl));
}

Roma::ILSType RomaTrailer::getType() const
{
    return (Roma::ILSType)m_impl->getType();
}

void RomaTrailer::setType(Roma::ILSType type)
{
    m_impl->setType(type);
}

unsigned long RomaTrailer::getMethod() const
{
    return m_impl->getMethod();
}

void RomaTrailer::setMethod(unsigned long method)
{
    m_impl->setMethod(method);
}

const void* RomaTrailer::getData(unsigned long& length) const
{
    return m_impl->getData(length);
}

void RomaTrailer::setData(const void* data, unsigned long length)
{
    m_impl->setData(data, length);
}


//====================================================================
// The RomaTrailerList class.
//====================================================================

RomaTrailerList::RomaTrailerList()
{
    m_impl = new RomaImplTrailerList;
}

RomaTrailerList::RomaTrailerList(RomaImplTrailerList* impl) : m_impl(impl)
{
}

RomaTrailerList::RomaTrailerList(const RomaTrailerList& list)
    : m_impl(list.m_impl)
{
    m_impl->lockInstance();
}

RomaTrailerList& RomaTrailerList::operator=(const RomaTrailerList& list)
{
    KXCO_ASSIGN_SHARED(list);
}

RomaTrailerList::~RomaTrailerList()
{
    m_impl->releaseInstance();
}

bool RomaTrailerList::isEmpty() const
{
    return m_impl->isEmpty();
}

unsigned RomaTrailerList::getCount() const
{
    return m_impl->getCount();
}

bool RomaTrailerList::getAt(RomaTrailer& trailer, unsigned i) const
{
    RomaImplTrailer* ptr = m_impl->getAt(i);

    if (ptr == NULL)
    {
        return false;
    }

    RomaImplProxy::setImpl(trailer, ptr);
    return true;
}

void RomaTrailerList::add(const RomaTrailer& trailer)
{
    m_impl->add(RomaImplProxy::getImplPtr(trailer));
}

bool RomaTrailerList::insert(const RomaTrailer& trailer, unsigned i)
{
    return m_impl->insert(RomaImplProxy::getImplPtr(trailer), i);
}

bool RomaTrailerList::remove(const RomaTrailer& trailer)
{
    return m_impl->remove(RomaImplProxy::getImplPtr(trailer));
}

bool RomaTrailerList::remove(unsigned i)
{
    return m_impl->remove(i);
}

void RomaTrailerList::clear()
{
    m_impl->clear();
}

RomaTrailerList RomaTrailerList::clone() const
{
    return RomaTrailerList(new RomaImplTrailerList(*m_impl));
}

RomaTrailerListIterator RomaTrailerList::getIterator() const
{
    return RomaTrailerListIterator(*m_impl);
}


//====================================================================
// The RomaConfig class.
//====================================================================

RomaConfig::RomaConfig()
{
    m_impl = new RomaImplRomaConfig;
}

RomaConfig::RomaConfig(RomaImplRomaConfig* impl) : m_impl(impl)
{
}

RomaConfig::~RomaConfig()
{
    m_impl->releaseInstance();
}

RomaConfig::RomaConfig(const RomaConfig& rc) : m_impl(rc.m_impl)
{
    m_impl->lockInstance();
}

RomaConfig& RomaConfig::operator=(const RomaConfig& rc)
{
    KXCO_ASSIGN_SHARED(rc);
}

Roma::AppType RomaConfig::getElementType() const
{
    return (Roma::AppType)m_impl->getElementType();
}

const char* RomaConfig::getElementName() const
{
    return m_impl->getElementName();
}

const char* RomaConfig::getComponentName() const
{
    return m_impl->getComponentName();
}

const char* RomaConfig::getQueueName() const
{
    return m_impl->getQueueName();
}

const char* RomaConfig::getServiceName() const
{
    return m_impl->getServiceName();
}

const char* RomaConfig::getNodeName() const
{
    return m_impl->getNodeName();
}


//====================================================================
// The RomaSystemConfig class.
//====================================================================

RomaSystemConfig::RomaSystemConfig()
{
    m_impl = new RomaImplSysConfig;
}

RomaSystemConfig::RomaSystemConfig(RomaImplSysConfig* impl) : m_impl(impl)
{
}

RomaSystemConfig::~RomaSystemConfig()
{
    m_impl->releaseInstance();
}

RomaSystemConfig::RomaSystemConfig(const RomaSystemConfig& sc)
    : m_impl(sc.m_impl)
{
    m_impl->lockInstance();
}

RomaSystemConfig& 
RomaSystemConfig::operator = (const RomaSystemConfig& sc)
{
    KXCO_ASSIGN_SHARED(sc);
}

Roma::EnvType RomaSystemConfig::getEnvType() const
{
    return (Roma::EnvType)(m_impl->getEnvType() & 0xFF000000);
}

Roma::EnvOS RomaSystemConfig::getEnvOS() const
{
    return (Roma::EnvOS)m_impl->getEnvType();
}

const char* RomaSystemConfig::getDomainName() const
{
    return m_impl->getDomainName();
}

const char* RomaSystemConfig::getProgramName() const
{
    return m_impl->getProgramName();
}

const char* RomaSystemConfig::getUserId() const
{
    return m_impl->getUserId();
}

unsigned long RomaSystemConfig::getThreadId() const
{
    return m_impl->getThreadId();
}

unsigned long RomaSystemConfig::getProcessId() const
{
    return m_impl->getProcessId();
}


//====================================================================
// The RomaWorkflowService class.
//====================================================================

RomaWorkflowService::RomaWorkflowService()
{
    m_impl = new RomaImplWfService;
}

RomaWorkflowService::RomaWorkflowService(RomaImplWfService* impl)
    : m_impl(impl)
{
}

RomaWorkflowService::~RomaWorkflowService()
{
    m_impl->releaseInstance();
}

RomaWorkflowService::RomaWorkflowService(const RomaWorkflowService& ws)
    : m_impl(ws.m_impl)
{
    m_impl->lockInstance();
}

RomaWorkflowService&
RomaWorkflowService::operator = (const RomaWorkflowService& ws)
{
    KXCO_ASSIGN_SHARED(ws);
}

RomaWorkflowService RomaWorkflowService::clone() const
{
    return RomaWorkflowService(new RomaImplWfService(*m_impl));
}

const char* RomaWorkflowService::getService() const
{
    return m_impl->getService();
}

void RomaWorkflowService::setService(const char* str)
{
    m_impl->setService(str);
}

const char* RomaWorkflowService::getComponent() const
{
    return m_impl->getComponent();
}

void RomaWorkflowService::setComponent(const char* str)
{
    m_impl->setComponent(str);
}

const char* RomaWorkflowService::getAudit() const
{
    return m_impl->getAudit();
}

void RomaWorkflowService::setAudit(const char* str)
{
    m_impl->setAudit(str);
}

const char* RomaWorkflowService::getSourceService() const
{
    return m_impl->getSourceService();
}

const char* RomaWorkflowService::getSourceFlow() const
{
    return m_impl->getSourceFlow();
}

void RomaWorkflowService::setSourceFlow(const char* str)
{
    m_impl->setSourceFlow(str);
}

const char* RomaWorkflowService::getSourceOperation() const
{
    return m_impl->getSourceOperation();
}

void RomaWorkflowService::setSourceOperation(const char* str)
{
    m_impl->setSourceOperation(str);
}


//====================================================================
// The RomaWorkflow class.
//====================================================================

RomaWorkflow::RomaWorkflow()
{
    m_impl = new RomaImplWorkflow;
}

RomaWorkflow::RomaWorkflow(RomaImplWorkflow* impl) : m_impl(impl)
{
}

RomaWorkflow::~RomaWorkflow()
{
    m_impl->releaseInstance();
}

RomaWorkflow::RomaWorkflow(const RomaWorkflow& wf) : m_impl(wf.m_impl)
{
    m_impl->lockInstance();
}

RomaWorkflow& RomaWorkflow::operator=(const RomaWorkflow& wf)
{
    KXCO_ASSIGN_SHARED(wf);
}

RomaWorkflow RomaWorkflow::clone() const
{
    return RomaWorkflow(new RomaImplWorkflow(*m_impl));
}

RomaWorkflowService RomaWorkflow::getWorkflowService() const
{
    return RomaImplProxy::getPublicWithLock(m_impl->getWorkflowService());
}

void RomaWorkflow::setWorkflowService(const RomaWorkflowService& ws)
{
    m_impl->setWorkflowService(ws);
}

RomaRouteList RomaWorkflow::getRoutes() const
{
    return RomaImplProxy::getPublicWithLock(m_impl->getRoutes());
}

void RomaWorkflow::setRoutes(const RomaRouteList& list)
{
    m_impl->setRoutes(list);
}


//====================================================================
// The RomaEteInfo class.
//====================================================================

RomaEteInfo::RomaEteInfo(RomaImplEteInfo* impl) : m_impl(impl)
{
}

RomaEteInfo::~RomaEteInfo()
{
    if (m_impl != NULL) m_impl->releaseInstance();
}

RomaEteInfo::RomaEteInfo(const RomaEteInfo& ete) : m_impl(ete.m_impl)
{
    if (m_impl != NULL) m_impl->lockInstance();
}

RomaEteInfo& RomaEteInfo::operator=(const RomaEteInfo& ete)
{
    if (m_impl != ete.m_impl)
    {
        if (m_impl != NULL) m_impl->releaseInstance();
        m_impl = ete.m_impl;
        if (m_impl != NULL) m_impl->lockInstance();
    }

    return *this;
}


//====================================================================
// The RomaExceptionInfo class.
//====================================================================

RomaExceptionInfo::RomaExceptionInfo()
{
    m_impl = new RomaImplExcInfo;
}

RomaExceptionInfo::RomaExceptionInfo(RomaImplExcInfo* impl) : m_impl(impl)
{
}

RomaExceptionInfo::~RomaExceptionInfo()
{
    m_impl->releaseInstance();
}

RomaExceptionInfo::RomaExceptionInfo(const RomaExceptionInfo& ei)
    : m_impl(ei.m_impl)
{
    m_impl->lockInstance();
}

RomaExceptionInfo&
RomaExceptionInfo::operator = (const RomaExceptionInfo& ei)
{
    KXCO_ASSIGN_SHARED(ei);
}

bool RomaExceptionInfo::hasTimestamp() const
{
    return (m_impl->getTimestamp() != NULL);
}

bool RomaExceptionInfo::hasRomaConfig() const
{
    return (m_impl->getRomaConfig() != NULL);
}

bool RomaExceptionInfo::hasSystemConfig() const
{
    return (m_impl->getSystemConfig() != NULL);
}

bool RomaExceptionInfo::hasErrorData() const
{
    return (m_impl->getErrorData() != NULL);
}

bool RomaExceptionInfo::hasUserData() const
{
    unsigned long length = 0;
    return (m_impl->getUserData(length) != NULL && length);
}

RomaTimestamp RomaExceptionInfo::getTimestamp() const
{
    RomaTimestamp* ts = m_impl->getTimestamp();

    if (ts == NULL)
    {
        return RomaTimestamp();
    }

    return *ts;
}

RomaConfig RomaExceptionInfo::getRomaConfig() const
{
    RomaImplRomaConfig* rc = m_impl->getRomaConfig();

    if (rc == NULL)
    {
        return RomaImplProxy::getPublic(new RomaImplRomaConfig);
    }

    return RomaImplProxy::getPublicWithLock(rc);
}

RomaSystemConfig RomaExceptionInfo::getSystemConfig() const
{
    RomaImplSysConfig* sc = m_impl->getSystemConfig();

    if (sc == NULL)
    {
        return RomaImplProxy::getPublic(new RomaImplSysConfig);
    }

    return RomaImplProxy::getPublicWithLock(sc);
}

RomaExcErrorData RomaExceptionInfo::getErrorData() const
{
    RomaImplExcErrorData* ed = m_impl->getErrorData();

    if (ed == NULL)
    {
        return RomaImplProxy::getPublic(new RomaImplExcErrorData);
    }

    return RomaImplProxy::getPublicWithLock(ed);
}

const void* RomaExceptionInfo::getUserData(unsigned long& length) const
{
    return m_impl->getUserData(length);
}


//====================================================================
// The RomaExcErrorData class.
//====================================================================

RomaExcErrorData::RomaExcErrorData()
{
    m_impl = new RomaImplExcErrorData;
}

RomaExcErrorData::RomaExcErrorData(RomaImplExcErrorData* impl)
    : m_impl(impl)
{
}

RomaExcErrorData::~RomaExcErrorData()
{
    m_impl->releaseInstance();
}

RomaExcErrorData::RomaExcErrorData(const RomaExcErrorData& ed)
    : m_impl(ed.m_impl)
{
    m_impl->lockInstance();
}

RomaExcErrorData& RomaExcErrorData::operator=(const RomaExcErrorData& ed)
{
    KXCO_ASSIGN_SHARED(ed);
}

const RomaError& RomaExcErrorData::getError() const
{
    return m_impl->getError();
}

unsigned long RomaExcErrorData::getCallType() const
{
    return m_impl->getCallType();
}

const char* RomaExcErrorData::getQueueName() const
{
    return m_impl->getQueueName();
}


//====================================================================
// The RomaMessageInfo class.
//====================================================================

RomaMessageInfo::RomaMessageInfo() : m_impl(NULL)
{
}

RomaMessageInfo::RomaMessageInfo(RomaImplMsgInfo* impl) : m_impl(impl)
{
}

RomaMessageInfo::~RomaMessageInfo()
{
    if (m_impl != NULL) m_impl->releaseInstance();
}

RomaMessageInfo::RomaMessageInfo(const RomaMessageInfo& mi)
    : m_impl(mi.m_impl)
{
    if (m_impl != NULL) m_impl->lockInstance();
}

RomaMessageInfo& RomaMessageInfo::operator=(const RomaMessageInfo& mi)
{
    if (m_impl != mi.m_impl)
    {
        if (m_impl != NULL) m_impl->releaseInstance();
        m_impl = mi.m_impl;
        if (m_impl != NULL) m_impl->lockInstance();
    }

    return *this;
}

RomaWorkflow RomaMessageInfo::getWorkflow() const
{
    RomaImplWorkflow* ptr = (m_impl == NULL ? NULL : m_impl->getWorkflow());

    if (ptr == NULL)
    {
        return RomaWorkflow();
    }

    return RomaImplProxy::getPublicWithLock(ptr);
}

RomaTheme RomaMessageInfo::getTheme() const
{
    RomaTheme theme;

    if (m_impl != NULL)
    {
        RomaImplProxy::getImpl(theme) = m_impl->getTheme();
    }

    return theme;
}

RomaExceptionInfo RomaMessageInfo::getException() const
{
    RomaImplExcInfo* ptr = (m_impl == NULL ? NULL : m_impl->getException());

    if (ptr == NULL)
    {
        return RomaImplProxy::getPublic(new RomaImplExcInfo);
    }

    return RomaImplProxy::getPublicWithLock(ptr);
}

RomaBeRequest RomaMessageInfo::getBeRequest() const
{
    RomaImplMsgIncoming* ptr = (m_impl == NULL ? NULL : m_impl->getMessage());

    if (ptr == NULL)
    {
        return RomaImplProxy::getPublicBR(new RomaImplMsgIncoming);
    }

    return RomaImplProxy::getPublicWithLockBR(ptr);
}

RomaClientResponse RomaMessageInfo::getClientResponse() const
{
    RomaImplMsgIncoming* ptr = (m_impl == NULL ? NULL : m_impl->getMessage());

    if (ptr == NULL)
    {
        return RomaImplProxy::getPublicCR(new RomaImplMsgIncoming);
    }

    return RomaImplProxy::getPublicWithLockCR(ptr);
}

bool RomaMessageInfo::hasTheme() const
{
    return (m_impl != NULL && m_impl->getTheme() != NULL_HKXCTHEME);
}

bool RomaMessageInfo::hasMessage() const
{
    return (m_impl != NULL && m_impl->getMessage() != NULL);
}

bool RomaMessageInfo::hasWorkflow() const
{
    return (m_impl != NULL && m_impl->getWorkflow() != NULL);
}

bool RomaMessageInfo::hasException() const
{
    return (m_impl != NULL && m_impl->getException() != NULL);
}


//====================================================================
// The RomaThemeInfo class.
//====================================================================

RomaThemeInfo::RomaThemeInfo() : m_impl(NULL)
{
}

RomaThemeInfo::RomaThemeInfo(RomaImplThemeInfo* impl) : m_impl(impl)
{
}

RomaThemeInfo::~RomaThemeInfo()
{
    if (m_impl != NULL)
    {
        m_impl->releaseInstance();
    }
}

RomaThemeInfo::RomaThemeInfo(const RomaThemeInfo& ti) : m_impl(ti.m_impl)
{
    if (m_impl != NULL)
    {
        m_impl->lockInstance();
    }
}

RomaThemeInfo& RomaThemeInfo::operator=(const RomaThemeInfo& ti)
{
    if (m_impl != ti.m_impl)
    {
        if (m_impl != NULL) m_impl->releaseInstance();
        m_impl = ti.m_impl;
        if (m_impl != NULL) m_impl->lockInstance();
    }

    return *this;
}

RomaThemeInfo RomaThemeInfo::clone() const
{
    if (m_impl == NULL)
    {
        return RomaThemeInfo();
    }

    return RomaThemeInfo(new RomaImplThemeInfo(*m_impl));
}

RomaWorkflow RomaThemeInfo::getWorkflow() const
{
    if (m_impl == NULL)
    {
       ((RomaThemeInfo*)this)->m_impl = new RomaImplThemeInfo;
    }

    return RomaImplProxy::getPublicWithLock(m_impl->getWorkflow());
}

void RomaThemeInfo::setWorkflow(const RomaWorkflow& wf)
{
    if (m_impl == NULL) m_impl = new RomaImplThemeInfo;
    m_impl->setWorkflow(wf);
}

unsigned long RomaThemeInfo::getRetryCount() const
{
    return (m_impl == NULL ? 0 : m_impl->getRetryCount());
}

void RomaThemeInfo::setRetryCount(unsigned long rc)
{
    if (m_impl == NULL) m_impl = new RomaImplThemeInfo;
    m_impl->setRetryCount(rc);
}

RomaMsgId RomaThemeInfo::getThemeId() const
{
    if (m_impl == NULL)
    {
        ((RomaThemeInfo*)this)->m_impl = new RomaImplThemeInfo;
    }

    return RomaImplProxy::getPublicWithLock(m_impl->getThemeId());
}

void RomaThemeInfo::setThemeId(const RomaMsgId& id)
{
    if (m_impl == NULL) m_impl = new RomaImplThemeInfo;
    m_impl->setThemeId(id);
}

Roma::ConfirmFlags RomaThemeInfo::getConfirmFlags() const
{
    return (Roma::ConfirmFlags)(m_impl == NULL ? 0 : m_impl->getConfirmFlags());
}

void RomaThemeInfo::setConfirmFlags(Roma::ConfirmFlags flags)
{
    if (m_impl == NULL) m_impl = new RomaImplThemeInfo;
    m_impl->setConfirmFlags(flags);
}

RomaTrailerList RomaThemeInfo::getTrailers() const
{
    if (m_impl == NULL)
    {
        ((RomaThemeInfo*)this)->m_impl = new RomaImplThemeInfo;
    }

    return RomaImplProxy::getPublicWithLock(m_impl->getTrailers());
}

void RomaThemeInfo::setTrailers(const RomaTrailerList& list)
{
    if (m_impl == NULL) m_impl = new RomaImplThemeInfo;
    m_impl->setTrailers(list);
}

RomaEteInfo RomaThemeInfo::getEteInfo() const
{
    RomaImplEteInfo* ete = (m_impl == NULL ? NULL : m_impl->getEteInfo());

    if (ete != NULL)
    {
        ete->lockInstance();
    }

    return RomaImplProxy::getPublic(ete);
}

void RomaThemeInfo::setEteInfo(const RomaEteInfo& ete)
{
    if (m_impl == NULL) m_impl = new RomaImplThemeInfo;
    m_impl->setEteInfo(ete);
}


//====================================================================
// The RomaTrailerListIterator class.
//====================================================================

RomaTrailerListIterator::RomaTrailerListIterator(
    RomaImplTrailerList& list)
{
    m_impl = new RomaImplTrailerListIter(list);
}

RomaTrailerListIterator::~RomaTrailerListIterator()
{
    delete m_impl;
}

RomaTrailerListIterator::RomaTrailerListIterator(
    const RomaTrailerListIterator& it)
{
    m_impl = new RomaImplTrailerListIter(*it.m_impl);
}

RomaTrailerListIterator& 
RomaTrailerListIterator::operator=(const RomaTrailerListIterator& it)
{
    if (m_impl != it.m_impl)
    {
        *m_impl = *it.m_impl;
    }

    return *this;
}

bool RomaTrailerListIterator::getNext(RomaTrailer& rt) const
{
    RomaImplTrailer* ptr = m_impl->getNext();
    if (ptr == NULL) return false;

    RomaImplProxy::setImpl(rt, ptr);
    return true;
}


//====================================================================
// The RomaRouteListIterator class.
//====================================================================

RomaRouteListIterator::RomaRouteListIterator(RomaImplRouteList& list)
{
    m_impl = new RomaImplRouteListIter(list);
}

RomaRouteListIterator::~RomaRouteListIterator()
{
    delete m_impl;
}

RomaRouteListIterator::RomaRouteListIterator(
    const RomaRouteListIterator& it)
{
    m_impl = new RomaImplRouteListIter(*it.m_impl);
}

RomaRouteListIterator& 
RomaRouteListIterator::operator=(const RomaRouteListIterator& it)
{
    if (m_impl != it.m_impl)
    {
        *m_impl = *it.m_impl;
    }

    return *this;
}

bool RomaRouteListIterator::getNext(RomaRoute& rt) const
{
    RomaImplRoute* ptr = m_impl->getNext();
    if (ptr == NULL) return false;

    RomaImplProxy::setImpl(rt, ptr);
    return true;
}

