//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoerra.cpp,v 1.12 2003/12/11 18:30:05 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoerra.cpp,v $
// $Revision: 1.12 $
//--------------------------------------------------------------------
// The RomaError public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoierr.hpp>
#include <kxcoerr.hpp>
#include <kxcoigdt.hpp>

TRACE_MODULE_VERSION($Revision: 1.12 $);


//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaError::RomaError()
{
    m_impl = new RomaImplError;
}

RomaError::RomaError(RomaImplError* impl) : m_impl(impl)
{
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaError::RomaError(const RomaError& error) : m_impl(error.m_impl)
{
    m_impl->lockInstance();
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaError::~RomaError()
{
    m_impl->releaseInstance();
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaError& RomaError::operator=(const RomaError& error)
{
    KXCO_ASSIGN_SHARED(error);
}

//--------------------------------------------------------------------
// Indicates whether error information is set.
//--------------------------------------------------------------------
bool RomaError::anyError() const
{
    return m_impl->anyError();
}

//--------------------------------------------------------------------
// Get the error status.
//--------------------------------------------------------------------
Roma::ErrorStatus RomaError::status() const
{
    return m_impl->getStatus();
}

//--------------------------------------------------------------------
// Get the completion code.
//--------------------------------------------------------------------
Roma::CompCode RomaError::compCode(Roma::ErrorSubclass which) const
{
    return m_impl->getCompCode(which);
}

//--------------------------------------------------------------------
// Get the return code.
//--------------------------------------------------------------------
int RomaError::returnCode(Roma::ErrorSubclass which) const
{
    return m_impl->getReturnCode(which);
}

//--------------------------------------------------------------------
// Get the error reason.
//--------------------------------------------------------------------
int RomaError::reason(Roma::ErrorSubclass which) const
{
    return m_impl->getReason(which);
}

//--------------------------------------------------------------------
// Get the operation type.
//--------------------------------------------------------------------
int RomaError::operationType(Roma::ErrorSubclass which) const
{
    return m_impl->getOperationType(which);
}

//--------------------------------------------------------------------
// Get the error position.
//--------------------------------------------------------------------
int RomaError::position(Roma::ErrorSubclass which) const
{
    return m_impl->getPosition(which);
}

//--------------------------------------------------------------------
// Get the error type.
//--------------------------------------------------------------------
Roma::ErrorType RomaError::errorType(Roma::ErrorSubclass which) const
{
    return m_impl->getType(which);
}

//--------------------------------------------------------------------
// Get the additional error information.
//--------------------------------------------------------------------
const char* RomaError::additionalInfo(Roma::ErrorSubclass which) const
{
    return m_impl->getAdditionalInfo(which);
}

//--------------------------------------------------------------------
// Get the error description.
//--------------------------------------------------------------------
const char* RomaError::errorText(Roma::ErrorSubclass which,
                                 Roma::ErrorFormat format) const
{
    return m_impl->getText(which, format);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaError::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaError, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

