//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003-2004
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoiems.hpp,v 1.3 2004/02/13 18:42:42 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoiems.hpp,v $
// $Revision: 1.3 $
//--------------------------------------------------------------------
// The internal implementation of Roma EMS objects.
//--------------------------------------------------------------------

#ifndef __KXCOIEMS_HPP__
#define __KXCOIEMS_HPP__

#include "kxcoigdt.hpp"
#include "kxcoall.hpp"
#include "kxcoilst.hpp"


//--------------------------------------------------------------------
// Business Flow implementation class.
//--------------------------------------------------------------------
class RomaImplBusFlow
{
public:
    RomaImplBusFlow(const char* name, const char* oper);
    ~RomaImplBusFlow();

    // Compiler-generated copy and assignment
    // RomaImplBusFlow(const RomaImplBusFlow&);
    // RomaImplBusFlow& operator = (const RomaImplBusFlow&);

    const char* getFlowName() const;
    void setFlowName(const char* flowName);

    const char* getOperationName() const;
    void setOperationName(const char* operationName);

    void populate(KXCBUSFLOW&) const;

private:
    KXCBUSFLOW m_flow;
};


//--------------------------------------------------------------------
// Business Service implementation class.
//--------------------------------------------------------------------
class RomaImplBusService
{
public:
    RomaImplBusService(const char* name);
    ~RomaImplBusService();

    // Compiler-generated copy and assignment
    // RomaImplBusService(const RomaImplBusService&);
    // RomaImplBusService& operator = (const RomaImplBusService&);

    const char* getName() const;
    void setName(const char* name);

    void populate(KXCSERVICE&) const;

private:
    KXCSERVICE m_service;
};


//--------------------------------------------------------------------
// ETE information implementation class.
//--------------------------------------------------------------------
class RomaImplEteInfo : public RomaImplShareable
{
public:
    RomaImplEteInfo();
    RomaImplEteInfo(const void* data, unsigned long length);

    const void* getData(unsigned long& length) const;

private:
    ~RomaImplEteInfo();

    RomaImplEteInfo(const RomaImplEteInfo&);
    RomaImplEteInfo& operator = (const RomaImplEteInfo&);

    char* m_data;
    unsigned long m_length;
};


//--------------------------------------------------------------------
// Roma configuration implementation class.
//--------------------------------------------------------------------
class RomaImplRomaConfig : public RomaImplShareable
{
public:
    RomaImplRomaConfig();
    RomaImplRomaConfig(const KXCEXCROMACONF&);

    unsigned long getElementType() const;
    const char* getElementName() const;
    const char* getComponentName() const;
    const char* getQueueName() const;
    const char* getServiceName() const;
    const char* getNodeName() const;

private:
    ~RomaImplRomaConfig();

    KXCEXCROMACONF m_conf;
};


//--------------------------------------------------------------------
// System configuration implementation class.
//--------------------------------------------------------------------
class RomaImplSysConfig : public RomaImplShareable
{
public:
    RomaImplSysConfig();
    RomaImplSysConfig(const KXCEXCSYSCONF&);

    unsigned long getEnvType() const;
    const char* getDomainName() const;
    const char* getProgramName() const;
    const char* getUserId() const;
    unsigned long getThreadId() const;
    unsigned long getProcessId() const;

private:
    ~RomaImplSysConfig();

    KXCEXCSYSCONF m_conf;
};


//--------------------------------------------------------------------
// Exception error data implementation class.
//--------------------------------------------------------------------
class RomaImplExcErrorData : public RomaImplShareable
{
public:
    RomaImplExcErrorData();
    RomaImplExcErrorData(const KXCEXCERRDATA&);

    const RomaError& getError();
    unsigned long getCallType() const;
    const char* getQueueName() const;

private:
    ~RomaImplExcErrorData();

    RomaError*  m_re;
    KXCULONG    m_call;
    char m_queue[KXCOBJECT_NAME_LENGTH];
};


//--------------------------------------------------------------------
// Exception information implementation class.
//--------------------------------------------------------------------
class RomaImplExcInfo : public RomaImplShareable
{
public:
    RomaImplExcInfo();
    RomaImplExcInfo(const KXCEXCEPTION& ex);

    RomaTimestamp* getTimestamp();
    RomaImplRomaConfig* getRomaConfig();
    RomaImplSysConfig* getSystemConfig();
    RomaImplExcErrorData* getErrorData();
    const void* getUserData(unsigned long& length) const;

    void clear();

private:
    ~RomaImplExcInfo();

    RomaImplExcInfo(const RomaImplExcInfo&);
    RomaImplExcInfo& operator = (const RomaImplExcInfo&);

    RomaTimestamp*          m_ts;
    RomaImplRomaConfig*     m_rc;
    RomaImplSysConfig*      m_sc;
    RomaImplExcErrorData*   m_ed;
    char*                   m_udb;
    unsigned long           m_udl;
};


//--------------------------------------------------------------------
// Workflow route implementation class.
//--------------------------------------------------------------------
class RomaImplRoute : public RomaImplShareable,
                      public RomaImplListItem<RomaImplRoute>
{
public:
    RomaImplRoute();
    RomaImplRoute(const KXCWROUTE&);
    RomaImplRoute(const KXCWROUTE_3&);

    RomaImplRoute(const RomaImplRoute&);
    RomaImplRoute& operator = (const RomaImplRoute&);

    unsigned long getType() const;
    void setType(unsigned long);

    unsigned long getFlags() const;
    void setFlags(unsigned long);

    const char* getTargetQueue() const;
    void setTargetQueue(const char*);

    const char* getTargetElement() const;
    void setTargetElement(const char*);

    const char* getTargetComponent() const;
    void setTargetComponent(const char*);

    const char* getTargetCodePage() const;
    void setTargetCodePage(const char*);

    const char* getSourceService() const;
    void setSourceService(const char*);

    const char* getSourceFlow() const;
    void setSourceFlow(const char*);

    const char* getSourceOperation() const;
    void setSourceOperation(const char*);

    void populate(KXCWROUTE&) const;

private:
    ~RomaImplRoute();

    KXCWROUTE m_route;
};


//--------------------------------------------------------------------
// Persistent trailer implementation class.
//--------------------------------------------------------------------
class RomaImplTrailer : public RomaImplShareable,
                        public RomaImplListItem<RomaImplTrailer> 
{
public:
    RomaImplTrailer();
    RomaImplTrailer(const PKXCTHEMEPTRLDATA);
    RomaImplTrailer(unsigned long type, unsigned long method,
                    const void* data, unsigned long length);

    RomaImplTrailer(const RomaImplTrailer&);
    RomaImplTrailer& operator = (const RomaImplTrailer&);

    unsigned long getType() const;
    void setType(unsigned long);

    unsigned long getMethod() const;
    void setMethod(unsigned long);

    const void* getData(unsigned long& length) const;
    void setData(const void* data, unsigned long length);

private:
    ~RomaImplTrailer();

    unsigned long   m_type;
    unsigned long   m_method;
    unsigned long   m_length;
    char*           m_data;
};


//--------------------------------------------------------------------
// Persistent trailers list implementation class.
//--------------------------------------------------------------------
class RomaImplTrailerList : public RomaImplShareable,
                             public RomaImplList<RomaImplTrailer>
{
public:
    RomaImplTrailerList() {}
    RomaImplTrailerList(const RomaImplTrailerList&);
    RomaImplTrailerList& operator=(const RomaImplTrailerList&);

private:
    ~RomaImplTrailerList() {}
};


//--------------------------------------------------------------------
// Persistent trailers list iterator implementation class.
//--------------------------------------------------------------------
class RomaImplTrailerListIter 
    : public RomaImplListIter<RomaImplTrailer,RomaImplTrailerList>
{
public:
    explicit RomaImplTrailerListIter(RomaImplTrailerList&);
    RomaImplTrailerListIter(const RomaImplTrailerListIter&);
    RomaImplTrailerListIter& operator=(const RomaImplTrailerListIter&);
};


//--------------------------------------------------------------------
// Workflow routes list implementation class.
//--------------------------------------------------------------------
class RomaImplRouteList : public RomaImplShareable,
                          public RomaImplList<RomaImplRoute>
{
public:
    RomaImplRouteList() {}
    RomaImplRouteList(const KXCWFLOW&);

    RomaImplRouteList(const RomaImplRouteList&);
    RomaImplRouteList& operator=(const RomaImplRouteList&);

private:
    ~RomaImplRouteList() {}
};


//--------------------------------------------------------------------
// Workflow routes list iterator implementation class.
//--------------------------------------------------------------------
class RomaImplRouteListIter 
    : public RomaImplListIter<RomaImplRoute,RomaImplRouteList>
{
public:
    explicit RomaImplRouteListIter(RomaImplRouteList&);
    RomaImplRouteListIter(const RomaImplRouteListIter&);
    RomaImplRouteListIter& operator=(const RomaImplRouteListIter&);
};


//--------------------------------------------------------------------
// Workflow Service implementation class.
//--------------------------------------------------------------------
class RomaImplWfService : public RomaImplShareable
{
public:
    RomaImplWfService();
    RomaImplWfService(const KXCWSERVICE&);
    RomaImplWfService(const KXCWSERVICE_2&);

    RomaImplWfService(const RomaImplWfService&);
    RomaImplWfService& operator=(const RomaImplWfService&);

    const char* getService() const;
    void setService(const char*);

    const char* getComponent() const;
    void setComponent(const char*);

    const char* getAudit() const;
    void setAudit(const char*);

    const char* getSourceService() const;
    void setSourceService(const char*);

    const char* getSourceFlow() const;
    void setSourceFlow(const char*);

    const char* getSourceOperation() const;
    void setSourceOperation(const char*);

    void populate(KXCWSERVICE&) const;

private:
    ~RomaImplWfService();

    KXCWSERVICE m_data;
};


//--------------------------------------------------------------------
// Workflow implementation class.
//--------------------------------------------------------------------
class RomaImplWorkflow : public RomaImplShareable
{
public:
    RomaImplWorkflow();
    RomaImplWorkflow(const KXCWFLOW&);

    RomaImplWorkflow(const RomaImplWorkflow&);
    RomaImplWorkflow& operator=(const RomaImplWorkflow&);

    RomaImplWfService* getWorkflowService();
    void setWorkflowService(const RomaWorkflowService&);

    RomaImplRouteList* getRoutes();
    void setRoutes(const RomaRouteList&);

private:
    ~RomaImplWorkflow();

    RomaImplWfService*  m_wserv;
    RomaImplRouteList* m_rlist;
};


//--------------------------------------------------------------------
// Theme information implementation class.
//--------------------------------------------------------------------
class RomaImplThemeInfo : public RomaImplShareable
{
public:
    RomaImplThemeInfo();
    RomaImplThemeInfo(const void* buf, bool client);

    RomaImplThemeInfo(const RomaImplThemeInfo&);
    RomaImplThemeInfo& operator=(const RomaImplThemeInfo&);

    RomaImplWorkflow* getWorkflow();
    void setWorkflow(const RomaWorkflow& wf);

    unsigned long getRetryCount() const;
    void setRetryCount(unsigned long);

    RomaImplMsgId* getThemeId();
    void setThemeId(const RomaMsgId&);

    unsigned long getConfirmFlags() const;
    void setConfirmFlags(unsigned long);

    RomaImplTrailerList* getTrailers();
    void setTrailers(const RomaTrailerList&);

    RomaImplEteInfo* getEteInfo();
    void setEteInfo(const RomaEteInfo&);

private:
    ~RomaImplThemeInfo();

    RomaImplEteInfo*        m_ete;
    RomaImplWorkflow*       m_wflow;
    RomaImplTrailerList*    m_trailers;
    RomaImplMsgId*          m_themeId;
    unsigned long           m_retry;
    unsigned int            m_confirm;
};


//--------------------------------------------------------------------
// Message information implementation class.
//--------------------------------------------------------------------
class RomaImplMsgInfo : public RomaImplShareable
{
public:
    RomaImplMsgInfo();
    RomaImplMsgInfo(const KXCMSGINFO&, RomaImplContext&);

    RomaImplWorkflow* getWorkflow();
    RomaImplExcInfo* getException();
    RomaImplMsgIncoming* getMessage();
    HKXCTHEME getTheme() const;

private:
    ~RomaImplMsgInfo();

    RomaImplMsgInfo(const RomaImplMsgInfo&);
    RomaImplMsgInfo& operator=(const RomaImplMsgInfo&);

    RomaImplWorkflow*       m_wflow;
    RomaImplExcInfo*        m_einfo;
    RomaImplMsgIncoming*    m_msg;
    HKXCTHEME               m_theme;
};


#endif

