//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoimsg.cpp,v 1.22 2004/01/22 17:53:25 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoimsg.cpp,v $
// $Revision: 1.22 $
//--------------------------------------------------------------------
// The internal implementation of various Roma messages.
//--------------------------------------------------------------------

#include "kxcointa.hpp"
#include "kxcoimsg.hpp"
#include "kxcoipxy.hpp"
#include "kxcodefs.hpp"

TRACE_MODULE_VERSION($Revision: 1.22 $);


//====================================================================
// The RomaImplMessage class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplMessage::RomaImplMessage() 
    : m_smart(false), m_msgCopy(false), m_tagCopy(false),
      m_ilsIncoming(NULL_HKXCILS), m_context(NULL),
      m_bufOwner(NULL_HKXC)
{
    TRACE_ENTRY(RomaImplMessage::RomaImplMessage, << TRACE(this));

    kxcSetMsgDefault(&m_kxcmsg);

    // Set the message and correlation IDs to refer to the
    // corresponding members of the KXCMSG structure.
    RomaImplProxy::getImpl(m_msgId).useExternal(m_kxcmsg.MsgId);
    RomaImplProxy::getImpl(m_correlId).useExternal(m_kxcmsg.CorrelId);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaImplMessage::RomaImplMessage(const RomaImplMessage& msg) 
    : m_smart(false), m_msgCopy(true), m_tagCopy(true),
      m_ilsIncoming(msg.m_ilsIncoming), m_context(NULL),
      m_bufOwner(NULL_HKXC)
{
    if (m_ilsIncoming != NULL_HKXCILS && msg.m_context != NULL)
    {
        m_context = msg.m_context;
        m_context->lockInstance();
        m_bufOwner = (HKXC)(*m_context);
    }

    m_kxcmsg = msg.m_kxcmsg;

    // Copy message data
    if (m_kxcmsg.MsgLen)
    {
        m_kxcmsg.MsgBuffer.pBuffer = new char[m_kxcmsg.MsgBuffer.BufLen];
        memcpy(m_kxcmsg.MsgBuffer.pBuffer, msg.m_kxcmsg.MsgBuffer.pBuffer,
            m_kxcmsg.MsgBuffer.BufLen);
    }
    else
    {
        m_kxcmsg.MsgBuffer.pBuffer = NULL;
        m_kxcmsg.MsgBuffer.BufLen = 0;
    }

    // Copy tag data
    if (m_kxcmsg.TagLen)
    {
        m_kxcmsg.TagBuffer.pBuffer = new char[m_kxcmsg.TagBuffer.BufLen];
        memcpy(m_kxcmsg.TagBuffer.pBuffer, msg.m_kxcmsg.TagBuffer.pBuffer,
            m_kxcmsg.TagBuffer.BufLen);
    }
    else
    {
        m_kxcmsg.TagBuffer.pBuffer = NULL;
        m_kxcmsg.TagBuffer.BufLen = 0;
    }

    m_kxcmsg.MsgBuffer.Status = m_kxcmsg.TagBuffer.Status = KXCBUFSTAT_USER;

    // Set the message and correlation IDs to refer to the
    // corresponding members of the KXCMSG structure.
    RomaImplProxy::getImpl(m_msgId).useExternal(m_kxcmsg.MsgId);
    RomaImplProxy::getImpl(m_correlId).useExternal(m_kxcmsg.CorrelId);
}

//--------------------------------------------------------------------
// Constructor.
// Creates the object from the KXCMSG structure. It is used to create
// incoming messages which data is provided by the C API in the KXMMSG
// structure.
//--------------------------------------------------------------------
RomaImplMessage::RomaImplMessage(const KXCMSG& msg,
                                 HKXCILS ilsData,
                                 RomaImplContext* cxt) 
    : m_smart(false), m_msgCopy(false), m_tagCopy(false),
      m_ilsIncoming(ilsData), m_context(cxt), m_bufOwner(NULL_HKXC)
{
    TRACE_ENTRY(RomaImplMessage::RomaImplMessage, << TRACE(this)); 

    if (m_context != NULL)
    {
        m_context->lockInstance();
        m_bufOwner = (HKXC)(*m_context);
    }

    // Save message data.
    m_kxcmsg = msg;

    // Set the message and correlation IDs to refer to the
    // corresponding members of the KXCMSG structure.
    RomaImplProxy::getImpl(m_msgId).useExternal(m_kxcmsg.MsgId);
    RomaImplProxy::getImpl(m_correlId).useExternal(m_kxcmsg.CorrelId);

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaImplMessage::~RomaImplMessage()
{
    TRACE_ENTRY(RomaImplMessage::~RomaImplMessage, << TRACE(this));

    clear();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// This method releases dynamically allocated memory of the message.
//--------------------------------------------------------------------
void RomaImplMessage::clear()
{
    TRACE_ENTRY(RomaImplMessage::clear, << TRACE(this)); 

    if (m_smart)
    {
        // The buffers are allocated by the C++ API, thus we
        // need to free them with 'delete[]' method.
        freeBuffer(m_kxcmsg.MsgBuffer);
        freeBuffer(m_kxcmsg.TagBuffer);
    }
    else
    {
        if (m_msgCopy)
        {
            // The buffer is allocated by the C++ API during message
            // copy, thus we need to free it with 'delete[]' method.
            freeBuffer(m_kxcmsg.MsgBuffer);
        }
        else
        {
            // The buffer is auto-allocated by the C API for an incoming
            // message or provided by an application. We call the C API
            // method which will determine who owns the buffer, an
            // application or the API, and free it if it's owned by the
            // API. An application is responsible to free buffers provided
            // for the API.
            if (m_context != NULL && 
                m_context->isConnected() &&
                m_bufOwner == (HKXC)(*m_context))
            {
                kxcSetMsgBuffer(&m_kxcmsg, 0, NULL);
            }
        }

        if (m_tagCopy)
        {
            // The buffer is allocated by the C++ API during message
            // copy, thus we need to free it with 'delete[]' method.
            freeBuffer(m_kxcmsg.TagBuffer);
        }
        else
        {
            // The buffer is auto-allocated by the C API for an incoming
            // message or provided by an application. We call the C API
            // method which will determine who owns the buffer, an
            // application or the API, and free it if it's owned by the
            // API. An application is responsible to free buffers provided
            // for the API.
            if (m_context != NULL && 
                m_context->isConnected() &&
                m_bufOwner == (HKXC)(*m_context))
            {
                kxcSetTagBuffer(&m_kxcmsg, 0, NULL);
            }
        }
    }

    m_ilsIncoming = NULL_HKXCILS;
    m_bufOwner = NULL_HKXC;

    if (m_context != NULL) 
    {
        m_context->releaseInstance();
        m_context = NULL;
    }

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Release memory of the KXCBUFFER, that was dynamically allocated
// by the API.
//--------------------------------------------------------------------
void RomaImplMessage::freeBuffer(KXCBUFFER& buf)
{
    delete[] (char*)buf.pBuffer;
    buf.pBuffer = NULL;
    buf.BufLen = 0;
    buf.Status = KXCBUFSTAT_USER;
}

//--------------------------------------------------------------------
// Set data buffer.
//--------------------------------------------------------------------
bool RomaImplMessage::setBuffer(KXCBUFFER& target, 
                                const void* buffer, 
                                int length)
{
    TRACE_ENTRY(RomaImplMessage::setBuffer, << TRACE(this)
                                            << TRACE(buffer)
                                            << TRACE(length));

    // Check the data length.
    if (length < 0 || (!buffer && length))
    {
        TRACE_INT_RETURN(false);
    }

    // Free the buffer.
    freeBuffer(target);

    if (!length)
    {
        TRACE_INT_RETURN(true);
    }

    // Allocate a new buffer.
    target.pBuffer = new char[length];

    bool success = (target.pBuffer != NULL);
    
    if (success)
    {
        // Copy buffer data.
        memcpy(target.pBuffer, buffer, length);
        target.BufLen = length;
    }
    else
    {
        // Out of memory.
        TRACE_WARNING(KXCOM_MEMALLOC_FAILED(length));
    }

    TRACE_INT_RETURN(success);
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaImplMessage&
RomaImplMessage::operator = (const RomaImplMessage& msg)
{
    TRACE_ENTRY(RomaImplMessage::operator =, << TRACE(this)); 

    if (this != &msg)
    {
        // Remove existing data.
        clear();

        m_kxcmsg = msg.m_kxcmsg;

        m_kxcmsg.MsgBuffer.pBuffer = m_kxcmsg.TagBuffer.pBuffer = NULL;
        m_kxcmsg.MsgBuffer.BufLen = m_kxcmsg.TagBuffer.BufLen = 0;
        m_kxcmsg.MsgBuffer.Status = m_kxcmsg.TagBuffer.Status = 
            KXCBUFSTAT_USER;

        // Copy message data.
        m_kxcmsg.MsgLen = (setBuffer(m_kxcmsg.MsgBuffer, 
            msg.m_kxcmsg.MsgBuffer.pBuffer, msg.m_kxcmsg.MsgLen) ? 
            msg.m_kxcmsg.MsgLen : 0);

        // Copy tag data.
        m_kxcmsg.TagLen = (setBuffer(m_kxcmsg.TagBuffer, 
            msg.m_kxcmsg.TagBuffer.pBuffer, msg.m_kxcmsg.TagLen) ?
            msg.m_kxcmsg.TagLen : 0);

        // Indicate that we own message and tag data memory.
        m_msgCopy = m_tagCopy = true;

        m_msgId.set(msg.m_msgId);
        m_correlId.set(msg.m_correlId);
        m_ilsIncoming = msg.m_ilsIncoming;

        if (m_ilsIncoming != NULL_HKXCILS && msg.m_context != NULL)
        {
            m_context = msg.m_context;
            m_context->lockInstance();
            m_bufOwner = msg.m_bufOwner;
        }
    }

    TRACE_RETURN(*this);
}

//--------------------------------------------------------------------
// Set a message buffer.
// By design, the API uses buffers specified by an application in
// order to avoid message data copy. An application is responsible for
// a carefull life-time management of the buffers. The ActiveX API
// sets m_smart to true to delegate all message memory management to
// the C++ API.
//--------------------------------------------------------------------
bool RomaImplMessage::setMsgBuffer(const void* buffer, int length)
{
    TRACE_ENTRY(RomaImplMessage::setMsgBuffer, << TRACE(this)
                                               << TRACE(buffer) 
                                               << TRACE(length));

    if (m_smart)
    {
        // Allocate appropriate amount of memory and copy
        // message data into it.
        if (!setBuffer(m_kxcmsg.MsgBuffer, buffer, length))
        {
            // Out of memory.
            TRACE_INT_RETURN(false);
        }
    }
    else
    {
        if (m_msgCopy)
        {
            // We own exising message buffer, thus we need
            // to release it at first.
            freeBuffer(m_kxcmsg.MsgBuffer);
            m_msgCopy = false;
        }

        // Set the specified message buffer pointer and length
        // to the KXCMSG structure.
        if (m_kxcmsg.MsgBuffer.Status == KXCBUFSTAT_KXC &&
            (m_context == NULL || !m_context->isConnected() ||
                m_bufOwner != (HKXC)(*m_context)))
        {
            m_kxcmsg.MsgBuffer.Status = KXCBUFSTAT_USER;
        }

        kxcSetMsgBuffer(&m_kxcmsg, length, (void*)buffer);
    }

    m_kxcmsg.MsgLen = length;

    TRACE_INT_RETURN(true);
}

//--------------------------------------------------------------------
// Set a tag buffer.
// By design, the API uses buffers specified by an application in
// order to avoid message data copy. An application is responsible for
// a carefull life-time management of the buffers. The ActiveX API
// sets m_smart to true to delegate all message memory management to
// the C++ API.
//--------------------------------------------------------------------
bool RomaImplMessage::setTagBuffer(const void* buffer, int length)
{
    TRACE_ENTRY(RomaImplMessage::setTagBuffer, << TRACE(this)
                                               << TRACE(buffer) 
                                               << TRACE(length));

    if (m_smart)
    {
        // Allocate appropriate amount of memory and copy
        // tag data into it.
        if (!setBuffer(m_kxcmsg.TagBuffer, buffer, length))
        {
            // Out of memory.
            TRACE_INT_RETURN(false);
        }
    }
    else
    {
        if (m_tagCopy)
        {
            // We own exising tag buffer, thus we need
            // to release it at first.
            freeBuffer(m_kxcmsg.TagBuffer);
            m_tagCopy = false;
        }

        // Set the specified tag buffer pointer and length
        // to the KXCMSG structure.
        if (m_kxcmsg.TagBuffer.Status == KXCBUFSTAT_KXC &&
            (m_context == NULL || !m_context->isConnected() ||
                m_bufOwner != (HKXC)(*m_context)))
        {
            m_kxcmsg.TagBuffer.Status = KXCBUFSTAT_USER;
        }

        kxcSetTagBuffer(&m_kxcmsg, length, (void*)buffer);
    }

    m_kxcmsg.TagLen = length;

    TRACE_INT_RETURN(true);
}

//--------------------------------------------------------------------
// Get ILS data.
//--------------------------------------------------------------------
RomaILSData* RomaImplMessage::getILSData()
{
    if (m_ilsIncoming == NULL_HKXCILS)
    {
        return NULL;
    }
    
    return RomaImplProxy::createPublic(
        new RomaImplILSData(m_ilsIncoming, m_context, m_bufOwner));
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplMessage::trace(RomaTraceStream& s) const
{
    s << TRACE(m_smart) << TRACE(m_msgCopy) << TRACE(m_tagCopy);
}


//====================================================================
// The RomaImplMsgIncoming class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplMsgIncoming::RomaImplMsgIncoming()
{
}

//--------------------------------------------------------------------
// Constructor.
// Creates the object from the KXCMSG structure. It is used to create
// incoming messages which data is provided by the C API in the KXMMSG
// structure.
//--------------------------------------------------------------------
RomaImplMsgIncoming::RomaImplMsgIncoming(const KXCMSG& msg,
                                         HKXCTHEME theme,
                                         HKXCILS ilsData,
                                         RomaImplContext& cxt)
    : RomaImplMessage(msg, ilsData, &cxt)
{
    RomaImplProxy::getImpl(m_theme) = theme;
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaImplMsgIncoming& 
RomaImplMsgIncoming::operator = (const RomaImplMsgIncoming& msg)
{
    RomaImplMessage::operator = (msg);
    m_theme = msg.m_theme;
    return *this;
}

//--------------------------------------------------------------------
// Set the theme.
//--------------------------------------------------------------------
void RomaImplMsgIncoming::setTheme(HKXCTHEME theme)
{
    RomaImplProxy::getImpl(m_theme) = theme;
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplMsgIncoming::trace(RomaTraceStream& s) const
{
    RomaImplMessage::trace(s);
    s << TRACE(m_theme); 
}


//====================================================================
// RomaImplMsgOutgoing class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplMsgOutgoing::RomaImplMsgOutgoing()
{
}

void RomaImplMsgOutgoing::setIncomingILSData(HKXCILS data,
                                             RomaImplContext* cxt)
{
    m_ilsIncoming = data;

    if (data != NULL_HKXCILS && m_context == NULL && cxt != NULL)
    {
        m_context = cxt;
        m_context->lockInstance();
    }
}

//--------------------------------------------------------------------
// Set the security definition.
//--------------------------------------------------------------------
void RomaImplMsgOutgoing::setSecurityDef(const RomaSecurityDef& def)
{
    RomaImplProxy::getImpl(def).populate(m_security);
}

//--------------------------------------------------------------------
// Fill the C API structures with message data.
//--------------------------------------------------------------------
void RomaImplMsgOutgoing::populate(KXCPUTOPT& po, 
                                   KXCSYNC& sync, 
                                   HKXCTHEME& theme) const
{
    RomaImplCommonPutOptions::populate(po, sync, theme);
    po.pSecurity = (PKXCSECDEF)RomaImplProxy::getImpl(m_security);
    RomaImplProxy::getImpl(m_msgId).copy(po.MsgId);
    RomaImplProxy::getImpl(m_correlId).copy(po.CorrelId);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplMsgOutgoing::trace(RomaTraceStream& s) const
{
    RomaImplMessage::trace(s);
    RomaImplCommonPutOptions::trace(s);
    s << TRACE(m_security);
}


//====================================================================
// The RomaImplClientRequest class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplClientRequest::RomaImplClientRequest()
{
    *m_service = '\0';
}

//--------------------------------------------------------------------
// Set the business service name.
//--------------------------------------------------------------------
void RomaImplClientRequest::setServiceName(const char* name)
{
    TRACE_ENTRY(RomaImplClientRequest::setServiceName, << TRACE(this)
                                                       << TRACE(name));

    RomaUtil::setString(m_service, name, sizeof(m_service));

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplClientRequest::trace(RomaTraceStream& s) const
{
    RomaImplMsgOutgoing::trace(s);
    s << TRACE(m_service);
}


//====================================================================
// RomaImplElementResponse class.
//====================================================================

//--------------------------------------------------------------------
// Constructor.
// Initializes the object with the specified theme.
//--------------------------------------------------------------------
RomaImplElementResponse::RomaImplElementResponse(const RomaTheme& theme)
{
    m_theme = theme;
}


//====================================================================
// The RomaImplStorable class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplStorable::RomaImplStorable()
{
}

//--------------------------------------------------------------------
// Constructor.
// Creates the object from the KXCMSG structure.
//--------------------------------------------------------------------
RomaImplStorable::RomaImplStorable(const KXCMSG& msg, 
                                   HKXCTHEME theme,
                                   RomaImplContext& cxt)
    : RomaImplMsgIncoming(msg, theme, NULL_HKXCTHEME, cxt)
{
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaImplStorable::RomaImplStorable(const RomaImplStorable& msg)
{
    TRACE_ENTRY(RomaImplStorable::RomaImplStorable, << TRACE(this));

    *this = msg;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaImplStorable& 
RomaImplStorable::operator = (const RomaImplStorable& msg)
{
    TRACE_ENTRY(RomaImplStorable::operator =, << TRACE(this));

    if (this != &msg)
    {
        RomaImplMsgIncoming::operator = (msg);
    }

    TRACE_RETURN(*this);
}

//--------------------------------------------------------------------
// Fill the C API structures with message data.
//--------------------------------------------------------------------
void RomaImplStorable::populate(KXCPUTOPT& po, HKXCTHEME& theme) const
{
    RomaImplProxy::getImpl(m_msgId).copy(po.MsgId);
    RomaImplProxy::getImpl(m_correlId).copy(po.CorrelId);
    theme = (HKXCTHEME)m_theme;
}


//====================================================================
// The RomaImplExceptionMessage class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplExceptionMessage::RomaImplExceptionMessage() : m_elements(0)
{
    kxcSetExceptionDefault(&m_data);
}

//--------------------------------------------------------------------
// Constructor.
// Creates the object from the KXCMSG structure.
//--------------------------------------------------------------------
RomaImplExceptionMessage::RomaImplExceptionMessage(
    const RomaImplMessage& msg)
    : RomaImplMessage(msg), m_elements(0)
{
    kxcSetExceptionDefault(&m_data);
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaImplExceptionMessage::~RomaImplExceptionMessage()
{
    if (m_smart)
    {
        freeBuffer(m_data.UserData.UserBuf);
    }
}

//--------------------------------------------------------------------
// This method releases dynamically allocated memory of the message.
//--------------------------------------------------------------------
void RomaImplExceptionMessage::clear()
{
    TRACE_ENTRY(RomaImplExceptionMessage::clear, << TRACE(this));

    RomaImplMessage::clear();

    if (m_smart)
    {
        freeBuffer(m_data.UserData.UserBuf);
    }

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Set a call type.
//--------------------------------------------------------------------
void RomaImplExceptionMessage::setCallType(Roma::CallType type)
{
    m_data.ErrorData.CallType = type;
}

//--------------------------------------------------------------------
// Set a status.
//--------------------------------------------------------------------
void RomaImplExceptionMessage::setStatus(Roma::ErrorStatus status)
{
    m_data.ErrorData.Error.Status = status;
}

//--------------------------------------------------------------------
// Set a completion code.
//--------------------------------------------------------------------
void RomaImplExceptionMessage::setCompCode(Roma::ErrorSubclass which,
                                           Roma::CompCode code)
{
    if (which == Roma::SUBCLASS_SECONDARY)
    {
        m_data.ErrorData.Error.SubResult.LastCompCode = code;
    }
    else
    {
        m_data.ErrorData.Error.Result.LastCompCode = code;
    }
}

//--------------------------------------------------------------------
// Set a return code.
//--------------------------------------------------------------------
void RomaImplExceptionMessage::setReturnCode(Roma::ErrorSubclass which,
                                             int code)
{
    if (which == Roma::SUBCLASS_SECONDARY)
    {
        m_data.ErrorData.Error.SubResult.LastReturnCode = code;
    }
    else
    {
        m_data.ErrorData.Error.Result.LastReturnCode = code;
    }
}

//--------------------------------------------------------------------
// Set an error reason.
//--------------------------------------------------------------------
void RomaImplExceptionMessage::setReason(Roma::ErrorSubclass which,
                                         int reason)
{
    if (which == Roma::SUBCLASS_SECONDARY)
    {
        m_data.ErrorData.Error.SubResult.LastReason = reason;
    }
    else
    {
        m_data.ErrorData.Error.Result.LastReason = reason;
    }
}

//--------------------------------------------------------------------
// Set an operation type.
//--------------------------------------------------------------------
void RomaImplExceptionMessage::setOperationType(Roma::ErrorSubclass which,
                                                int type)
{
    if (which == Roma::SUBCLASS_SECONDARY)
    {
        m_data.ErrorData.Error.SubResult.LastOpType = type;
    }
    else
    {
        m_data.ErrorData.Error.Result.LastOpType = type;
    }
}

//--------------------------------------------------------------------
// Set an error position.
//--------------------------------------------------------------------
void RomaImplExceptionMessage::setPosition(Roma::ErrorSubclass which,
                                           int position)
{
    if (which == Roma::SUBCLASS_SECONDARY)
    {
        m_data.ErrorData.Error.SubResult.LastPosition = position;
    }
    else
    {
        m_data.ErrorData.Error.Result.LastPosition = position;
    }
}

//--------------------------------------------------------------------
// Set an error type.
//--------------------------------------------------------------------
void RomaImplExceptionMessage::setErrorType(Roma::ErrorSubclass which,
                                            Roma::ErrorType type)
{
    if (which == Roma::SUBCLASS_SECONDARY)
    {
        m_data.ErrorData.Error.SubResult.LastErrorType = type;
    }
    else
    {
        m_data.ErrorData.Error.Result.LastErrorType = type;
    }
}

//--------------------------------------------------------------------
// Set a detailed error description.
//--------------------------------------------------------------------
void RomaImplExceptionMessage::setAdditionalInfo(Roma::ErrorSubclass which,
                                                 const char* info)
{
    char* str = (char*)(which == Roma::SUBCLASS_SECONDARY ?
                        m_data.ErrorData.Error.SubResult.LastLocation :
                        m_data.ErrorData.Error.Result.LastLocation);

    if (info)
    {
        strncpy(str, info, KXCFORMAT_BUFFER_LEN - 1);
        str[KXCFORMAT_BUFFER_LEN - 1] = '\0';
    }
    else
    {
        *str = '\0';
    }
}

//--------------------------------------------------------------------
// Set user data.
//--------------------------------------------------------------------
bool RomaImplExceptionMessage::setUserBuffer(const void* buffer,
                                             int length)
{
    TRACE_ENTRY(RomaImplExceptionMessage::setUserBuffer, 
        << TRACE(this) << TRACE(buffer) << TRACE(length));

    bool res = true;

    if (m_smart)
    {
        res = setBuffer(m_data.UserData.UserBuf, buffer, length);
        m_data.UserData.UserBufLen = res ? length : 0;
    }
    else
    {
        m_data.UserData.UserBufLen = 
            m_data.UserData.UserBuf.BufLen = length;
        m_data.UserData.UserBuf.pBuffer = (void*)buffer;
    }

    TRACE_INT_RETURN(res);
}

//--------------------------------------------------------------------
// Save the theme handle on the message.
//--------------------------------------------------------------------
void RomaImplExceptionMessage::capture(HKXCTHEME theme)
{
    RomaImplProxy::getImpl(m_theme) = theme;
}

//--------------------------------------------------------------------
// Fill the KXCEXCEPTION structure with exception data.
//--------------------------------------------------------------------
void RomaImplExceptionMessage::populate(KXCEXCEPTION& ex) const
{
    static KXCULONG elemToExc[] =
    {
        Roma::ELEMENT_TIMESTAMP, KXCEXC_TIMESTAMP,
        Roma::ELEMENT_CONFIG,    KXCEXC_ROMADATA,
        Roma::ELEMENT_SYSCONFIG, KXCEXC_SYSTEMDATA,
        Roma::ELEMENT_ERROR,     KXCEXC_ERRORDATA,
        Roma::ELEMENT_USERDATA,  KXCEXC_USERDATA
    };

    ex = m_data;
    ex.ExceptionFlags = 0;

    size_t i = 0;
    for (; i < sizeof(elemToExc) / sizeof(*elemToExc); ++i)
    {
        if ((elemToExc[i++] & m_elements) != 0)
        {
            ex.ExceptionFlags |= elemToExc[i];
        }
    }
}

//--------------------------------------------------------------------
// Fill the KXCPUTOPT structure with message data.
//--------------------------------------------------------------------
void RomaImplExceptionMessage::populate(KXCPUTOPT& po) const
{
    if (po.RequestOptions != KXCPOPT_USEDEFAULT)
    {
        static KXCULONG elemToPut[] =
        {
            Roma::ELEMENT_TIMESTAMP, KXCPOPT_EXCTIMESTAMP,
            Roma::ELEMENT_CONFIG,    KXCPOPT_EXCROMADATA,
            Roma::ELEMENT_SYSCONFIG, KXCPOPT_EXCSYSDATA,
            Roma::ELEMENT_ERROR,     KXCPOPT_EXCERRDATA,
            Roma::ELEMENT_USERDATA,  KXCPOPT_EXCUSERDATA
        };

        size_t i = 0;
        for (; i < sizeof(elemToPut) / sizeof(*elemToPut); ++i)
        {
            if ((elemToPut[i++] & m_elements) != 0)
            {
                po.RequestOptions |= elemToPut[i];
            }
        }
    }

    RomaImplProxy::getImpl(m_msgId).copy(po.MsgId);
    RomaImplProxy::getImpl(m_correlId).copy(po.CorrelId);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplExceptionMessage::trace(RomaTraceStream& s) const
{
    RomaImplMessage::trace(s);
    s << TRACE_HEX(m_elements);
}


//====================================================================
// The RomaImplPublication class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplPublication::RomaImplPublication()
    : m_retain(false), m_subject(NULL)
{
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplPublication::trace(RomaTraceStream& s) const
{
    RomaImplClientRequest::trace(s);
    s << TRACE(m_retain) << TRACE(m_subject);
}


//====================================================================
// The RomaImplSubscription class.
//====================================================================

//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaImplSubscription::RomaImplSubscription()
    : m_subject(NULL), m_newPubsOnly(false), m_onRequest(false)
{
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaImplSubscription::trace(RomaTraceStream& s) const
{
    RomaImplClientRequest::trace(s);
    s << TRACE(m_newPubsOnly) << TRACE(m_onRequest) << TRACE(m_subject);
}

