//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : kxcointa.hpp
//
// Description  : KXC C++ internal header top-level
//
//--------------------------------------------------------------------
// $Id: kxcointa.hpp,v 1.9 2003/02/21 22:48:05 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcointa.hpp,v $
// $Revision: 1.9 $
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Protect against multiple includes
//--------------------------------------------------------------------
#ifndef __KXCOINTA_INCLUDED
#define __KXCOINTA_INCLUDED

#ifndef KXCO_BUILD
#define KXCO_BUILD "CASP v2.0 XC320 OM " __DATE__
#endif

static char Copyright[]= "Copyright Candle Corporation 1998-2002";

//--------------------------------------------------------------------
// Include external dependencies, including Roma KXC.
//--------------------------------------------------------------------
#include <kxco.hpp>

//--------------------------------------------------------------------
// Include the platform specific definitions.
//--------------------------------------------------------------------
#if defined(KXC_WINNT)
#include <kxcopltn.hpp>
#elif defined(KXC_WIN95)
#include <kxcopltw.hpp>
#elif defined(KXC_AIX)
#include <kxcopltx.hpp>
#elif defined(KXC_SL2)
#include <kxcoplts.hpp>
#elif defined(KXC_HPUX)
#include <kxcoplth.hpp>
#elif defined(KXC_DUX)
#include <kxcopltd.hpp>
#elif defined(KXC_MVS)
#include <kxcopltm.hpp>
#else
#error Specify a platform
#endif

#if defined(KXC_DUX)
#include <kxcopltd.hpp>
#endif

//--------------------------------------------------------------------
// Enable C++ support
//--------------------------------------------------------------------
#if defined(__cplusplus)
  extern "C" {
#endif

//--------------------------------------------------------------------
// Include Roma public macro header file
//--------------------------------------------------------------------
#include <kxcmac.h>

//--------------------------------------------------------------------
// Include KBB if required
//--------------------------------------------------------------------
#if defined(KXC_I_SUPPORT_KBB) || defined(KXC_I_SUPPORT_RAS)
  #include <bss1.h>
  #include <ras1.h>
  #if (!defined(KXC_WINNT) | defined(WIN32_LEAN_AND_MEAN)) & \
      !defined(KXC_I_SUPPORT_RAS)
    #include <ppfm.h>
  #endif
  #include <kxctrace.h>
#endif

//--------------------------------------------------------------------
// Enable C++ support
//--------------------------------------------------------------------
#if defined(__cplusplus)
  }
#endif

#include <kxcotrc.hpp>

#define KXCO_SET_CPP_ERROR( retCode ) \
    setCppError(  \
        retCode,  \
        KXCPOS_0, \
        KXCETYPE_ERROR, \
        "", \
        __LINE__, \
        __FILE__ )


// copyZString always called internally, with size set to sizeof(dest)
inline void
copyZString(void* dest, const void* src, size_t size)
{
    src  = src ? src : "";
    size = (size < 1) ? 1 : size;

    strncpy( (char*)dest, (const char*)src, size-1 );
    ((char*)dest)[size-1] = 0;
}

#if defined(KXCO_DEFINE_PROHIBITED_OPS)

#define KXCO_ASSERT_PROHIBITED_OPS( Class ) \
    Class::Class( const Class& ) \
    { \
      const int Class##_prohibited_copy_constructor_called = 0; \
      assert( Class##_prohibited_copy_constructor_called ); \
    } \
    Class& Class::operator = ( const Class& ) \
    { \
      const int Class##_prohibited_assignment_operator_called = 0; \
      assert( Class##_prohibited_assignment_operator_called ); \
    }

#else

#define KXCO_ASSERT_PROHIBITED_OPS( Class )

#endif


//--------------------------------------------------------------------
// End of module kxcointa.h
//--------------------------------------------------------------------
#endif

