//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2001-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcopbl.cpp,v 1.10 2003/02/21 22:48:06 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcopbl.cpp,v $
// $Revision: 1.10 $
//--------------------------------------------------------------------
// The RomaPublication public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoimsg.hpp>
#include <kxcopbl.hpp>

TRACE_MODULE_VERSION($Revision: 1.10 $);


//--------------------------------------------------------------------
// Define RTTI information
//--------------------------------------------------------------------
ROMA_DEFINE_TYPEID(RomaPublication);


//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaPublication::RomaPublication()
{
    TRACE_ENTRY(RomaPublication::RomaPublication, << TRACE(this));

    m_impl = new RomaImplPublication;

    TRACE_VOID_RETURN;
};

//--------------------------------------------------------------------
// Constructor initializing the object with the business service name
// and the subject.
//--------------------------------------------------------------------
RomaPublication::RomaPublication(const char* serviceName,
                                 const RomaSubject& subject)
{
    TRACE_ENTRY(RomaPublication::RomaPublication, << TRACE(this));

    m_impl = new RomaImplPublication;
    m_impl->setServiceName(serviceName);
    m_impl->setSubject(subject);

    TRACE_VOID_RETURN;
};

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaPublication::~RomaPublication() 
{
    TRACE_ENTRY(RomaPublication::~RomaPublication, << TRACE(this));

    m_impl->releaseInstance();

    TRACE_VOID_RETURN;
};

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaPublication::RomaPublication(const RomaPublication& publ)
    : m_impl(publ.m_impl)
{
    TRACE_ENTRY(RomaPublication::RomaPublication, << TRACE(this)
                                                  << TRACE(publ));
    m_impl->lockInstance();

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaPublication& 
RomaPublication::operator = (const RomaPublication& publ)
{
    TRACE_ENTRY(RomaPublication::operator =, << TRACE(this)
                                             << TRACE(publ));

    if (this != &publ)
    {
        m_impl->releaseInstance();
        m_impl = publ.m_impl;
        m_impl->lockInstance();
    }

    TRACE_RETURN(*this);
}

//--------------------------------------------------------------------
// Specify true if the publication must be retained.
// The default is false.
//--------------------------------------------------------------------
void RomaPublication::setRetain(bool flag)
{
    m_impl->setRetain(flag);
}

//--------------------------------------------------------------------
// Set a business service name.
//--------------------------------------------------------------------
void RomaPublication::setServiceName(const char* name)
{
    m_impl->setServiceName(name);
}

//--------------------------------------------------------------------
// Set theme handling options.
//--------------------------------------------------------------------
void RomaPublication::setNewTheme(bool f)
{
    m_impl->setNewTheme(f);
}

void RomaPublication::setUseTheme(bool f)
{
    m_impl->setUseTheme(f);
}

void RomaPublication::setRetainTheme(bool f)
{
    m_impl->setRetainTheme(f);
}

void RomaPublication::setCloseTheme(bool f)
{
    m_impl->setCloseTheme(f);
}

//--------------------------------------------------------------------
// Specify true to send the message as a datagram.
//--------------------------------------------------------------------
void RomaPublication::setDatagram(bool f)
{
    m_impl->setDatagram(f);
}

//--------------------------------------------------------------------
// Specify true to set "Confirmation of delivery" flag.
//--------------------------------------------------------------------
void RomaPublication::setCOD(bool f)
{
    m_impl->setCOD(f);
}

//--------------------------------------------------------------------
// Specify true to set "Confirmation of arrival" flag.
//--------------------------------------------------------------------
void RomaPublication::setCOA(bool f)
{
    m_impl->setCOA(f);
}

//--------------------------------------------------------------------
// Set a security definition.
//--------------------------------------------------------------------
void RomaPublication::setSecurityDef(const RomaSecurityDef& def)
{
    m_impl->setSecurityDef(def);
}

//--------------------------------------------------------------------
// This method may be overriden by an application to marshal message
// data.
//--------------------------------------------------------------------
bool RomaPublication::marshalRequest()
{
    return true;
}

//--------------------------------------------------------------------
// Get the theme.
//--------------------------------------------------------------------
const RomaTheme& RomaPublication::theme() const
{
    return m_impl->getTheme();
}

//--------------------------------------------------------------------
// Set the theme.
//--------------------------------------------------------------------
void RomaPublication::setTheme(const RomaTheme& theme)
{
    m_impl->setTheme(theme);
}

//--------------------------------------------------------------------
// Get the Message ID.
//--------------------------------------------------------------------
const RomaMsgId& RomaPublication::msgId() const
{
    return m_impl->getMsgId();
}

//--------------------------------------------------------------------
// Set the Message ID.
//--------------------------------------------------------------------
void RomaPublication::setMsgId(const RomaMsgId& msgId)
{
    m_impl->setMsgId(msgId);
}

//--------------------------------------------------------------------
// Get the Correlation ID.
//--------------------------------------------------------------------
const RomaMsgId& RomaPublication::correlId() const
{
    return m_impl->getCorrelId();
}

//--------------------------------------------------------------------
// Set the Correlation ID.
//--------------------------------------------------------------------
void RomaPublication::setCorrelId(const RomaMsgId& correlId)
{
    m_impl->setCorrelId(correlId);
}

//--------------------------------------------------------------------
// Set the message priority.
//--------------------------------------------------------------------
void RomaPublication::setPriority(int priority)
{
    m_impl->setPriority(priority);
}

//--------------------------------------------------------------------
// Set the message persistence.
//--------------------------------------------------------------------
void RomaPublication::setPersistence(Roma::Persistence persistence)
{
    m_impl->setPersistence(persistence);
}

//--------------------------------------------------------------------
// Set the message format.
//--------------------------------------------------------------------
void RomaPublication::setFormat(int format)
{
    m_impl->setFormat(format);
}

//--------------------------------------------------------------------
// Set the message feedback code.
//--------------------------------------------------------------------
void RomaPublication::setFeedback(int feedback)
{
    m_impl->setFeedback(feedback);
}

//--------------------------------------------------------------------
// Set the message expiry interval.
//--------------------------------------------------------------------
void RomaPublication::setExpiry(int expiry)
{
    m_impl->setExpiry(expiry);
}

//--------------------------------------------------------------------
// Set the message codepage.
//--------------------------------------------------------------------
void RomaPublication::setCodepage(int codepage)
{
    m_impl->setCodepage(codepage);
}

//--------------------------------------------------------------------
// Set the message encoding.
//--------------------------------------------------------------------
void RomaPublication::setEncoding(int encoding)
{
    m_impl->setEncoding(encoding);
}

//--------------------------------------------------------------------
// Set the message format name.
//--------------------------------------------------------------------
void RomaPublication::setFormatName(const char* name)
{
    m_impl->setFormatName(name);
}

//--------------------------------------------------------------------
// Set the message name.
//--------------------------------------------------------------------
void RomaPublication::setMsgName(const char* name)
{
    m_impl->setMsgName(name);
}

//--------------------------------------------------------------------
// Set the message data buffer.
//--------------------------------------------------------------------
bool RomaPublication::setMsgBuffer(const void* buffer, int length)
{
    return m_impl->setMsgBuffer(buffer, length);
}

//--------------------------------------------------------------------
// Set the message put selector.
//--------------------------------------------------------------------
void RomaPublication::setSelector(Roma::PutSelector selector)
{
    m_impl->setSelector(selector);
}

//--------------------------------------------------------------------
// Specify true to send the transaction message to a non-transactional
// queue.
//--------------------------------------------------------------------
void RomaPublication::setNoSyncAllowed(bool f)
{
    m_impl->setNoSyncAllowed(f);
}

//--------------------------------------------------------------------
// Set the API profile name.
//--------------------------------------------------------------------
void  RomaPublication::setApiProfileName(const char* name)
{
    m_impl->setApiProfile(name);
}

//--------------------------------------------------------------------
// Set the error profile name.
//--------------------------------------------------------------------
void RomaPublication::setErrorProfileName(const char* name)
{
    m_impl->setErrorProfile(name);
}

//--------------------------------------------------------------------
// Specify true to prohibit an EMS invocation on an error.
//--------------------------------------------------------------------
void RomaPublication::setNoEms(bool f)
{
    m_impl->setNoEms(f);
}

//--------------------------------------------------------------------
// Specify true to prohibit notifications.
//--------------------------------------------------------------------
void RomaPublication::setNoNotification(bool f)
{
    m_impl->setNoNotification(f);
}

//--------------------------------------------------------------------
// Specify true to prohibit a message transformation.
//--------------------------------------------------------------------
void RomaPublication::setNoTransformation(bool f)
{
    m_impl->setNoTransformation(f);
}

//--------------------------------------------------------------------
// Set the termination indicator.
//--------------------------------------------------------------------
void RomaPublication::setTermSupported(bool f)
{
    m_impl->setTermSupported(f);
}

//--------------------------------------------------------------------
// Specify true to suppress an error profile use.
//--------------------------------------------------------------------
void RomaPublication::setSuppressErrorProfile(bool f)
{
    m_impl->setSuppressErrorProfile(f);
}

//--------------------------------------------------------------------
// Set ILS data.
//--------------------------------------------------------------------
void RomaPublication::setILSData(const RomaILSData& data)
{
    m_impl->setILSData(data);
}

//--------------------------------------------------------------------
// Get ILS data.
//--------------------------------------------------------------------
RomaILSData* RomaPublication::getILSData() const
{
    return m_impl->getILSData();
}

//--------------------------------------------------------------------
// Set a syncpoint definition.
//--------------------------------------------------------------------
void RomaPublication::setSyncpointDef(const RomaSyncpointDef& def)
{
    m_impl->setSyncpointDef(def);
}

//--------------------------------------------------------------------
// Set the subject.
//--------------------------------------------------------------------
void RomaPublication::setSubject(const RomaSubject& subject)
{
    m_impl->setSubject(subject);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaPublication::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaPublication, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

