//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcosoa.cpp,v 1.10 2003/02/21 22:48:08 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcosoa.cpp,v $
// $Revision: 1.10 $
//--------------------------------------------------------------------
// The RomaStoreOptions public interface.
//--------------------------------------------------------------------

#include <kxcointa.hpp>
#include <kxcoiopt.hpp>
#include <kxcoso.hpp>

TRACE_MODULE_VERSION($Revision: 1.10 $);


//--------------------------------------------------------------------
// Default constructor.
//--------------------------------------------------------------------
RomaStoreOptions::RomaStoreOptions()
{
    TRACE_ENTRY(RomaStoreOptions ctor, << TRACE(this));

    m_impl = new RomaImplStoreOptions;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaStoreOptions::~RomaStoreOptions()
{
    TRACE_ENTRY(RomaStoreOptions dtor, << TRACE(this));

    delete m_impl;

    TRACE_VOID_RETURN;
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaStoreOptions::RomaStoreOptions(const RomaStoreOptions& opt)
{
    m_impl = new RomaImplStoreOptions(*opt.m_impl);
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaStoreOptions&
RomaStoreOptions::operator = (const RomaStoreOptions& opt)
{
    *m_impl = *opt.m_impl;
    return *this;
}

//--------------------------------------------------------------------
// Set message selector.
//--------------------------------------------------------------------
void RomaStoreOptions::setSelector(Roma::PutSelector sel)
{
    m_impl->setSelector(sel);
}

//--------------------------------------------------------------------
// Specify true to send a message under transaction control to a
// nontransactional queue.
//--------------------------------------------------------------------
void RomaStoreOptions::setNoSyncAllowed(bool f)
{
    m_impl->setNoSyncAllowed(f);
}

//--------------------------------------------------------------------
// Set a name of the API profile.
//--------------------------------------------------------------------
void RomaStoreOptions::setApiProfileName(const char* name)
{
    m_impl->setApiProfile(name);
}


// THEME combinations (2 pairs of mutually exclusive options) ...
//
//        USE   NEW   CLOSE   RETAIN
//USE      -     X      ok      ok
//
//NEW      X     -      ok      ok
//
//CLOSE    ok    ok     -        X
//
//RETAIN   ok    ok     X       -

void RomaStoreOptions::setNewTheme(bool f)
{
    m_impl->setNewTheme(f);
}

void RomaStoreOptions::setUseTheme(bool f)
{
    m_impl->setUseTheme(f);
}

void RomaStoreOptions::setCloseTheme(bool f)
{
    m_impl->setCloseTheme(f);
}

void RomaStoreOptions::setRetainTheme(bool f)
{
    m_impl->setRetainTheme(f);
}

//--------------------------------------------------------------------
// Set the syncpoint definition.
//--------------------------------------------------------------------
void RomaStoreOptions::setSyncpointDef(const RomaSyncpointDef& def)
{
    m_impl->setSyncpointDef(def);
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaStoreOptions::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaStoreOptions, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

