//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1998-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcosyna.cpp,v 1.7 2003/02/21 22:48:09 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcosyna.cpp,v $
// $Revision: 1.7 $
//--------------------------------------------------------------------
// The RomaSyncpointDef public interface.
//--------------------------------------------------------------------

#include "kxcointa.hpp"
#include "kxcoigdt.hpp"
#include "kxcosyn.hpp"

//--------------------------------------------------------------------
// Default constructor.
// Optionally accepts the syncpoint type to initialize the object.
//--------------------------------------------------------------------
RomaSyncpointDef::RomaSyncpointDef(Roma::SyncType type)
{
    m_impl = new RomaImplSyncpoint(type);
}

//--------------------------------------------------------------------
// Constructor.
// Creates the global syncpoint definition.
// The specified pointer will be used as a transaction context.
// The syncpoint type will be set to SYNC_GLOBAL.
//--------------------------------------------------------------------
RomaSyncpointDef::RomaSyncpointDef(PKXCVOID context)
{
    m_impl = new RomaImplSyncpoint(context);
}

//--------------------------------------------------------------------
// Destructor.
//--------------------------------------------------------------------
RomaSyncpointDef::~RomaSyncpointDef()
{
    delete m_impl;
}

//--------------------------------------------------------------------
// Set the syncpoint type.
//--------------------------------------------------------------------
void RomaSyncpointDef::setType(Roma::SyncType type)
{
    m_impl->setType(type);
}

//--------------------------------------------------------------------
// Set the transaction context.
//--------------------------------------------------------------------
void RomaSyncpointDef::setContext(PKXCVOID context)
{
    m_impl->setContext(context);
}

//--------------------------------------------------------------------
// Copy constructor.
//--------------------------------------------------------------------
RomaSyncpointDef::RomaSyncpointDef(const RomaSyncpointDef& def)
{
    m_impl = new RomaImplSyncpoint(*def.m_impl);
}

//--------------------------------------------------------------------
// Assignment operator.
//--------------------------------------------------------------------
RomaSyncpointDef& 
RomaSyncpointDef::operator = (const RomaSyncpointDef& def)
{
    *m_impl = *def.m_impl;
    return *this;
}

//--------------------------------------------------------------------
// Output trace information.
//--------------------------------------------------------------------
void RomaSyncpointDef::trace(RomaTraceStream& s) const
{
    s << TRACE_CLASS(RomaSyncpointDef, << TRACE(m_impl)) << "->";
    m_impl->trace(s);
}

