//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaClientResponse.java
//
// Defines      : class RomaClientResponse
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;
import com.candle.roma.kxc.raw.*;
import java.util.*;

/**
* This class encapsulates responses returned to Roma client
* applications from a Roma Business Service.
* It also provides a framework for converting the message
* response data to structured application-specific response data.
* <p>
* This class may be subclassed to provide unmarshalling for 
* response data following a client request. 
* 
*/
public class RomaClientResponse 
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
  // RomaClientReponse has a RomaMessage
  // Package visibility so other classes can access
  RomaMessage m_Message = new RomaMessage();
  
  public RomaClientResponse()
  {
  }
  
  public RomaClientResponse( RomaClientResponse response )
  {
    m_Message = response.m_Message;
  }
  
  RomaClientResponse( RomaMessage msg )
  {
    m_Message = msg;
  }
  
  //== RomaMesssage Access functions ==================================
  
  /**
  * Return the message type. This can be one of the following :-
  * <ul>
  * <li>RESPONSE    - response received by the client
  * <li>NOTIFICATION - notification response received by the client
  * </ul>
  * Note that these are values returned to indicate the type of 
  * an incoming message.  An application cannot set the message
  * type in order to create a specific type of message.  The
  * type of an outgoing message is defined implicitly by the
  * options selected when the message is sent.
  */
  public int msgType()
  {
    return m_Message.msgType();
  }
  
  /**
  * Return the message priority. 
  */
  public int priority()
  {
    return m_Message.priority();
  }
  
  /**
  * Return the persistence. This can be one of the following :-
  * <ul>
  * <li>PERSISTENCE_YES     - persistent
  * <li>PERSISTENCE_NO      - not persistent
  * </ul>
  */
  public int persistence()
  {
    return m_Message.persistence();
  }
  
  /**
  * Return the message format.
  */
  public int format()
  {
    return m_Message.format();
  }
  
  /**
  * Get the message feedback code.
  */
  public int feedback()
  {
    return m_Message.feedback();
  }
  
  /**
  * Return the message expiry time in seconds.
  */
  public int expiry()
  {
    return m_Message.expiry();
  }
  
  /**
  * Return the message codepage.
  */
  public int codepage()
  {
    return m_Message.codepage();
  }
  
  /**
  * Return the encoding. This can be one of the following :-
  * <ul>
  * <li>ENCODING_DEFAULT     - default encoding
  * <li>ENCODING_NORMAL      - normal encoding
  * <li>ENCODING_REVERSED    - reversed encoding
  * </ul>
  */
  public int encoding()
  {
    return m_Message.encoding();
  }
  
  /**
  * Return the message identifier in a byte array.
  */
  public byte[] msgId()
  {
    return m_Message.msgId();
  }
  
  /**
  * Return the message correlation identifier.
  */
  public byte[] correlId()
  {
    return m_Message.correlId();
  }
  
  /**
  * Return reference to the message data buffer.
  */
  public byte[] msgBuffer()
  {
    return m_Message.msgBuffer();
  }
  
  /**
  * Return message data buffer length.
  */
  public int msgDataLength()
  {
    return m_Message.msgDataLength();
  }
  
  /**
  * Return reference to the message tag buffer.
  */
  public byte[] tagBuffer()
  {
    return m_Message.tagBuffer();
  }
  
  /**
  * Return message data buffer length.
  */
  public int tagDataLength()
  {
    return m_Message.tagDataLength();
  }
  
  /**
  * Return the message status. This can be one of the following :-
  * <ul>
  * <li>STATUS_NONE           - anonymous message authentication used
  * <li>STATUS_AUTHENTICATED  - message authenticated
  * <li>STATUS_OK_TO_RESPOND  - safe to respond to this message
  * <li>STATUS_OK_TO_NOTIFY   - safe to send a notification message
  * </ul>
  */
  public int status()
  {
    return m_Message.status();
  }
  
  /**
  * Return the backout count. This is the number 
  * of times that the message has been rolled back.
  */
  public int backoutCount()
  {
    return m_Message.backoutCount();
  }
  
  /**
  * Return the message format name.
  */
  public String formatName()
  {
    return m_Message.formatName();
  }
  
  /**
  * Returns the time stamp which, for get operations, is the 
  * time at which the put operation occurred.
  */
  public Date timeStamp( )
  {
    return m_Message.timeStamp();
  }
  
  /**
  * Return the message theme. Theme can be set internally when
  * the message is being sent or received or else explicitly by
  * the user.
  */
  public RomaTheme theme()
  {
    return m_Message.theme();
  }
  
  /**
  * Returns name of message
  */
  public String msgName( )
  {
    return m_Message.messageName();
  }
  
  /**
  * Returns the subject name of the received message.
  */
  public String subject( )
  {
    return m_Message.subject();
  }
  
  /**
  * Get incoming ILS data
  *
  */
  public RomaILSData getILSData()
  {							 
    if ( m_Message.getILSDataHandle() == Kxc.NULL_ILS_DATA_HANDLE)
      return null;
    else
      return new RomaILSData( m_Message.getILSDataHandle() );
  }
}
