//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: RomaError.java,v 1.6 2003/11/18 15:18:36 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/java/com/candle/roma/kxc/RomaError.java,v $
//--------------------------------------------------------------------
//
// Module       : RomaError.java
//
// Defines      : class RomaError
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;
import java.io.*;

/**
* This class represents Roma error information.
*/
public class RomaError extends Exception implements Roma
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999-2003";
  
  int m_status;
  
  RomaErrorInformation m_primInfo = new RomaErrorInformation();
  RomaErrorInformation m_secInfo = new RomaErrorInformation();
  
  /**
  * Constructs a new RomaError with no additional information.
  */
  public RomaError()
  {
  }
  
  /**
  * Constructs a new RomaError object with a given textual
  * description.
  * Package level visibility only
  */
  RomaError( String s )
  {
    super( s );
  }
  
  /**
  * Non-public constructor that takes an existing KxcError object
  * and sets the members of this new instance according to the
  * state of that object.
  */
  RomaError( KxcError err )
  {
    super( createErrorText( err ) );
    m_status = err.status();
    
    if ( hasPrimaryInfo() )
      m_primInfo = new RomaErrorInformation( err.result() );
    else
      m_primInfo = new RomaErrorInformation();
    
    if ( hasSecondaryInfo() )
      m_secInfo = new RomaErrorInformation( err.subResult() );
    else
      m_secInfo = new RomaErrorInformation();
  }
  
  /**
  * @return <code>true</code> if error information is set.
  */
  public boolean anyError()
  {
    return m_status != STATUS_NONE;
  }
  
  /**
  * @param errorSubclass either <code>Roma.SUBCLASS_PRIMARY</code> to access primary error information or
  *                      <code>Roma.SUBCLASS_SECONDARY</code> to access secondary error information.
  *
  * @return One of the <code>Roma.CC_*</code> constants.
  */
  public int compCode( int errorSubclass )
  {
    if ( anyError() )
    {
      if ( errorSubclass == Kxc.PRIMARY_INFO )
        return m_primInfo.compCode();
      else
        return m_secInfo.compCode();
    }
    else
      return Roma.CC_OK;
  }
  
  /**
  * @param errorSubclass either <code>Roma.SUBCLASS_PRIMARY</code> to access primary error information or
  *                      <code>Roma.SUBCLASS_SECONDARY</code> to access secondary error information.
  *
  * @return One of the <code>Roma.RC_*</code> constants.
  */
  public int returnCode( int errorSubclass )
  {
    if ( anyError() )
    {
      if ( errorSubclass == Kxc.SUBCLASS_PRIMARY )
        return m_primInfo.returnCode();
      else
        return m_secInfo.returnCode();
    }
    else
      return 0;
  }
  
  /**
  * @return Either a reason code of the error or 0
  * if no error information is available.
  *
  * @param errorSubclass either <code>Roma.SUBCLASS_PRIMARY</code> to access primary error information or
  *                      <code>Roma.SUBCLASS_SECONDARY</code> to access secondary error information.
  *
  */
  public int reason( int errorSubclass )
  {
    if ( anyError() )
    {
      if ( errorSubclass == Kxc.SUBCLASS_PRIMARY )
        return m_primInfo.reason();
      else
        return m_secInfo.reason();
    }
    else
      return 0;
  }
  
  /**
  * @param errorSubclass either <code>Roma.SUBCLASS_PRIMARY</code> to access primary error information or
  *                      <code>Roma.SUBCLASS_SECONDARY</code> to access secondary error information.
  *
  * @return One of the <code>Roma.ETYPE_*</code> constants or 0 if no error information is available.
  */
  public int errorType( int errorSubclass )
  {
    if ( anyError() )
    {
      if ( errorSubclass == Kxc.SUBCLASS_PRIMARY )
        return m_primInfo.type();
      else
        return m_secInfo.type();
    }
    else
      return 0;
  }
  
  /**
  * @param errorSubclass either <code>Roma.SUBCLASS_PRIMARY</code> to access primary error information or
  *                      <code>Roma.SUBCLASS_SECONDARY</code> to access secondary error information.
  *
  * @return One of the <code>Roma.OP_*</code> constants.
  */
  public int operationType( int errorSubclass )
  {
    if ( anyError() )
    {
      if ( errorSubclass == Kxc.SUBCLASS_PRIMARY )
        return m_primInfo.operationType();
      else
        return m_secInfo.operationType();
    }
    else
      return 0;
  }
  
  /**
  * @param errorSubclass either <code>Roma.SUBCLASS_PRIMARY</code> to access primary error information or
  *                      <code>Roma.SUBCLASS_SECONDARY</code> to access secondary error information.
  *
  * @return A position where the error has occured.
  */
  public int position( int errorSubclass )
  {
    if ( anyError() )
    {
      if ( errorSubclass == Kxc.SUBCLASS_PRIMARY )
        return m_primInfo.position();
      else
        return m_secInfo.position();
    }
    else
      return 0;
  }
  
  /**
  * @param errorSubclass either <code>Roma.SUBCLASS_PRIMARY</code> to access primary error information or
  *                      <code>Roma.SUBCLASS_SECONDARY</code> to access secondary error information.
  *
  * @return Additional error information.
  */
  public String additionalInfo( int errorSubclass )
  {
    if ( anyError() )
    {
      if ( errorSubclass == Kxc.SUBCLASS_PRIMARY )
        return m_primInfo.additionalInfo();
      else
        return m_secInfo.additionalInfo();
    }
    else
      return "";
  }
  
  /**
  * @param errorSubclass either <code>Roma.SUBCLASS_PRIMARY</code> to access primary error information or
  *                      <code>Roma.SUBCLASS_SECONDARY</code> to access secondary error information.
  *
  * @param errorFormat either <code>Roma.ERROR_FULL</code> to get a detailed error description or
  *                    <code>Roma.ERROR_COMPACT</code> to get a brief error description.
  *
  * @return An error description.
  */
  public String errorText( int errorSubclass, int errorFormat )
  {
    if ( anyError() )
    {
      RomaErrorInformation info;
      if ( errorSubclass == Kxc.SUBCLASS_PRIMARY )
        info = m_primInfo;
      else
        info = m_secInfo;
      if ( errorFormat == Kxc.ERROR_TEXT_FULL )
        return info.fullText();
      else
        return info.shortText();
    }
    else
      return "";
  }
  
  /**
  * @return One of the <code>Roma.ERROR_STATUS_*</code> constants.
  */
  public int status()
  {
    return m_status;
  }
  
  // Methods for use within package
  
  /**
  * Sets the error status.
  *
  * @param status   new error status.
  */
  void setStatus( int status )
  {
    m_status = status;
  }
  
  /**
  * Returns true if primary error information is set.
  */
  boolean hasPrimaryInfo()
  {
    return (m_status & Roma.ERROR_STATUS_PRIMARY) != 0;
  }
  
  /**
  * Returns the primary error information.
  */
  RomaErrorInformation primaryInfo()
  {
    return m_primInfo;
  }
  
  /**
  * Returns true if secondary error information has been set.
  * (i.e. status() == STATUS_BOTH)
  */
  boolean hasSecondaryInfo()
  {
    return m_status == Roma.ERROR_STATUS_BOTH;
  }
  
  /**
  * Returns the secondary error information.
  */
  RomaErrorInformation secondaryInfo()
  {
    return m_secInfo;
  }
  
  /**
  * Receives a KxcError object and sets it up using the state
  * of this object.
  *
  * @param err  KxcError instance - ie, an encapsulation of a
  *             KXCERROR structure in the C level layer.
  *
  * @see #RomaError( KxcError )
  *
  */
  void setPeer( KxcError err )
  {
    err.setStatus( m_status );
    
    if ((m_status & Kxc.ERRSTAT_MAIN) != 0)
      m_primInfo.setPeer( err.result() );
    
    if ((m_status & Kxc.ERRSTAT_SUB) != 0)
      m_secInfo.setPeer( err.subResult() );
  }
  
  private static String createErrorText( KxcError err )
  {
    return
      "Roma error status: "+err.status()+
      ", main info: "+subErrorText( err.result() )+
      ", sub info: "+subErrorText( err.subResult() )
      ;
  }
  
  private static String subErrorText( KxcSubError res )
  {
    return res == null || res.fullText() == null? "": res.fullText();
  }

  RomaError(int status, int comp1, int ret1, int oper1, int reason1,
            int pos1, int type1, String loc1, String short1, String full1,
            int comp2, int ret2, int oper2, int reason2, int pos2, int type2,
            String loc2, String short2, String full2)
  {
    super(full1);

    m_status = status;

    if ( hasPrimaryInfo() )
      m_primInfo = new RomaErrorInformation(
        comp1, ret1, oper1, reason1, pos1, type1, loc1, short1, full1 );
    else
      m_primInfo = new RomaErrorInformation();
    
    if ( hasSecondaryInfo() )
      m_secInfo = new RomaErrorInformation(
        comp2, ret2, oper2, reason2, pos2, type2, loc2, short2, full2 );
    else
      m_secInfo = new RomaErrorInformation();
  }
}


//--------------------------------------------------------------------
// End of file. RomaError.java.
//--------------------------------------------------------------------
