//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : RomaStoreOptions.java
//
// Defines      : class RomaStoreOptions
//
//--------------------------------------------------------------------

package com.candle.roma.kxc;

import com.candle.roma.kxc.raw.*;

/**
* This class encapsulates the options available when storing a 
* message to a storage queue.
*/
public class RomaStoreOptions 
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999,2000,2001";
  
 	private RomaBasePutOptions m_po = null; 
  
  /**
  * Constructs a new instance with store options initialised to 
  * the default.
  */
  public RomaStoreOptions()
  {
    //----------------------------------------------------------------
    // Calling base class constructor with true means that the
    // new theme flag is selected.
    //----------------------------------------------------------------
    m_po = new RomaBasePutOptions(true);
  }
  
  // Methods from base put options
  
  /**
  * Sets the put selector.
  *
  * @param sel the new value of the put selector, one of :-
  * <ul>
  *   <li>USE_NONE
  *   <li>USE_MSGID
  *   <li>USE_CORRELID
  *   <li>USE_MSGID_AND_CORRELID
  * </ul>
  */
  public void setSelector( int sel )
  {
    m_po.setSelector( sel );
  }
  
  /**
  * Controls whether an MSMQ message can be placed outside syncpoint
  * control. If the application attempts to put a message to MSMQ inside  
  * syncpoint control but the target queue does not support transactions,
  * the call is normally rejected with error code RC_NONXACTIONAL.
  * The application can call this method with true flag to force an MSMQ
  * message to be placed outside syncpoint control.
  * @param flag  one of :-
  * <ul>
  *  <li>true - allow MSMQ messages to be placed outside syncpoint control
  *  <li>false - don't allow MSMQ messages to be placed outside syncpoint
  *              control
  * </ul>
  */
  public void setNoSyncAllowed( boolean flag )
  {
    m_po.setNoSyncAllowed( flag );
  }
  
  /**
  * Associates an API profile with the put operation.
  *
  * @param profile  the name of the API profile.
  */
  public void setApiProfileName( String profile )
  {
    m_po.setApiProfileName( profile );
  }
  
  /**
  * Selects whether or not a new theme will be allocated
  * when applying these options.
  *
  * @param flag  if true then new theme allocation is enabled,
  *   otherwise it is disabled. If disabled, the theme already
  *   associated with the RomaMessage will be used.
  */
  public void setNewTheme( boolean flag )
  {
    m_po.setNewTheme( flag );
  }
  
  /**
  * Selects whether or not a new theme will be allocated
  * when applying these options
  * 
  * @param flag  if true theme already associated with the 
  * RomaMessage will be used. If false the new theme 
  * allocation will be done.
  */
  public void setUseTheme(boolean flag)
  {
    setNewTheme(!flag);
  }
  
  /**
  * Selects whether or not the theme is to be closed or else
  * retained in the put operation.
  * 
  * @param flag  if true then new retain the theme. Otherwise
  *   close the theme.
  */
  public void setRetainTheme( boolean flag )
  {
    m_po.setRetainTheme( flag );
  }
  
  /**
  * Selects whether of not the theme is to be closed or retained
  * in the put operation.
  * 
  * @param flag if true close the theme otherwise retain the theme
  */
  public void setCloseTheme(boolean flag)
  {
    m_po.setCloseTheme( flag );
  }
  
  /**
  * Specify sync point definitiion
  * <p>
  * @param  sync definition to be used.
  */
  public void setSyncpointDef( RomaSyncpointDef sync )
  {
    m_po.setSyncpointDef( sync );
  }
  
  // end of base put option methods
  
  RomaBasePutOptions getBasePutOptions()
  { 
    return m_po; 
  }
  
}
