//--------------------------------------------------------------------
//
// Copyright Candle Corporation 1999-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
//
// Module       : KxcServer.java
//
// Defines      : class KxcServer
//
//--------------------------------------------------------------------

package com.candle.roma.kxc.raw;

/**
* Introduces Server abstraction in the raw interface.
* Extends the KxcContext class with put/get server
* operations and ability to receive and parse exceptions.
*/

public class KxcServer extends KxcContext
{
  private static final String copyrightNotice =
    "Copyright Candle Corporation 1999-2002";
  
  // Initilizes new server context.
  // Configures kxcServerInitialize operation
  // which should be call by the user subsequently
  KxcServer( KxcOper  oper, 
             String   name, 
             int      opt, 
             byte     sec[], 
             String   modq,
             long     ilsDataHandle,
             String   id,
             String   pwd )
  {
    super( oper );
    m_op[NAME]         = name;
    m_ip[OPT]          = opt;
    m_op[SECBUF]       = sec;
    m_op[MODEL_QUEUE]  = modq;
    m_ip[OP_CODE]      = NIOP_SVRINIT;
    m_lp[ILS_DATA_HANDLE] = ilsDataHandle;
    m_op[DIR_IDENTITY] = id;
    m_op[DIR_PASSWORD] = pwd;
  }
  
  // configures and returns kxcGetRequest operation.
  public KxcOper get()
  {
    m_ip[OP_CODE] = NIOP_GETREQ;
    return m_oper;
  }
  
  // configures and returns kxcPutResponse operation.
  public KxcOper put()
  {
    m_ip[OP_CODE] = NIOP_PUTRESP;
    return m_oper;
  }
  
  // configures and returns subsequent kxcGetRequest
  // and kxcInterpretMessage operations.
  public KxcOper getException( int options )
  {
    m_ip[INTERPRET_OPT] = options;
    m_ip[OP_CODE]       = NIOP_GETEXCEPTION;
    return m_oper;
  }
}

//--------------------------------------------------------------------
// End of file KxcServer.java
//--------------------------------------------------------------------
