//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2001-2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcodac.hpp,v 1.5 2003/02/21 22:48:03 emueh Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcodac.hpp,v $
// $Revision: 1.5 $
//--------------------------------------------------------------------
// The RomaDirectoryAccess public interface.
//--------------------------------------------------------------------

#ifndef __KXCODAC_HPP__
#define __KXCODAC_HPP__

#include <kxcotrb.hpp>

//--------------------------------------------------------------------
// The internal implementation class.
//--------------------------------------------------------------------
class RomaImplDirAccess;


//--------------------------------------------------------------------
// The RomaDirectoryAccess class.
//
// Represents LDAP authentication information to be checked
// against the ACL.
//--------------------------------------------------------------------
class KXCODLL RomaDirectoryAccess : public RomaTraceable
{
public:
    //----------------------------------------------------------------
    // Default constructor.
    //----------------------------------------------------------------
    RomaDirectoryAccess();

    //----------------------------------------------------------------
    // Constructor.
    // Initializes the object with the user ID and password.
    //----------------------------------------------------------------
    RomaDirectoryAccess(const char* userID, 
                        const char* password);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaDirectoryAccess();

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaDirectoryAccess(const RomaDirectoryAccess&);
    RomaDirectoryAccess& operator = (const RomaDirectoryAccess&);

    //----------------------------------------------------------------
    // Set the LDAP user ID.
    //----------------------------------------------------------------
    bool setUserId(const char*);

    //----------------------------------------------------------------
    // Get the LDAP user ID.
    //----------------------------------------------------------------
    const char* getUserId() const;

    //----------------------------------------------------------------
    // Set the LDAP password.
    //----------------------------------------------------------------
    bool setPassword(const char*);

    //----------------------------------------------------------------
    // Get the LDAP password.
    //----------------------------------------------------------------
    const char* getPassword() const;


    //----------------------------------------------------------------
    // IMPLEMENTATION METHODS.
    //----------------------------------------------------------------
    virtual void trace(RomaTraceStream&) const;


protected:
    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplDirAccess* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCODAC_HPP__

