//--------------------------------------------------------------------
//
// Copyright Candle Corporation 2003
//
// This code is the confidential and proprietary property of Candle
// Corporation that is licensed for your internal company use only
// and is not to be reproduced, used, licensed, sold or disclosed
// outside of your corporation except upon written authorization of
// Candle Corporation. The code can be used for internal development,
// test and production applications.  Any unauthorized use,
// duplication, or disclosure is unlawful. You may not sell or license
// this code in any commercial products.
//
// Permission is granted to you to alter the code for your internal
// use only, subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented; you must
//    not claim that you wrote the code.  If you use this code in
//    conjunction with other code, an acknowledgement is required.
//
// 2. Altered versions of this code must be plainly marked as such,
//    and must not be misrepresented as being the original code.
//
// 3. This notice may not be removed or altered from any source
//    distribution.
//
// This code is provided with RESTRICTED RIGHTS.  Use, duplication or
// disclosure by the Government is subject to restrictions set forth
// in the applicable license agreement and/or Government Data Rights
// Clauses(s), as applicable (including but not limited to DFARS
// 252.227-7013(c)(ii) and FAR 52.227-19(a)(d).  Unpublished-All
// rights reserved under United States Copyright Laws.  Manufacturer
// is Candle Corporation, 100 N. Sepulveda Boulevard, El Segundo, CA 90245.
//
//--------------------------------------------------------------------
// $Id: kxcoems.hpp,v 1.2 2004/01/22 17:53:24 e_dm Exp $
// $Source: /romacvs/root/roma/src/kxc/omodel/cpp/kxcoems.hpp,v $
// $Revision: 1.2 $
//--------------------------------------------------------------------
// The EMS API public classes.
//--------------------------------------------------------------------

#ifndef __KXCOEMS_HPP
#define __KXCOEMS_HPP

#include <kxcots.hpp>
#include <kxcothm.hpp>
#include <kxcoerr.hpp>
#include <kxcobrq.hpp>
#include <kxcocrs.hpp>

//--------------------------------------------------------------------
// Forward declaration of the internal implementation classes.
//--------------------------------------------------------------------
class RomaImplBusFlow;
class RomaImplBusService;
class RomaImplRomaConfig;
class RomaImplEteInfo;
class RomaImplExcInfo;
class RomaImplExcErrorData;
class RomaImplMsgInfo;
class RomaImplRoute;
class RomaImplSysConfig;
class RomaImplTrailer;
class RomaImplWfService;
class RomaImplWorkflow;
class RomaImplRouteList;
class RomaImplTrailerList;
class RomaImplThemeInfo;
class RomaImplRouteListIter;
class RomaImplTrailerListIter;

class RomaTrailerList;
class RomaRouteList;


//--------------------------------------------------------------------
// The RomaBusinessFlow class.
//
// Defines business flow information. Used to set business flow
// information on the context with the setBusinessFlow() method of
// RomaClient or RomaBusinessElement.
//--------------------------------------------------------------------
class KXCODLL RomaBusinessFlow
{
public:
    //----------------------------------------------------------------
    // Constructor.
    //----------------------------------------------------------------
    explicit RomaBusinessFlow(const char* flowName,
                              const char* operationName = NULL);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaBusinessFlow();

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaBusinessFlow(const RomaBusinessFlow&);
    RomaBusinessFlow& operator=(const RomaBusinessFlow&);

    //----------------------------------------------------------------
    // Returns the business flow name.
    //----------------------------------------------------------------
    const char* getFlowName() const;

    //----------------------------------------------------------------
    // Sets the business flow name.
    //----------------------------------------------------------------
    void setFlowName(const char* flowName);

    //----------------------------------------------------------------
    // Returns the business operation name.
    //----------------------------------------------------------------
    const char* getOperationName() const;

    //----------------------------------------------------------------
    // Sets the business operation name.
    //----------------------------------------------------------------
    void setOperationName(const char* operationName);


private:
    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplBusFlow* m_impl;

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaBusinessService class.
//
// Defines a business service that is used to insert or replace
// a theme workflow. Refer to the insertWorkflow() and
// replaceWorkflow() methods of RomaClient and RomaBusinessElement.
//--------------------------------------------------------------------
class KXCODLL RomaBusinessService
{
public:
    //----------------------------------------------------------------
    // Constructor.
    //----------------------------------------------------------------
    explicit RomaBusinessService(const char* serviceName);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaBusinessService();

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaBusinessService(const RomaBusinessService&);
    RomaBusinessService& operator=(const RomaBusinessService&);

    //----------------------------------------------------------------
    // Returns the business service name.
    //----------------------------------------------------------------
    const char* getName() const;

    //----------------------------------------------------------------
    // Sets the business service name.
    //----------------------------------------------------------------
    void setName(const char* serviceName);


private:
    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplBusService* m_impl;

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaRoute class.
//
// Defines a workflow route step.
//--------------------------------------------------------------------
class KXCODLL RomaRoute
{
public:
    //----------------------------------------------------------------
    // Constructor.
    //----------------------------------------------------------------
    RomaRoute();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaRoute();

    //----------------------------------------------------------------
    // Shallow copy and assignment.
    //----------------------------------------------------------------
    RomaRoute(const RomaRoute&);
    RomaRoute& operator=(const RomaRoute&);

    //----------------------------------------------------------------
    // Deep copy.
    //----------------------------------------------------------------
    RomaRoute clone() const;

    //----------------------------------------------------------------
    // Returns the route type.
    //----------------------------------------------------------------
    Roma::AppType getType() const;

    //----------------------------------------------------------------
    // Sets the route type.
    //----------------------------------------------------------------
    void setType(Roma::AppType type);

    //----------------------------------------------------------------
    // Returns the route flags. See Roma::RouteFlags enumeration.
    //----------------------------------------------------------------
    unsigned long getFlags() const;

    //----------------------------------------------------------------
    // Sets the route flags. See Roma::RouteFlags enumeration.
    //----------------------------------------------------------------
    void setFlags(unsigned long);

    //----------------------------------------------------------------
    // Returns the target queue name.
    //----------------------------------------------------------------
    const char* getTargetQueue() const;

    //----------------------------------------------------------------
    // Sets the target queue name.
    //----------------------------------------------------------------
    void setTargetQueue(const char*);

    //----------------------------------------------------------------
    // Returns the target business element name.
    //----------------------------------------------------------------
    const char* getTargetElement() const;

    //----------------------------------------------------------------
    // Sets the target business element name.
    //----------------------------------------------------------------
    void setTargetElement(const char*);

    //----------------------------------------------------------------
    // Returns the target business component name.
    //----------------------------------------------------------------
    const char* getTargetComponent() const;

    //----------------------------------------------------------------
    // Sets the target business component name.
    //----------------------------------------------------------------
    void setTargetComponent(const char*);

    //----------------------------------------------------------------
    // Returns the target codepage.
    //----------------------------------------------------------------
    const char* getTargetCodePage() const;

    //----------------------------------------------------------------
    // Sets the target codepage.
    //----------------------------------------------------------------
    void setTargetCodePage(const char*);

    //----------------------------------------------------------------
    // Returns the source business service name.
    //----------------------------------------------------------------
    const char* getSourceService() const;

    //----------------------------------------------------------------
    // Sets the source business service name.
    //----------------------------------------------------------------
    void setSourceService(const char*);

    //----------------------------------------------------------------
    // Returns the source business flow name.
    //----------------------------------------------------------------
    const char* getSourceFlow() const;

    //----------------------------------------------------------------
    // Sets the source business flow name.
    //----------------------------------------------------------------
    void setSourceFlow(const char*);

    //----------------------------------------------------------------
    // Returns the source business operation name.
    //----------------------------------------------------------------
    const char* getSourceOperation() const;

    //----------------------------------------------------------------
    // Sets the source business operation name.
    //----------------------------------------------------------------
    void setSourceOperation(const char*);


protected:
    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplRoute* m_impl;

private:
    explicit RomaRoute(RomaImplRoute*);

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaRouteListIterator class.
//
// Defines an iterator for the workflow routes list.
//--------------------------------------------------------------------
class KXCODLL RomaRouteListIterator
{
public:
    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaRouteListIterator();

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaRouteListIterator(const RomaRouteListIterator&);
    RomaRouteListIterator& operator=(const RomaRouteListIterator&);

    //----------------------------------------------------------------
    // Assigns the next route in the list to the specified route.
    //----------------------------------------------------------------
    bool getNext(RomaRoute&) const;

private:
    //----------------------------------------------------------------
    // Constructor.
    //----------------------------------------------------------------
    RomaRouteListIterator(RomaImplRouteList&);

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplRouteListIter* m_impl;

    friend class RomaRouteList;
};


//--------------------------------------------------------------------
// The RomaRouteList class.
//
// Defines a list of route steps in the workflow.
//--------------------------------------------------------------------
class KXCODLL RomaRouteList
{
public:
    //----------------------------------------------------------------
    // Constructor.
    //----------------------------------------------------------------
    RomaRouteList();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaRouteList();

    //----------------------------------------------------------------
    // Shallow copy and assignment.
    //----------------------------------------------------------------
    RomaRouteList(const RomaRouteList&);
    RomaRouteList& operator=(const RomaRouteList&);

    //----------------------------------------------------------------
    // Deep copy.
    //----------------------------------------------------------------
    RomaRouteList clone() const;

    //----------------------------------------------------------------
    // Returns true if the list is empty.
    //----------------------------------------------------------------
    bool isEmpty() const;

    //----------------------------------------------------------------
    // Returns number of trailers in the list.
    //----------------------------------------------------------------
    unsigned int getCount() const;

    //----------------------------------------------------------------
    // Removes all routes from the list.
    //----------------------------------------------------------------
    void clear();

    //----------------------------------------------------------------
    // Adds the specified route to the end of the list.
    //----------------------------------------------------------------
    void add(const RomaRoute& route);

    //----------------------------------------------------------------
    // Inserts the specified route into the list at the specified
    // index.
    //----------------------------------------------------------------
    bool insert(const RomaRoute& route, unsigned int index = 0);

    //----------------------------------------------------------------
    // Removes the route at the specified index from the list.
    //----------------------------------------------------------------
    bool remove(unsigned int index);

    //----------------------------------------------------------------
    // Removes the specified route from the list.
    //----------------------------------------------------------------
    bool remove(const RomaRoute& route);

    //----------------------------------------------------------------
    // Assigns the route at the specified index to the specified
    // route.
    //----------------------------------------------------------------
    bool getAt(RomaRoute& route, unsigned int index) const;

    //----------------------------------------------------------------
    // Returns the list iterator.
    //----------------------------------------------------------------
    RomaRouteListIterator getIterator() const;


protected:
    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplRouteList* m_impl;

private:
    explicit RomaRouteList(RomaImplRouteList*);

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaTrailer class.
//
// Defines an ILS persistent trailer.
//--------------------------------------------------------------------
class KXCODLL RomaTrailer
{
public:
    //----------------------------------------------------------------
    // Constructors.
    //----------------------------------------------------------------
    RomaTrailer();
    RomaTrailer(Roma::ILSType type, unsigned long method,
                const void* data, unsigned long length);

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaTrailer();

    //----------------------------------------------------------------
    // Shallow copy and assignment.
    //----------------------------------------------------------------
    RomaTrailer(const RomaTrailer&);
    RomaTrailer& operator=(const RomaTrailer&);

    //----------------------------------------------------------------
    // Deep copy.
    //----------------------------------------------------------------
    RomaTrailer clone() const;

    //----------------------------------------------------------------
    // Returns the trailer type.
    //----------------------------------------------------------------
    Roma::ILSType getType() const;

    //----------------------------------------------------------------
    // Sets the trailer type.
    //----------------------------------------------------------------
    void setType(Roma::ILSType type);

    //----------------------------------------------------------------
    // Returns the trailer method.
    //----------------------------------------------------------------
    unsigned long getMethod() const;

    //----------------------------------------------------------------
    // Sets the trailer method.
    //----------------------------------------------------------------
    void setMethod(unsigned long method);

    //----------------------------------------------------------------
    // Returns the trailer data.
    //----------------------------------------------------------------
    const void* getData(unsigned long& length) const;

    //----------------------------------------------------------------
    // Sets the trailer data.
    //----------------------------------------------------------------
    void setData(const void* data, unsigned long length);


protected:
    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplTrailer* m_impl;

private:
    explicit RomaTrailer(RomaImplTrailer*);

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaTrailerListIterator class.
//
// Defines an iterator for the persistent trailers list.
//--------------------------------------------------------------------
class KXCODLL RomaTrailerListIterator
{
public:
    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaTrailerListIterator();

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaTrailerListIterator(const RomaTrailerListIterator&);
    RomaTrailerListIterator& operator=(const RomaTrailerListIterator&);

    //----------------------------------------------------------------
    // Assigns the next route in the list to the specified route.
    //----------------------------------------------------------------
    bool getNext(RomaTrailer&) const;

private:
    //----------------------------------------------------------------
    // Constructor.
    //----------------------------------------------------------------
    RomaTrailerListIterator(RomaImplTrailerList&);

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplTrailerListIter* m_impl;

    friend class RomaTrailerList;
};


//--------------------------------------------------------------------
// The RomaTrailerList class.
//
// Defines a list of persistent trailers in the theme information.
//--------------------------------------------------------------------
class KXCODLL RomaTrailerList
{
public:
    //----------------------------------------------------------------
    // Constructor.
    //----------------------------------------------------------------
    RomaTrailerList();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaTrailerList();

    //----------------------------------------------------------------
    // Shallow copy and assignment.
    //----------------------------------------------------------------
    RomaTrailerList(const RomaTrailerList&);
    RomaTrailerList& operator=(const RomaTrailerList&);

    //----------------------------------------------------------------
    // Deep copy.
    //----------------------------------------------------------------
    RomaTrailerList clone() const;

    //----------------------------------------------------------------
    // Returns true if the list is empty.
    //----------------------------------------------------------------
    bool isEmpty() const;

    //----------------------------------------------------------------
    // Returns number of trailers in the list.
    //----------------------------------------------------------------
    unsigned int getCount() const;

    //----------------------------------------------------------------
    // Removes all trailers from the list.
    //----------------------------------------------------------------
    void clear();

    //----------------------------------------------------------------
    // Assigns the trailer at the specified index to the specified
    // trailer.
    //----------------------------------------------------------------
    bool getAt(RomaTrailer& trailer, unsigned int index) const;

    //----------------------------------------------------------------
    // Adds the specified trailer to the end of the list.
    //----------------------------------------------------------------
    void add(const RomaTrailer& trailer);

    //----------------------------------------------------------------
    // Inserts the specified trailer into the list at the specified
    // index.
    //----------------------------------------------------------------
    bool insert(const RomaTrailer& trailer, unsigned int index = 0);

    //----------------------------------------------------------------
    // Removes the specified trailer from the list.
    //----------------------------------------------------------------
    bool remove(const RomaTrailer& trailer);

    //----------------------------------------------------------------
    // Removes the trailer at the specified index from the list.
    //----------------------------------------------------------------
    bool remove(unsigned int index);

    //----------------------------------------------------------------
    // Returns the list iterator.
    //----------------------------------------------------------------
    RomaTrailerListIterator getIterator() const;


protected:
    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplTrailerList* m_impl;

private:
    explicit RomaTrailerList(RomaImplTrailerList*);

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaConfig class.
//
// Defines a Roma configuration of the application.
//--------------------------------------------------------------------
class KXCODLL RomaConfig
{
public:
    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaConfig();

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaConfig(const RomaConfig&);
    RomaConfig& operator=(const RomaConfig&);

    //----------------------------------------------------------------
    // Returns the business element type.
    //----------------------------------------------------------------
    Roma::AppType getElementType() const;

    //----------------------------------------------------------------
    // Returns the business element name.
    //----------------------------------------------------------------
    const char* getElementName() const;

    //----------------------------------------------------------------
    // Returns the business component name.
    //----------------------------------------------------------------
    const char* getComponentName() const;

    //----------------------------------------------------------------
    // Returns the queue name.
    //----------------------------------------------------------------
    const char* getQueueName() const;

    //----------------------------------------------------------------
    // Returns the business service name.
    //----------------------------------------------------------------
    const char* getServiceName() const;

    //----------------------------------------------------------------
    // Returns the node name.
    //----------------------------------------------------------------
    const char* getNodeName() const;


protected:
    //----------------------------------------------------------------
    // Constructors.
    //----------------------------------------------------------------
    RomaConfig();
    explicit RomaConfig(RomaImplRomaConfig*);

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplRomaConfig* m_impl;

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaSystemConfig class.
//
// Defines a system configuration of the application.
//--------------------------------------------------------------------
class KXCODLL RomaSystemConfig
{
public:
    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaSystemConfig();

    //----------------------------------------------------------------
    // Copy and assignment.
    //----------------------------------------------------------------
    RomaSystemConfig(const RomaSystemConfig&);
    RomaSystemConfig& operator = (const RomaSystemConfig&);

    //----------------------------------------------------------------
    // Returns the environment type.
    //----------------------------------------------------------------
    Roma::EnvType getEnvType() const;

    //----------------------------------------------------------------
    // Returns the operating system type.
    //----------------------------------------------------------------
    Roma::EnvOS getEnvOS() const;

    //----------------------------------------------------------------
    // Returns the domain name.
    //----------------------------------------------------------------
    const char* getDomainName() const;

    //----------------------------------------------------------------
    // Returns the program name.
    //----------------------------------------------------------------
    const char* getProgramName() const;

    //----------------------------------------------------------------
    // Returns the user ID.
    //----------------------------------------------------------------
    const char* getUserId() const;

    //----------------------------------------------------------------
    // Returns the thread ID.
    //----------------------------------------------------------------
    unsigned long getThreadId() const;

    //----------------------------------------------------------------
    // Returns the process ID.
    //----------------------------------------------------------------
    unsigned long getProcessId() const;


protected:
    //----------------------------------------------------------------
    // Constructors.
    //----------------------------------------------------------------
    RomaSystemConfig();
    explicit RomaSystemConfig(RomaImplSysConfig*);

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplSysConfig* m_impl;

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaWorkflowService class.
//
// Defines workflow service information.
//--------------------------------------------------------------------
class KXCODLL RomaWorkflowService
{
public:
    //----------------------------------------------------------------
    // Constructor.
    //----------------------------------------------------------------
    RomaWorkflowService();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaWorkflowService();

    //----------------------------------------------------------------
    // Shallow copy and assignment.
    //----------------------------------------------------------------
    RomaWorkflowService(const RomaWorkflowService&);
    RomaWorkflowService& operator = (const RomaWorkflowService&);

    //----------------------------------------------------------------
    // Deep copy.
    //----------------------------------------------------------------
    RomaWorkflowService clone() const;

    //----------------------------------------------------------------
    // Returns the current business service name.
    //----------------------------------------------------------------
    const char* getService() const;

    //----------------------------------------------------------------
    // Sets the current business service name.
    //----------------------------------------------------------------
    void setService(const char*);

    //----------------------------------------------------------------
    // Returns the current business component name.
    //----------------------------------------------------------------
    const char* getComponent() const;

    //----------------------------------------------------------------
    // Sets the current business component name.
    //----------------------------------------------------------------
    void setComponent(const char*);

    //----------------------------------------------------------------
    // Returns the audit name.
    //----------------------------------------------------------------
    const char* getAudit() const;

    //----------------------------------------------------------------
    // Sets the audit name.
    //----------------------------------------------------------------
    void setAudit(const char*);

    //----------------------------------------------------------------
    // Returns the source business service name.
    //----------------------------------------------------------------
    const char* getSourceService() const;

    //----------------------------------------------------------------
    // Returns the source business flow name.
    //----------------------------------------------------------------
    const char* getSourceFlow() const;

    //----------------------------------------------------------------
    // Sets the source business flow name.
    //----------------------------------------------------------------
    void setSourceFlow(const char*);

    //----------------------------------------------------------------
    // Returns the source business operation name.
    //----------------------------------------------------------------
    const char* getSourceOperation() const;

    //----------------------------------------------------------------
    // Sets the source business operation name.
    //----------------------------------------------------------------
    void setSourceOperation(const char*);


protected:
    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplWfService* m_impl;


private:
    explicit RomaWorkflowService(RomaImplWfService*);

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaWorkflow class.
//
// Defines theme workflow information.
//--------------------------------------------------------------------
class KXCODLL RomaWorkflow
{
public:
    //----------------------------------------------------------------
    // Constructor.
    //----------------------------------------------------------------
    RomaWorkflow();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaWorkflow();

    //----------------------------------------------------------------
    // Shallow copy and assignment.
    //----------------------------------------------------------------
    RomaWorkflow(const RomaWorkflow&);
    RomaWorkflow& operator=(const RomaWorkflow&);

    //----------------------------------------------------------------
    // Deep copy.
    //----------------------------------------------------------------
    RomaWorkflow clone() const;

    //----------------------------------------------------------------
    // Returns the workflow service information.
    //----------------------------------------------------------------
    RomaWorkflowService getWorkflowService() const;

    //----------------------------------------------------------------
    // Sets the workflow service information.
    //----------------------------------------------------------------
    void setWorkflowService(const RomaWorkflowService&);

    //----------------------------------------------------------------
    // Returns the list of the workflow routes.
    //----------------------------------------------------------------
    RomaRouteList getRoutes() const;

    //----------------------------------------------------------------
    // Sets the list of the workflow routes.
    //----------------------------------------------------------------
    void setRoutes(const RomaRouteList&);

private:
    explicit RomaWorkflow(RomaImplWorkflow*);

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplWorkflow* m_impl;

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaEteInfo class.
//
// Defines the end-to-end and in-flight information of the message.
// Currently, this information is private.
//--------------------------------------------------------------------
class KXCODLL RomaEteInfo
{
public:
    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaEteInfo();

    //----------------------------------------------------------------
    // Shallow copy and assignment.
    //----------------------------------------------------------------
    RomaEteInfo(const RomaEteInfo&);
    RomaEteInfo& operator = (const RomaEteInfo&);


protected:
    explicit RomaEteInfo(RomaImplEteInfo*);

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplEteInfo* m_impl;

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaExcErrorData class.
//
// Defines error data of the exception message.
//--------------------------------------------------------------------
class KXCODLL RomaExcErrorData
{
public:
    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaExcErrorData();

    //----------------------------------------------------------------
    // Shallow copy and assignment.
    //----------------------------------------------------------------
    RomaExcErrorData(const RomaExcErrorData&);
    RomaExcErrorData& operator=(const RomaExcErrorData&);

    //----------------------------------------------------------------
    // Returns error information.
    //----------------------------------------------------------------
    const RomaError& getError() const;

    //----------------------------------------------------------------
    // Returns the call type caused the error.
    //----------------------------------------------------------------
    unsigned long getCallType() const;

    //----------------------------------------------------------------
    // Returns the queue name that was in use when the error occurred.
    //----------------------------------------------------------------
    const char* getQueueName() const;


protected:
    //----------------------------------------------------------------
    // Constructor.
    //----------------------------------------------------------------
    RomaExcErrorData();

private:
    explicit RomaExcErrorData(RomaImplExcErrorData*);

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplExcErrorData* m_impl;

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaExceptionInfo class.
//
//--------------------------------------------------------------------
class KXCODLL RomaExceptionInfo
{
public:
    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaExceptionInfo();

    //----------------------------------------------------------------
    // Shallow copy and assignment.
    //----------------------------------------------------------------
    RomaExceptionInfo(const RomaExceptionInfo&);
    RomaExceptionInfo& operator = (const RomaExceptionInfo&);

    //----------------------------------------------------------------
    // Returns true if timestamp information present.
    //----------------------------------------------------------------
    bool hasTimestamp() const;

    //----------------------------------------------------------------
    // Returns true if Roma configuration information present.
    //----------------------------------------------------------------
    bool hasRomaConfig() const;

    //----------------------------------------------------------------
    // Returns true if system configuration information present.
    //----------------------------------------------------------------
    bool hasSystemConfig() const;

    //----------------------------------------------------------------
    // Returns true if error data present.
    //----------------------------------------------------------------
    bool hasErrorData() const;

    //----------------------------------------------------------------
    // Returns true if user data present.
    //----------------------------------------------------------------
    bool hasUserData() const;

    //----------------------------------------------------------------
    // Returns exception timestamp.
    //----------------------------------------------------------------
    RomaTimestamp getTimestamp() const;

    //----------------------------------------------------------------
    // Returns Roma configuration information.
    //----------------------------------------------------------------
    RomaConfig getRomaConfig() const;

    //----------------------------------------------------------------
    // Returns system configuration information.
    //----------------------------------------------------------------
    RomaSystemConfig getSystemConfig() const;

    //----------------------------------------------------------------
    // Returns exception error data.
    //----------------------------------------------------------------
    RomaExcErrorData getErrorData() const;

    //----------------------------------------------------------------
    // Returns exception user data.
    //----------------------------------------------------------------
    const void* getUserData(unsigned long& length) const;


protected:
    //----------------------------------------------------------------
    // Constructor.
    //----------------------------------------------------------------
    RomaExceptionInfo();

private:
    explicit RomaExceptionInfo(RomaImplExcInfo*);

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplExcInfo* m_impl;

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaMessageInfo class.
//
//--------------------------------------------------------------------
class KXCODLL RomaMessageInfo
{
public:
    //----------------------------------------------------------------
    // Constructor.
    //----------------------------------------------------------------
    RomaMessageInfo();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaMessageInfo();

    //----------------------------------------------------------------
    // Shallow copy and assignment.
    //----------------------------------------------------------------
    RomaMessageInfo(const RomaMessageInfo&);
    RomaMessageInfo& operator = (const RomaMessageInfo&);

    //----------------------------------------------------------------
    // Returns true if the message theme present.
    //----------------------------------------------------------------
    bool hasTheme() const;

    //----------------------------------------------------------------
    // Returns true if the original message present.
    //----------------------------------------------------------------
    bool hasMessage() const;

    //----------------------------------------------------------------
    // Returns true if workflow information present.
    //----------------------------------------------------------------
    bool hasWorkflow() const;

    //----------------------------------------------------------------
    // Returns true if exception information present.
    //----------------------------------------------------------------
    bool hasException() const;

    //----------------------------------------------------------------
    // Returns workflow information.
    //----------------------------------------------------------------
    RomaWorkflow getWorkflow() const;

    //----------------------------------------------------------------
    // Returns the message theme.
    //----------------------------------------------------------------
    RomaTheme getTheme() const;

    //----------------------------------------------------------------
    // Returns exception information.
    //----------------------------------------------------------------
    RomaExceptionInfo getException() const;

    //----------------------------------------------------------------
    // Returns the original message as a business element request.
    //----------------------------------------------------------------
    RomaBeRequest getBeRequest() const;

    //----------------------------------------------------------------
    // Returns the original message as a client response.
    //----------------------------------------------------------------
    RomaClientResponse getClientResponse() const;


private:
    explicit RomaMessageInfo(RomaImplMsgInfo*);

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplMsgInfo* m_impl;

    friend class RomaImplProxy;
};


//--------------------------------------------------------------------
// The RomaThemeInfo class.
//
// Defines detailed theme information. This information can be
// extracted with the queryTheme() method or built into a theme with
// the buildTheme() method of RomaClient and RomaBusinessElement.
//--------------------------------------------------------------------
class KXCODLL RomaThemeInfo
{
public:
    //----------------------------------------------------------------
    // Constructor.
    //----------------------------------------------------------------
    RomaThemeInfo();

    //----------------------------------------------------------------
    // Destructor.
    //----------------------------------------------------------------
    virtual ~RomaThemeInfo();

    //----------------------------------------------------------------
    // Shallow copy and assignment.
    //----------------------------------------------------------------
    RomaThemeInfo(const RomaThemeInfo&);
    RomaThemeInfo& operator = (const RomaThemeInfo&);

    //----------------------------------------------------------------
    // Deep copy.
    //----------------------------------------------------------------
    RomaThemeInfo clone() const;

    //----------------------------------------------------------------
    // Returns workflow information.
    //----------------------------------------------------------------
    RomaWorkflow getWorkflow() const;

    //----------------------------------------------------------------
    // Sets workflow information.
    //----------------------------------------------------------------
    void setWorkflow(const RomaWorkflow& wflow);

    //----------------------------------------------------------------
    // Returns the retry count.
    //----------------------------------------------------------------
    unsigned long getRetryCount() const;

    //----------------------------------------------------------------
    // Sets the retry count.
    //----------------------------------------------------------------
    void setRetryCount(unsigned long count);

    //----------------------------------------------------------------
    // Returns the theme ID.
    //----------------------------------------------------------------
    RomaMsgId getThemeId() const;

    //----------------------------------------------------------------
    // Sets the theme ID.
    //----------------------------------------------------------------
    void setThemeId(const RomaMsgId& themeId);

    //----------------------------------------------------------------
    // Returns the confirmation flags.
    //----------------------------------------------------------------
    Roma::ConfirmFlags getConfirmFlags() const;

    //----------------------------------------------------------------
    // Sets the confirmation flags.
    //----------------------------------------------------------------
    void setConfirmFlags(Roma::ConfirmFlags flags);

    //----------------------------------------------------------------
    // Returns the list of persistent trailers.
    //----------------------------------------------------------------
    RomaTrailerList getTrailers() const;

    //----------------------------------------------------------------
    // Sets the list of persistent trailers.
    //----------------------------------------------------------------
    void setTrailers(const RomaTrailerList& list);

    //----------------------------------------------------------------
    // Returns ETE information.
    //----------------------------------------------------------------
    RomaEteInfo getEteInfo() const;

    //----------------------------------------------------------------
    // Sets ETE information.
    //----------------------------------------------------------------
    void setEteInfo(const RomaEteInfo&);


private:
    explicit RomaThemeInfo(RomaImplThemeInfo*);

    //----------------------------------------------------------------
    // Pointer to the internal implementation.
    //----------------------------------------------------------------
    RomaImplThemeInfo* m_impl;

    friend class RomaImplProxy;
};


#endif // __KXCOEMS_HPP

