/*********************************************************************
 * $Id: kxmmsg.hpp,v 1.76 2003/12/09 13:30:11 e_ab Exp $
 * $Source: /romacvs/root/roma/src/kxm/kxmcpphf/kxmmsg.hpp,v $
 *********************************************************************
 * Description: KXM C++ Helper Functions external definitions
 *********************************************************************
 *                                                                    
 * Copyright 1999-2003, Candle Corporation, a California                   
 * corporation.  All rights reserved.                                 
 * International copyright secured.                                   
 *                                                                    
 * This material is proprietary to Candle Corporation and is not to   
 * be reproduced, used, or disclosed except in accordance with        
 * program licenses or upon written authorization of Candle           
 * Corporation.                                                       
 *                                                                    
 ********************************************************************/

#ifndef __KXMMSG_HPP
#define __KXMMSG_HPP

#include <kxmenv.h>
#include <kxmchf.h>
#include <kxmbuff.hpp>
#include <kxmerr.hpp>
#include <stdlib.h>
#include <kxmstr.hpp>

// Implementation classes.
class KxmMhFieldIface;
class KxmMhMsgImpl;
class KxmMhFormatImpl;
class KxmMhConvertOptImpl;
class KxmMhCreateOptImpl;
class KxmMhWriteOptImpl;
class KxmMhQNameImpl;
class KxmMhTimeZoneImpl;
class KxmMhDurationImpl;
class KxmMhDateImpl;
class KxmDcObjectRef;
class KxmUtRange;

//--------------------------------------------------------------------
// Time zone information.
//--------------------------------------------------------------------
class KXMCPPHFAPI KxmTimeZone
{
public:
    enum Type
    {
        TZ_UNKNOWN,
        TZ_UTC,
        TZ_POS,
        TZ_NEG
    };

    // It is error to specify TZ_UNKNOWN and not 0 hours or minutes.
    // If TZ_POS or TZ_NEG is specified but hours and minutes are 0
    // then type will be changed to TZ_UTC.
    KxmTimeZone( Type = TZ_UNKNOWN, 
                 unsigned hours = 0, 
                 unsigned minutes = 0 );

    virtual ~KxmTimeZone();

    KxmTimeZone( const KxmTimeZone& );
    KxmTimeZone& operator = ( const KxmTimeZone& );

    Type getType() const;
    void setType( Type );

    unsigned getHours() const;
    void setHours( unsigned );

    unsigned getMinutes() const;
    void setMinutes( unsigned );

private:
    KxmMhTimeZoneImpl* m_impl;
};

//--------------------------------------------------------------------
// Qualified XML name data type.
//--------------------------------------------------------------------
class KXMCPPHFAPI KxmQName
{
public:
    KxmQName();
	KxmQName( const KxmStr& name, const KxmStr& uri );
    KxmQName( const KxmStr& prefix, 
              const KxmStr& localPart, 
              const KxmStr& uri );
    virtual ~KxmQName();

    KxmQName( const KxmQName& );
    KxmQName& operator = ( const KxmQName& );

	KxmStr getNsUri() const;
	void setNsUri( const KxmStr& );

	KxmStr getLocalPart() const;
	void setLocalPart( const KxmStr& );

	KxmStr getNsPrefix() const;
	void setNsPrefix( const KxmStr& );

	KxmStr getName() const;
	void setName( const KxmStr& );

private:
    KxmMhQNameImpl* m_impl;
};

//--------------------------------------------------------------------
// Time duration data type.
//--------------------------------------------------------------------
class KXMCPPHFAPI KxmDuration
{
public:
    KxmDuration(); // initializes everything to 0
    virtual ~KxmDuration();

    KxmDuration( const KxmDuration& );
    KxmDuration& operator = ( const KxmDuration& );

    bool isNegative() const;
    void setNegative( bool );

    unsigned int getYears() const;
    void setYears( unsigned int );

    unsigned int getMonths() const;
    void setMonths( unsigned int );

    unsigned int getDays() const;
    void setDays( unsigned int );

    unsigned int getHours() const;
    void setHours( unsigned int );

    unsigned int getMinutes() const;
    void setMinutes( unsigned int );

    unsigned int getSeconds() const;
    void setSeconds( unsigned int );

    // Milliseconds value should be less than 1000.
    // (milliseconds == nanoseconds / 1000000)
    // (milliseconds == microseconds / 1000)

    unsigned int getMilliseconds() const;
    void setMilliseconds( unsigned int );

    // Microseconds value should be less than 1000000.
    // (microseconds == nanoseconds / 1000)
    // (microseconds == milliseconds * 1000)

    unsigned int getMicroseconds() const;
    void setMicroseconds( unsigned int );

private:
    KxmMhDurationImpl* m_impl;
};

//--------------------------------------------------------------------
// Range of integer values.
// It is used in the KxmField interface.
//--------------------------------------------------------------------
class KXMCPPHFAPI KxmRange
{
public:
    enum Unlimited
    {
        UNLIMITED = (size_t)-1
    };

    KxmRange();
    KxmRange( size_t minVal, size_t maxVal );

    // Compiler generated:
    // ~KxmRange();
    // KxmRange( const KxmRange& );
    // KxmRange& operator = ( const KxmRange& );

    size_t getMin() const;
    size_t getMax() const;

    bool operator == ( const KxmRange& r );
    bool operator != ( const KxmRange& r );

private:
    friend class KxmField;

    KxmRange(const KxmUtRange&);
    operator KxmUtRange() const;

    size_t m_min;
    size_t m_max;
};


#if defined(BUILD_KXMCPPHF) || defined(BUILD_KXMXHF) 

//--------------------------------------------------------------------
// Date and time.
// Backward compatibility version.
//--------------------------------------------------------------------
class KXMCPPHFAPI KxmDate
{
public:
    enum Status
    {
        NONE = KXMDATE_STATUS_NONE,
        DATE = KXMDATE_STATUS_DATE,
        TIME = KXMDATE_STATUS_TIME,
        DATETIME = KXMDATE_STATUS_DATETIME
    };

    KxmDate();
    KxmDate( int year, int month, int day );
    KxmDate( int hours, int minutes, int seconds, int msecs );
    KxmDate( int year, int month, int day,
        int hours, int minutes, int seconds, int msecs = 0 );

    // Generated by compiler:
    //   KxmDate( const KxmDate& );
    //   KxmDate& operator = ( const KxmDate& );
    //   ~KxmDate()

    Status getStatus() const;
    bool isDate() const;
    bool isTime() const;
    bool isDateTime() const;

    int getYear() const;
    int getMonth() const;
    int getDay() const;
    int getHours() const;
    int getMinutes() const;
    int getSeconds() const;
    int getMsecs() const;

    void set( int year, int month, int day,
        int hours, int minutes, int seconds, int msecs = 0);

    void setDate( int year, int month, int day );

    void setTime( int hour, int minutes, int seconds, int msecs = 0 );

private:
    int m_status;
    int m_year;
    int m_month;
    int m_day;
    int m_hours;
    int m_minutes;
    int m_seconds;
    int m_millisecs;
};

#endif

//--------------------------------------------------------------------
// Date and time.
// CASP 2.0 version.
//--------------------------------------------------------------------

class KXMCPPHFAPI KxmDateEx
{
public:
    enum Status
    {
        NONE = KXMDATE_STATUS_NONE,
        DATE = KXMDATE_STATUS_DATE,
        TIME = KXMDATE_STATUS_TIME,
        DATETIME = KXMDATE_STATUS_DATETIME,
        GYEAR = KXMDATE_STATUS_GYEAR,
	    GYEARMONTH = KXMDATE_STATUS_GYEARMONTH,
	    GMONTHDAY = KXMDATE_STATUS_GMONTHDAY,
	    GDAY = KXMDATE_STATUS_GDAY,
	    GMONTH = KXMDATE_STATUS_GMONTH
    };

    KxmDateEx();

    KxmDateEx( const KxmDateEx& );
    KxmDateEx& operator = ( const KxmDateEx& );

    virtual ~KxmDateEx();

    Status getStatus() const;
    void setStatus( Status );

    int getYear() const;
    void setYear( int );

    int getMonth() const;
    void setMonth( int );

    int getDay() const;
    void setDay( int );

    int getHours() const;
    void setHours( int );

    int getMinutes() const;
    void setMinutes( int );

    int getSeconds() const;
    void setSeconds( int );

    // Milliseconds value should be 0 <= x < 1000.
    // (milliseconds == nanoseconds / 1000000)
    // (milliseconds == microseconds / 1000)

    int getMilliseconds() const;
    void setMilliseconds( int );

    // Microseconds value should be 0 <= x < 1000000.
    // (microseconds == nanoseconds / 1000)
    // (microseconds == milliseconds * 1000)

    int getMicroseconds() const;
    void setMicroseconds( int );

    const KxmTimeZone& getTimeZone() const;
    KxmTimeZone& getTimeZone();
    void setTimeZone( const KxmTimeZone& );
                
    // for backward compatibility

    KxmDateEx( int year, int month, int day );
    KxmDateEx( int hours, int minutes, int seconds, int msecs );
    KxmDateEx( int year, int month, int day,
        int hours, int minutes, int seconds, int msecs = 0 );

    void set( int year, int month, int day,
        int hours, int minutes, int seconds, int msecs = 0);

    void setDate( int year, int month, int day );

    void setTime( int hour, int minutes, int seconds, int msecs = 0 );
    
    int getMsecs() const;

    bool isDate() const;
    bool isTime() const;
    bool isDateTime() const;

private:
    KxmMhDateImpl* m_impl;

    friend class KxmCppHfPackage;
};

#if !defined(BUILD_KXMCPPHF) && !defined(BUILD_KXMXHF) 
typedef KxmDateEx KxmDate;
#endif

//--------------------------------------------------------------------
// Field format.
//--------------------------------------------------------------------
class KXMCPPHFAPI KxmFieldFormat
{
public:
    enum Format
    {
        NONE = 0,
        BINARY_BASE64 = 1,
        BINARY_HEX = 2
    };

    KxmFieldFormat();
    KxmFieldFormat( Format );
    KxmFieldFormat( const KxmFieldFormat& );

    ~KxmFieldFormat();

    KxmFieldFormat& operator = ( const KxmFieldFormat& );

    Format getFormat() const;

private:
    KxmMhFormatImpl* m_impl;
};

//--------------------------------------------------------------------
// Access to the field information and value.
//--------------------------------------------------------------------
class KXMCPPHFAPI KxmField
{
public:
    // Field type.
    enum Type
    {
        TYPE_NONE = KXMFIELDTYPE_NONE,
        TYPE_STRING = KXMFIELDTYPE_STRING,
        TYPE_BOOL = KXMFIELDTYPE_BOOL,
        TYPE_NUMBER = KXMFIELDTYPE_NUMBER,
        TYPE_BINARY = KXMFIELDTYPE_BINARY,
        TYPE_DATE = KXMFIELDTYPE_DATE,
        TYPE_DURATION = KXMFIELDTYPE_DURATION,
        TYPE_QNAME = KXMFIELDTYPE_QNAME,
        TYPE_GROUP = KXMFIELDTYPE_GROUP,
        TYPE_XMLELEM = KXMFIELDTYPE_XMLELEM,
        TYPE_XMLATTR = KXMFIELDTYPE_XMLATTR
    };

    // Number presentation.
    enum NumericType
    {
        NTYPE_NONE = KXMNUMTYPE_NONE,
        NTYPE_DECIMAL = KXMNUMTYPE_DECIMAL,
#ifndef KXM_NO_IEEE_FLOAT
        NTYPE_DOUBLE = KXMNUMTYPE_DOUBLE,
        NTYPE_FLOAT = KXMNUMTYPE_FLOAT,
#endif
        NTYPE_INTEGER = KXMNUMTYPE_INTEGER,
        NTYPE_INT8 = KXMNUMTYPE_INT8,
        NTYPE_UINT8 = KXMNUMTYPE_UINT8,
        NTYPE_INT16 = KXMNUMTYPE_INT16,
        NTYPE_UINT16 = KXMNUMTYPE_UINT16,
#ifndef KXM_NO_INT64
        NTYPE_INT64 = KXMNUMTYPE_INT64,
        NTYPE_UINT64 = KXMNUMTYPE_UINT64,
#endif
        NTYPE_INT32 = KXMNUMTYPE_INT32,
        NTYPE_UINT32 = KXMNUMTYPE_UINT32
    };

    // Date/time type.
    enum DateType
    {
        DTYPE_NONE = KXMDATETYPE_NONE,
        DTYPE_TIME = KXMDATETYPE_TIME,
        DTYPE_DATE = KXMDATETYPE_DATE,
        DTYPE_DATETIME = KXMDATETYPE_DATETIME,
        DTYPE_GYEAR = KXMDATETYPE_GYEAR,
        DTYPE_GYEARMONTH = KXMDATETYPE_GYEARMONTH,
        DTYPE_GMONTHDAY = KXMDATETYPE_GMONTHDAY,
        DTYPE_GDAY = KXMDATETYPE_GDAY,
        DTYPE_GMONTH = KXMDATETYPE_GMONTH
    };

    // Search options.
    enum SearchOpt
    {
        EXACT,
        SEARCH,
        UNIQUE
    };

    // Feature options.
    enum FeatureOpt
    {
        GET_SCHEMA_FIELDS   = 1
    };

    // Field flags
    enum FieldFlags
    {
        FLAG_ROOT          = KXMFIELDFLAG_IS_ROOT,
        FLAG_ARRAY         = KXMFIELDFLAG_IS_ARRAY,
        FLAG_GROUP         = KXMFIELDFLAG_IS_GROUP,
        FLAG_DEFAULT_VALUE = KXMFIELDFLAG_IS_DEFAULT_VALUE,
        FLAG_UNION         = KXMFIELDFLAG_IS_UNION,
        FLAG_XMLELEM       = KXMFIELDFLAG_IS_XMLELEM,
        FLAG_XMLATTR       = KXMFIELDFLAG_IS_XMLATTR,
        FLAG_XMLDOC        = KXMFIELDFLAG_IS_XMLDOC
    };

    // Default constructor, creates KxmField object
    // not attached to any field.
    KxmField();

    ~KxmField();

    // Makes the object refering to the specified
    // field, doesn't assign or copy value of the field itself
    KxmField( const KxmField& );
    KxmField& operator = ( const KxmField& );

    // Information methods.

    KxmStr getName() const KXMTHROW( KxmError );

    Type getType() const KXMTHROW( KxmError );

    NumericType getNumericType() const KXMTHROW( KxmError );

    DateType getDateType() const KXMTHROW( KxmError );

    // Minimum and maximum length of the string or binary field.
    KxmRange getValueLenRange() const KXMTHROW( KxmError );

    // Minimum and maximum number of elements in array.
    KxmRange getArrayLenRange() const KXMTHROW( KxmError );

    unsigned int getFlags() const KXMTHROW( KxmError );

    // State methods
    
    // No field is reffered by this object.
    // The field object has this state just after
    // construction by the default constructor.
    bool isInvalid() const;

    // Returns true if field doesn't contain value.
    // It is not the same as isInvalid(), which returns true
    // when KxmField object does not refer any field (not attached).
    bool isEmpty() const KXMTHROW( KxmError );

    // Emties the field.
    // For example, deletes underlying XML element.
    void setEmpty() const KXMTHROW( KxmError );

    // Misc methods

    // Gets outer group, returns isInvalid() == true KxmField object if 
    // field does not have parent.
    KxmField getParent() const KXMTHROW( KxmError );

    // Gets default value of a field, returns isInvalid() == true
    // KxmField object if field does not have default value.
    KxmField getDefault() const KXMTHROW( KxmError );

    // Group methods

    // Returns number of sub-fields.
    // This number is changed for the XML message as you set
    // or empty fields.
    size_t getNumOfFields() const KXMTHROW( KxmError );

    // The same method as above, but it allows to specify
    // the feature options (see FeatureOpt enumeration).
    size_t getNumOfFields( unsigned options ) const KXMTHROW( KxmError );

    // Throws error if field is not found.
    KxmField getField( const KxmStr& fieldName, SearchOpt opt = EXACT )
        const KXMTHROW( KxmError );

    // Returns isInvalid() == true KxmField object if field
    // is not found.
    KxmField findField( const KxmStr&, SearchOpt opt = EXACT )
        const KXMTHROW( KxmError );

    // Gets field by index (first field has index 0), use
    // getNumOfFields() to get number of fields.
    KxmField getField( size_t index ) const KXMTHROW( KxmError );

    // The same method as above, but it allows to specify
    // the feature options (see FeatureOpt enumeration).
    KxmField getField( size_t index, 
                       unsigned options ) const KXMTHROW( KxmError );

    // Set/get methods.
    // If set/get method does not match the field type then conversion
    // is done. If conversion is not possible then error is raised.
    // If field is an array then these methods set/get the first
    // element of the array.

    // Set methods.

    void set( const KxmStr& ) const KXMTHROW( KxmError );

    void set( int ) const KXMTHROW( KxmError );

    void set( unsigned int ) const KXMTHROW( KxmError );

    void set( long ) const KXMTHROW( KxmError );

    void set( unsigned long ) const KXMTHROW( KxmError );

#if !defined(KXM_NO_INT64) && !defined(KXM_64BIT_LONG)

    void set( KXMINT64 ) const KXMTHROW( KxmError );

    void set( KXMUINT64 ) const KXMTHROW( KxmError );

#endif

#ifndef KXM_NO_IEEE_FLOAT

    void set( double ) const KXMTHROW( KxmError );

#endif

    void set( const KxmDate& ) const KXMTHROW( KxmError );

#if defined(BUILD_KXMCPPHF) || defined(BUILD_KXMXHF) 
    void set( const KxmDateEx& ) const KXMTHROW( KxmError );
#endif

    void set( const void* data, size_t length ) const
        KXMTHROW( KxmError );

    void set( const KxmDataBuffer& buf ) const KXMTHROW( KxmError );

    void set( const KxmQName& ) const KXMTHROW( KxmError );

    void set( const KxmDuration& ) const KXMTHROW( KxmError );

    // Field to field assignment
    void set( const KxmField& ) const KXMTHROW(KxmError);

    void setBool( bool ) const KXMTHROW(KxmError);

    // Get methods.

    void get( KxmStr& ) const KXMTHROW( KxmError );

    void get( int& ) const KXMTHROW( KxmError );

    void get( unsigned int& ) const KXMTHROW( KxmError );

    void get( long& ) const KXMTHROW( KxmError );

    void get( unsigned long& ) const KXMTHROW( KxmError );

#if !defined(KXM_NO_INT64) && !defined(KXM_64BIT_LONG)

    void get( KXMINT64& ) const KXMTHROW( KxmError );

    void get( KXMUINT64& ) const KXMTHROW( KxmError );

#endif

#ifndef KXM_NO_IEEE_FLOAT

    void get( double& ) const KXMTHROW( KxmError );

#endif

    void get( KxmDate& ) const KXMTHROW( KxmError );

#if defined(BUILD_KXMCPPHF) || defined(BUILD_KXMXHF) 
    void get( KxmDateEx& ) const KXMTHROW( KxmError );
#endif

    void get( KxmDataBuffer& buf ) const KXMTHROW( KxmError );

    void get( KxmQName& ) const KXMTHROW( KxmError );

    void get( KxmDuration& ) const KXMTHROW( KxmError );

    bool getBool() const KXMTHROW( KxmError );

    // Array methods.

    size_t getArrayLen() const KXMTHROW( KxmError );
    void setArrayLen( size_t ) const KXMTHROW( KxmError );

    KxmField operator [] ( size_t ) const KXMTHROW( KxmError );
    KxmField at( size_t ) const KXMTHROW( KxmError );

    // Apply the specified format to the field.

    KxmField applyFormat( const KxmFieldFormat& format ) const;

    // XML namespaces support.

    void declareNs(const KxmStr& fieldName, 
                   const KxmStr& prefix,
                   const KxmStr& uri) KXMTHROW( KxmError );

    KxmStr getNsPrefix() const KXMTHROW( KxmError );
    KxmStr getNsUri() const KXMTHROW( KxmError );
    KxmStr getLocalPart() const KXMTHROW( KxmError );

    // XML Schema xsi:nil attribute support.
    // Throws an error for an invalid field and a non-XML field.

    void setNil() const KXMTHROW( KxmError );
    bool isNil() const KXMTHROW( KxmError );

    // The methods to set/get character data of an XML field.
    void setCharacterData( const KxmStr& ) KXMTHROW( KxmError );
    KxmStr getCharacterData() const KXMTHROW( KxmError );

    // for backward compatibility, use getFlags() instead.

    bool isArray() const KXMTHROW( KxmError );

    bool isRoot() const KXMTHROW( KxmError );

    bool isGroup() const KXMTHROW( KxmError );

    bool isDefaultValue() const KXMTHROW( KxmError );

private:
    friend class KxmCppHfPackage;

    KxmField(KxmMhFieldIface*);
    void setNumber(const KxmStr&) const;
    KxmStr getNumber() const;
    
protected:
    KxmField(const KxmDcObjectRef&);

    KxmMhFieldIface* m_impl;
};


// KxmMsgCreateOptions class
// providing message creation options
class KXMCPPHFAPI KxmMsgCreateOptions
{
public:
    enum Options
    {
        MDR = KXMMSGOPT_MDR,
        STANDALONE = KXMMSGOPT_STANDALONE,
        ANY = KXMMSGOPT_ANY,
        FORMAT_NAME = KXMMSGOPT_FORMAT_NAME,
        HANDLE_NAMESPACES = KXMMSGOPT_HANDLE_NAMESPACES,
        OLD_TEXT_FORMAT = KXMMSGOPT_OLD_TEXT_FORMAT,
        HEX_XML_FORMAT = KXMMSGOPT_HEX_XML_FORMAT,
        PRESERVE_XML_SPACES = KXMMSGOPT_PRESERVE_XML_SPACES,
        DISABLE_NAMESPACES = KXMMSGOPT_DISABLE_NAMESPACES,
        SCALAR_AS_ARRAY = KXMMSGOPT_SCALAR_AS_ARRAY
    };

    // default constructor
    KxmMsgCreateOptions(unsigned int = 0); 

    // copy constructor
    KxmMsgCreateOptions(const KxmMsgCreateOptions&);

    // destructor
    virtual ~KxmMsgCreateOptions();

    // assignment operator
    KxmMsgCreateOptions& operator = (const KxmMsgCreateOptions&);

    // sets encoding
    void setEncoding(const KxmEncoding&);

    // returns encoding
    KxmEncoding& getEncoding();
    const KxmEncoding& getEncoding() const;

    // sets options
    void setOptions(unsigned int);

    // returns options
    unsigned int getOptions() const; 

    // sets namespace URI for the root element
    void setNsUri(const KxmStr&);

    // returns namespace URI for the root element
    KxmStr getNsUri() const;

private:
    friend class KxmCppHfPackage;

    KxmMhCreateOptImpl* m_impl;
};


// KxmMsgWriteOptions class
// providing message extraction options
class KXMCPPHFAPI KxmMsgWriteOptions
{
public:
    enum Options
    {
        NO_MARGINS = KXMEXTOPT_NO_MARGINS
    };

    // default constructor
    KxmMsgWriteOptions(unsigned int = 0); 

    // copy constructor
    KxmMsgWriteOptions(const KxmMsgWriteOptions&);

    // destructor
    virtual ~KxmMsgWriteOptions();

    // assignment operator
    KxmMsgWriteOptions& operator = (const KxmMsgWriteOptions&);

    // sets encoding
    void setEncoding(const KxmEncoding&);

    // returns encoding
    KxmEncoding& getEncoding();
    const KxmEncoding& getEncoding() const;

    // sets options
    void setOptions(unsigned int);

    // returns options
    unsigned int getOptions() const; 

private:
    friend class KxmCppHfPackage;

    KxmMhWriteOptImpl* m_impl;
};


// KxmMsgConvertOptions class
// providing message conversion options
class KXMCPPHFAPI KxmMsgConvertOptions
{
public:
    enum Options
    {
        TO_XML = KXMCNVOPT_TO_XML,
        TO_FORMAT = KXMCNVOPT_TO_FORMAT,
        BY_MAP = KXMCNVOPT_BY_MAP,
        MAP_REQUIRED = KXMCNVOPT_MAP_REQUIRED,
        HANDLE_NAMESPACES = KXMCNVOPT_HANDLE_NAMESPACES,
        OLD_TEXT_FORMAT = KXMCNVOPT_OLD_TEXT_FORMAT,
        HEX_XML_FORMAT = KXMMSGOPT_HEX_XML_FORMAT,
        DISABLE_NAMESPACES = KXMCNVOPT_DISABLE_NAMESPACES,
        SCALAR_AS_ARRAY = KXMCNVOPT_SCALAR_AS_ARRAY
    };

    // default constructor
    KxmMsgConvertOptions(unsigned int = 0);

    // copy constructor
    KxmMsgConvertOptions(const KxmMsgConvertOptions&);

    // destructor
    virtual ~KxmMsgConvertOptions();

    // assignment operator
    KxmMsgConvertOptions& operator = (const KxmMsgConvertOptions&);

    // sets options
    void setOptions(unsigned int);

    // returns options
    unsigned int getOptions() const; 

    // sets namespace URI for the root element
    void setNsUri(const KxmStr&);

    // returns namespace URI for the root element
    KxmStr getNsUri() const;

private:
    friend class KxmCppHfPackage;

    KxmMhConvertOptImpl* m_impl;
};


//--------------------------------------------------------------------
// Parsed, in-memory message.
//--------------------------------------------------------------------
class KxmMessage : public KxmField
{
public:
    enum Type
    {
        MDR = KXMMSGOPT_MDR,
        STANDALONE = KXMMSGOPT_STANDALONE,
        ANY = KXMMSGOPT_ANY,
        FORMAT_NAME = KXMMSGOPT_FORMAT_NAME
    };

    enum Conversion
    {
        TO_XML = KXMCNVOPT_TO_XML,
        TO_FORMAT = KXMCNVOPT_TO_FORMAT,
        BY_MAP = KXMCNVOPT_BY_MAP
    };


    // Creates an empty message.
    KXMCPPHFAPI KxmMessage(const KxmStr& format, 
                           const KxmMsgCreateOptions& opt) KXMTHROW(KxmError);

    // Parses message.
    KXMCPPHFAPI KxmMessage(const void* data, size_t length,
                           const KxmStr& format, 
                           const KxmMsgCreateOptions& opt) KXMTHROW(KxmError);


    // Creates an empty message. (obsolete)
    KXMCPPHFAPI KxmMessage( const KxmStr& formatName, Type = MDR )
        KXMTHROW( KxmError );

    // Parses message. (obsolete)
    KXMCPPHFAPI KxmMessage( const KxmStr& formatName, 
        const KxmDataBuffer& data, Type t = MDR )
        KXMTHROW( KxmError );

    // Parses message. (obsolete)
    KXMCPPHFAPI KxmMessage( const void* data,
        size_t dataLen,
        const KxmStr& formatName, 
        Type t = MDR,
        const KxmEncoding& enc = KxmEncoding() )
        KXMTHROW( KxmError );

    // Parses message. (obsolete)
    KXMCPPHFAPI KxmMessage( const KxmDataBuffer& data, 
        Type t = STANDALONE ) KXMTHROW( KxmError );


    // Creates empty message object, not linked with any
    // format (isInvalid()==true). 
    KXMCPPHFAPI KxmMessage();

    // Frees object resources.
    KXMCPPHFAPI ~KxmMessage();

    // Makes copy of the message.
    KXMCPPHFAPI KxmMessage( const KxmMessage& );
    KXMEXPORT(KXMCPPHFAPI,KxmMessage&) operator = ( const KxmMessage& );

    // Information methods.
    KXMEXPORT(KXMCPPHFAPI,KxmStr) getFormatName() const
        KXMTHROW( KxmError );
    KXMEXPORT(KXMCPPHFAPI,KxmStr) getXmlDocName() const
        KXMTHROW( KxmError );

    // Converts message to the specified format.
    KXMEXPORT(KXMCPPHFCAPI,void) convert(const KxmStr&,
                              Conversion = TO_FORMAT,
                              KxmMessage* = NULL);

    KXMEXPORT(KXMCPPHFCAPI,void) convert(const KxmStr&,
                              const KxmMsgConvertOptions&,
                              KxmMessage* = NULL);

    // Writes message to the specified buffer.
    KXMEXPORT(KXMCPPHFAPI,void) write( KxmDataBuffer& buf ) const
        KXMTHROW( KxmError );

    KXMEXPORT(KXMCPPHFAPI,void) write( KxmDataBuffer& buf, const KxmEncoding& enc )
        const KXMTHROW( KxmError );

    KXMEXPORT(KXMCPPHFAPI,void) write( KxmDataBuffer& buf, const KxmMsgWriteOptions& opt )
        const KXMTHROW( KxmError );

    KXMEXPORT(KXMCPPHFAPI,const KxmEncoding&) getLastEncoding() const
        KXMTHROW( KxmError );

private:
    friend class KxmCppHfPackage;
    
    KXMCPPHFAPI KxmMessage(const KxmDcObjectRef&);

    void* m_reserved;
};


#endif

